Package: BayesianMCPMod
Title: Simulate, Evaluate, and Analyze Dose Finding Trials with
        Bayesian MCPMod
Version: 1.3.2
Authors@R: c(
    person("Boehringer Ingelheim Pharma GmbH & Co. KG", role = c("cph", "fnd")),
    person("Stephan", "Wojciekowski", , "stephan.wojciekowski@boehringer-ingelheim.com", role = c("aut", "cre")),
    person("Lars", "Andersen", , "lars.andersen@boehringer-ingelheim.com", role = "aut"),
    person("Jonas", "Schick", , "jonas.schick@boehringer-ingelheim.com", role = "ctb"),
    person("Sebastian", "Bossert", , "sebastian.bossert@boehringer-ingelheim.com", role = "aut")
  )
Description: Bayesian MCPMod (Fleischer et al. (2022)
    <doi:10.1002/pst.2193>) is an innovative method that improves the
    traditional MCPMod by systematically incorporating historical data,
    such as previous placebo group data. This package offers functions
    for simulating, analyzing, and evaluating Bayesian MCPMod trials with
    normally and binary distributed endpoints. It enables the assessment of trial
    designs incorporating historical data across various true
    dose-response relationships and sample sizes. Robust mixture prior
    distributions, such as those derived with the Meta-Analytic-Predictive
    approach (Schmidli et al. (2014) <doi:10.1111/biom.12242>), can be
    specified for each dose group.  Resulting mixture posterior
    distributions are used in the Bayesian Multiple Comparison Procedure
    and modeling steps. The modeling step also includes a weighted model
    averaging approach (Pinheiro et al. (2014) <doi:10.1002/sim.6052>).
    Estimated dose-response relationships can be bootstrapped and
    visualized.
License: Apache License (>= 2)
URL: https://boehringer-ingelheim.github.io/BayesianMCPMod/,
        https://github.com/Boehringer-Ingelheim/BayesianMCPMod
BugReports: https://github.com/Boehringer-Ingelheim/BayesianMCPMod/issues
Depends: R (>= 4.2)
Imports: checkmate, DoseFinding (>= 1.1-1), dplyr, ggplot2, logistf,
        methods, nloptr, RBesT, stats, tidyr
Suggests: clinDR, doFuture, future.apply, kableExtra, knitr,
        MCPModPack, reactable, rmarkdown, spelling, testthat (>=
        3.0.0), tibble
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-14 05:46:06 UTC; wojcie
Author: Boehringer Ingelheim Pharma GmbH & Co. KG [cph, fnd],
  Stephan Wojciekowski [aut, cre],
  Lars Andersen [aut],
  Jonas Schick [ctb],
  Sebastian Bossert [aut]
Maintainer: Stephan Wojciekowski <stephan.wojciekowski@boehringer-ingelheim.com>
Repository: CRAN
Date/Publication: 2026-05-14 14:00:19 UTC
Built: R 4.4.3; ; 2026-05-14 14:31:59 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,modelFits)
S3method(predict,modelFits)
S3method(print,BayesianMCP)
S3method(print,BayesianMCPMod)
S3method(print,modelFits)
S3method(print,postList)
S3method(summary,postList)
export(assessDesign)
export(getBootstrapQuantiles)
export(getBootstrapSamples)
export(getContr)
export(getCritProb)
export(getESS)
export(getMED)
export(getModelFits)
export(getPosterior)
export(performBayesianMCP)
export(performBayesianMCPMod)
export(simulateData)

<!-- badges: start -->
[![Linux](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/linux.yml/badge.svg?branch=main)](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/linux.yml)
[![Windows](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/windows.yml/badge.svg?branch=main)](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/windows.yml)
[![Macos](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/macos.yml/badge.svg?branch=main)](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/actions/workflows/macos.yml)
[![Codecov test coverage](https://codecov.io/gh/Boehringer-Ingelheim/BayesianMCPMod/branch/main/graph/badge.svg)](https://app.codecov.io/gh/Boehringer-Ingelheim/BayesianMCPMod?branch=main)
<!-- badges: end -->

# The `BayesianMCPmod` package

Simulate, analyze, and evaluate Bayesian MCPMod trials with normally and binary distributed endpoints.

## Installation

Install the package from CRAN using

```{r}
install.packages("BayesianMCPMod")
```

The development version can be installed from this repository.

```{r}
# install.packages("remotes")
remotes::install_github("https://github.com/Boehringer-Ingelheim/BayesianMCPmod")
```


## Documentation

The package documentation is hosted [here](https://boehringer-ingelheim.github.io/BayesianMCPMod/).

## ----setup--------------------------------------------------------------------
library(BayesianMCPMod)
library(RBesT)
library(clinDR)
library(dplyr)
library(tibble)
library(reactable)

set.seed(7015)

## ----table_function, collapse=TRUE--------------------------------------------
display_params_table <- function(named_list) {
  round_numeric <- function(x, digits = 3) if (is.numeric(x)) round(x, digits) else x
  tbl <- data.frame(
    Name  = names(named_list),
    Value = I(lapply(named_list, function(v) {
      if (inherits(v, "Date")) v <- as.character(v)
      if (!is.null(names(v))) paste0("{", paste(names(v), v, sep="=", collapse=", "), "}")
      else v
    }))
  )
  tbl$Value <- lapply(tbl$Value, round_numeric)
  knitr::kable(tbl)
}

## ----eval = FALSE-------------------------------------------------------------
# future::plan(future::multisession, workers = 4L)

## ----Historical Data for Control Arm------------------------------------------
data("metaData")
dataset     <- filter(as.data.frame(metaData), bname == "BRINTELLIX")
histcontrol <- filter(
  dataset,
  dose       == 0,
  primtime   == 8,
  indication == "MAJOR DEPRESSIVE DISORDER",
  protid     != 5)

hist_data   <- data.frame(
  trial = histcontrol$nctno,
  est   = histcontrol$rslt,
  se    = histcontrol$se,
  sd    = histcontrol$sd,
  n     = histcontrol$sampsize)

## ----Defining MAP prior function----------------------------------------------
getPriorList <- function (
  
  hist_data,
  dose_levels,
  dose_names    = NULL,
  robust_weight = 0.5
  
) {
  
  sd_tot <- with(hist_data, sum(sd * n) / sum(n))
  
  gmap <- RBesT::gMAP(
    formula    = cbind(est, se) ~ 1 | trial,
    weights    = hist_data$n,
    data       = hist_data,
    family     = gaussian,
    beta.prior = cbind(0, 100 * sd_tot),
    tau.dist   = "HalfNormal",
    tau.prior  = cbind(0, sd_tot / 4))
  
  prior_ctr <- RBesT::automixfit(gmap)
  
  if (!is.null(robust_weight)) {
    
    prior_ctr <- suppressMessages(RBesT::robustify(
      priormix = prior_ctr,
      weight   = robust_weight,
      sigma    = sd_tot))
    
  }
  
  prior_trt <- RBesT::mixnorm(
    comp1 = c(w = 1, m = summary(prior_ctr)[1], n = 1),
    sigma = sd_tot,
    param = "mn")
  
  prior_list <- c(list(prior_ctr),
                  rep(x     = list(prior_trt),
                      times = length(dose_levels[-1])))
  
  if (is.null(dose_names)) {
    
    dose_names <- c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
    
  }
  
  names(prior_list) <- dose_names
  
  return (prior_list)
  
}

## ----Getting the MAP prior----------------------------------------------------
dose_levels <- c(0, 2.5, 5, 10)

set.seed(7015) # re-sets seed only for this example; remove in your analysis script
prior_list  <- getPriorList(
  hist_data     = hist_data,
  dose_levels   = dose_levels,
  robust_weight = 0.3)

getESS(prior_list)

## -----------------------------------------------------------------------------
# Guesstimate estimation
exp_guesst  <- DoseFinding::guesst(
  model = "exponential", 
  d = 5, p = 0.2, Maxd = max(dose_levels)
)
emax_guesst <- DoseFinding::guesst(
  model = "emax",
  d = 2.5, p = 0.9
)
sigEmax_guesst <- DoseFinding::guesst(
  model = "sigEmax",
  d = c(2.5, 5), p = c(0.5, 0.95)
)
logistic_guesst <- DoseFinding::guesst(
  model = "logistic",
  d = c(5, 10), p = c(0.1, 0.85)
)

## -----------------------------------------------------------------------------
betaMod_params   <- c(delta1 = 1, delta2 = 1)
quadratic_params <- c(delta2 = -0.1)

## -----------------------------------------------------------------------------
mods <- DoseFinding::Mods(
  linear      = NULL,
  # guesstimate scale
  exponential = exp_guesst,
  emax        = emax_guesst,
  sigEmax     = sigEmax_guesst,
  logistic    = logistic_guesst,
  # parameter scale
  betaMod     = betaMod_params,
  quadratic   = quadratic_params,
  # Options for all models
  doses       = dose_levels,
  maxEff      = -1,
  placEff     = -12.8
)

plot(mods)

## -----------------------------------------------------------------------------
display_params_table(mods)

## -----------------------------------------------------------------------------
knitr::kable(DoseFinding::getResp(mods, doses = dose_levels))

## -----------------------------------------------------------------------------
data("metaData")

trial_data <- dplyr::filter(
  dplyr::filter(tibble::tibble(metaData), bname == "BRINTELLIX"),
  primtime   == 8,
  indication == "MAJOR DEPRESSIVE DISORDER",
  protid     == 5
)

n_patients <- c(128, 124, 129, 122)

## -----------------------------------------------------------------------------
posterior <- getPosterior(
  prior_list = prior_list,
  mu_hat     = trial_data$rslt,
  S_hat      = diag(trial_data$se^2),
  calc_ess   = TRUE
)

knitr::kable(summary(posterior))

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
crit_pval <- getCritProb(
  mods           = mods,
  dose_levels    = dose_levels,
  cov_new_trial  = diag(trial_data$se^2),
  alpha_crit_val = 0.05
)

contr_mat <- getContr(
  mods          = mods,
  dose_levels   = dose_levels,
  cov_posterior = diag(summary(posterior)[, 2]^2)
)

## -----------------------------------------------------------------------------
# # i) the frequentist contrast
# contr_mat_prior <- getContr(
#   mods           = mods,
#   dose_levels    = dose_levels,
#   dose_weights   = n_patients,
#   prior_list     = prior_list)
# # ii) re-estimated frequentist contrasts
# contr_mat_prior <- getContr(
#   mods           = mods,
#   dose_levels    = dose_levels,
#   cov_new_trial  = diag(trial_data$se^2))
# # iii)  Bayesian approach using number of patients for new trial and prior distribution
# contr_mat_prior <- getContr(
#   mods           = mods,
#   dose_levels    = dose_levels,
#   dose_weights   = n_patients,
#   prior_list     = prior_list)

## -----------------------------------------------------------------------------
BMCP_result <- performBayesianMCP(
  posterior_list = posterior,
  contr          = contr_mat, 
  crit_prob_adj  = crit_pval)

## -----------------------------------------------------------------------------
BMCP_result

## -----------------------------------------------------------------------------
# If simple = TRUE, uses approx posterior
# Here we use complete posterior distribution
model_fits <- getModelFits(
  models      = mods,
  dose_levels = dose_levels,
  posterior   = posterior,
  simple      = FALSE)

## -----------------------------------------------------------------------------
display_params_table(stats::predict(model_fits, doses = c(0, 2.5, 4, 5, 7, 10)))

## -----------------------------------------------------------------------------
plot(model_fits)

## -----------------------------------------------------------------------------
plot(model_fits, cr_bands = TRUE)

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
bootstrap_quantiles <- getBootstrapQuantiles(
  model_fits = model_fits,
  quantiles  = c(0.025, 0.5, 0.975),
  doses      = c(0, 2.5, 4, 5, 7, 10),
  n_samples  = 10)

## ----collapse = TRUE----------------------------------------------------------
reactable::reactable(
  data = bootstrap_quantiles |>
    tidyr::pivot_wider(names_from = q_prob, values_from = q_val),
  groupBy = "model",
  columns = list(
    dose    = colDef(format = list(aggregated = colFormat(suffix = " dose"))),
    "0.025" = colDef(format = list(cell = colFormat(digits = 4))),
    "0.5"   = colDef(format = list(cell = colFormat(digits = 4))),
    "0.975" = colDef(format = list(cell = colFormat(digits = 4)))
  )
)

## -----------------------------------------------------------------------------
getMED(
  delta       = 4,
  model_fits  = model_fits,
  dose_levels = seq(min(dose_levels), max(dose_levels), by = 0.01))

## -----------------------------------------------------------------------------
# performBayesianMCPMod(
#   posterior_list   = posterior,
#   contr            = contr_mat,
#   crit_prob_adj    = crit_pval,
#   delta            = 4,
#   simple           = FALSE)

## ----collapse=TRUE------------------------------------------------------------
library(BayesianMCPMod)
library(DoseFinding)
library(MCPModPack)

library(tidyr)
library(dplyr)

library(kableExtra)
library(ggplot2)

library(doFuture)

## ----echo = FALSE-------------------------------------------------------------

load_path <- file.path(getwd(), "data", "simulation_comparison")


## ----collapse=TRUE------------------------------------------------------------
doses_sim     <- c(0, 1, 2, 4, 8) 
n_sample      <- c(40, 40, 40, 40, 40)
sd_sim        <- 0.4

max_dose      <- max(doses_sim)    
plc_eff_guess <- 0    

alpha         <- 0.05

## ----collapse=TRUE------------------------------------------------------------
set.seed(7015)
n_sim <- 10000

plan(multisession)
registerDoFuture()

## ----echo = FALSE-------------------------------------------------------------

n_sim <- 10


## ----fig.height=5, collapse=TRUE----------------------------------------------
emax_guess     <- guesst(d = doses_sim[2], p = 0.6, "emax") 
exp_guess      <- guesst(d = doses_sim[2], p = 0.05, model = "exponential", Maxd = max_dose)
logit_guess    <- guesst(d = c(doses_sim[2], doses_sim[3]), p = c(0.1, 0.9), "logistic", Maxd = max_dose) 
sig_emax_guess <- guesst(d = c(doses_sim[2], doses_sim[3]), p = c(0.15, 0.75), model = "sigEmax") 

plot(Mods(linear      = NULL,
          exponential = exp_guess,
          emax        = emax_guess,
          logistic    = logit_guess,
          sigEmax     = sig_emax_guess,
          doses       = doses_sim,
          placEff     = plc_eff_guess,
          direction   = "increasing"),
     main = "Candidate Models")

## ----collapse=TRUE------------------------------------------------------------
exp_eff <- c(0.0001, 0.05, 0.1, 0.2, 0.3, 0.5)

## ----collapse=TRUE------------------------------------------------------------
# Simulation parameters
sim_parameters <- list(n            = n_sample,
                       doses        = doses_sim,
                       dropout_rate = 0.0,
                       go_threshold = 0.1,
                       nsims        = n_sim)

# Candidate dose - response models
models_MCPModPack = list(linear      = NA,
                         exponential = exp_guess,
                         emax        = emax_guess,
                         logistic    = logit_guess,
                         sigemax     = sig_emax_guess)

# Assumed dose - response models (models will be added in loop)
sim_models_part <- list(max_effect     = exp_eff,
                        sd             = rep(sd_sim, length(doses_sim)),
                        placebo_effect = plc_eff_guess)

# Parallelization across assumed dose - response models
powers_MCPModPack_eff <- foreach(
  k = seq_along(models_MCPModPack),
  .combine = cbind, 
  .options.future = list(seed = TRUE)) %dofuture% {
    
    sim_model_k <- c(models_MCPModPack[k], sim_models_part)
    
    MCPModSimulation(endpoint_type   = "Normal",
                     models          = models_MCPModPack,
                     alpha           = alpha,
                     direction       = "increasing",
                     model_selection = "aveAIC",
                     Delta           = 0.1,
                     sim_models      = sim_model_k,
                     sim_parameters  = sim_parameters)$sim_results$power
  
  }

# Post-processing result for printing
colnames(powers_MCPModPack_eff) <- names(models_MCPModPack)
results_MCPModPack_eff <- cbind(max_eff = round(exp_eff, digits = 2),
                                powers_MCPModPack_eff) %>%
  data.frame() %>%
  rename(sigEmax = sigemax) %>%
  mutate(average = rowMeans(select(., linear:sigEmax)))

## ----echo = FALSE-------------------------------------------------------------

results_MCPModPack_eff <- readRDS(file.path(load_path, "results_MCPModPack_eff.rds"))


## ----collapse=TRUE------------------------------------------------------------
# Vague prior specification
prior_list_vague <- rep(list(RBesT::mixnorm(comp1 = c(w = 1, m = 0, n = 1),
                                            sigma = sd_sim, param = "mn")),
                        times = length(doses_sim))
names(prior_list_vague) <- c("Ctrl", "DG_1", "DG_2", "DG_3", "DG_4")

# Parallelization across the expected effects for maximum dose
success_rates_BayesianMCPMod_eff <- foreach(
  k               = seq_along(exp_eff),
  .combine        = rbind, 
  .options.future = list(seed = TRUE)) %dofuture% {
    
    exp_eff_k <- exp_eff[k]
    
    models_BayesianMCPMod <- Mods(linear      = NULL,
                                  exponential = exp_guess,
                                  emax        = emax_guess,
                                  logistic    = logit_guess,
                                  sigEmax     = sig_emax_guess,
                                  doses       = doses_sim,
                                  placEff     = plc_eff_guess,
                                  maxEff      = exp_eff_k,
                                  direction   = "increasing")
       
    # Optimal contrasts
    contr <- getContr(mods         = models_BayesianMCPMod,
                      dose_levels  = doses_sim,
                      prior_list   = prior_list_vague,
                      dose_weights = rep(1, length(doses_sim)))
  
    # Perform Simulations
    sim_result <- assessDesign(n_patients     = n_sample,
                               mods           = models_BayesianMCPMod,
                               prior_list     = prior_list_vague,
                               sd             = sd_sim,
                               n_sim          = n_sim,
                               alpha_crit_val = alpha,
                               contr          = contr)
    
    c(sapply(sim_result, attr, "successRate"),
      average = attr(sim_result, "avgSuccessRate"))
  
  }

# Post-processing result for printing
rownames(success_rates_BayesianMCPMod_eff) <- NULL
results_BayesianMCPMod_eff <- data.frame(cbind(max_eff = round(exp_eff, digits = 2),
                                           success_rates_BayesianMCPMod_eff))

## ----echo = FALSE-------------------------------------------------------------

results_BayesianMCPMod_eff <- readRDS(file.path(load_path, "results_BayesianMCPMod_eff.rds"))


## ----fig.height=5-------------------------------------------------------------
## pre-processing the data
df_plot_eff <- rbind(results_MCPModPack_eff %>% 
                     mutate(package_name = "MCPModPack"),
                     results_BayesianMCPMod_eff %>% 
                     mutate(package_name = "BayesianMCPMod")) %>%
  pivot_longer(cols      = names(results_BayesianMCPMod_eff)[-1],
               names_to  = "model_shape",
               values_to = "success_rate") %>%
  filter(model_shape != "average") %>%
  spread(key = package_name, value = success_rate)  %>%
  mutate(model_shape = as.factor(model_shape),
         max_eff     = as.factor(max_eff)) %>%
  group_by(max_eff) %>%
  mutate(offset = 0.1 * (seq_along(model_shape) - ceiling(length(model_shape) / 2)))

# Plot with short horizontal dashes for each model_shape
ggplot(df_plot_eff, aes(x = as.numeric(max_eff) + offset, y = BayesianMCPMod, color = model_shape)) +
  geom_segment(aes(x = as.numeric(max_eff) + offset - 0.05, xend = as.numeric(max_eff) + offset + 0.05,
                   y = BayesianMCPMod, yend = BayesianMCPMod)) +
  geom_segment(aes(xend = as.numeric(max_eff) + offset, yend = MCPModPack)) +
  scale_x_continuous(breaks = unique(as.numeric(df_plot_eff$max_eff)), labels = levels(df_plot_eff$max_eff)) +
  labs(title    = "Comparing Power and Success Rate",
       subtitle = "For Different Expected Effects for Maximum Dose",
       x        = "Expected Effect for Maximum Dose",
       y        = "Success Rate / Power",
       color    = "Assumed True Model Shapes",
       linetype = "Type",
       caption  = "Horizontal lines represent the BayesianMCPMod success rates and vertical lines mark the distance to the MCPModPack power.") +
  theme_minimal() + 
  theme(legend.position = "bottom")

## -----------------------------------------------------------------------------
kable(results_MCPModPack_eff) %>%
  kable_classic() %>%
    add_header_above(c("Power Values Across Different Expected Effects" = 7),
                     font_size = 15, bold = TRUE) %>%
    add_header_above(c("MCPModPack" = 7), font_size = 15, bold = TRUE)

## -----------------------------------------------------------------------------
kable(results_BayesianMCPMod_eff) %>%
  kable_classic(full_width = TRUE) %>%
    add_header_above(c("Success Rates Across Different Expected Effects" = 7),
                     font_size = 15, bold = TRUE) %>%
    add_header_above(c("BayesianMCPMod" = 7), font_size = 15, bold = TRUE) 

## -----------------------------------------------------------------------------
exp_eff_fix <- 0.2

## -----------------------------------------------------------------------------
n_sim_vec <- seq(2, 10, 1)^4

## ----echo = FALSE-------------------------------------------------------------
n_sim_vec <- c(5, 10)

## -----------------------------------------------------------------------------
# Updating the assumed dose - response models for the fixed expected effect (models will be added in loop)
sim_models_part$max_effect <- exp_eff_fix

# Parallelization across assumed dose - response models & number of simulations
## Reversing the numbers speeds up parallelization in case there are not enough workers
sim_grid <- expand.grid(names(models_MCPModPack), rev(n_sim_vec))
powers_MCPModPack_conv <- foreach(
  k = seq_len(nrow(sim_grid)),
  .combine = c,
  .options.future = list(seed = TRUE)) %dofuture% {
    
    sim_model_k <- c(models_MCPModPack[sim_grid[k, 1]], sim_models_part)
    
    sim_params_k <- sim_parameters
    sim_params_k$nsims <- sim_grid[k, 2]
    
    MCPModSimulation(endpoint_type   = "Normal",
                     models          = models_MCPModPack,
                     alpha           = alpha,
                     direction       = "increasing",
                     model_selection = "aveAIC",
                     Delta           = 0.1,
                     sim_models      = sim_model_k,
                     sim_parameters  = sim_params_k)$sim_results$power
  
  }

#Post-processing
results_MCPModPack_conv <- cbind(sim_grid, powers_MCPModPack_conv) %>%
  arrange(desc(row_number())) %>%
  rename(model_name   = Var1,
         n_sim        = Var2,
         success_rate = powers_MCPModPack_conv) %>%
  mutate(model_name = if_else(model_name == "sigemax",
                             "sigEmax", model_name)) %>%
  mutate(package_name = "MCPModPack")

## ----echo = FALSE-------------------------------------------------------------

results_MCPModPack_conv <- readRDS(file.path(load_path, "results_MCPModPack_conv.rds"))


## -----------------------------------------------------------------------------
# Model specifications with fixed expected effect
models_BayesianMCPMod <- Mods(linear      = NULL,
                              exponential = exp_guess,
                              emax        = emax_guess,
                              logistic    = logit_guess,
                              sigEmax     = sig_emax_guess,
                              doses       = doses_sim,
                              placEff     = plc_eff_guess,
                              maxEff      = exp_eff_fix,
                              direction   = "increasing")
       
# Optimal contrasts
contr <- getContr(mods         = models_BayesianMCPMod,
                  dose_levels  = doses_sim,
                  prior_list   = prior_list_vague,
                  dose_weights = rep(1, length(doses_sim)))

# Perform Simulations
sim_result <- assessDesign(n_patients     = n_sample,
                           mods           = models_BayesianMCPMod,
                           prior_list     = prior_list_vague,
                           sd             = sd_sim,
                           n_sim          = max(n_sim_vec),
                           alpha_crit_val = alpha,
                           contr          = contr)

# Getting success rates at different numbers of simulations
results_BayesianMCPMod_conv <- sapply(sim_result, function (model_result) {
  
  sapply(n_sim_vec, function (n_sim_x) {
    
    success_rate_x <- mean(model_result[seq_len(n_sim_x), 1])
    
  })
  
}) %>%
  # Post-processing
  as.data.frame %>%
  mutate(n_sim = n_sim_vec) %>%
  pivot_longer(cols = -n_sim,
               names_to = "model_name",
               values_to = "success_rate") %>%
  mutate(package_name = "BayesianMCPMod")

## ----echo = FALSE-------------------------------------------------------------

results_BayesianMCPMod_conv <- readRDS(file.path(load_path, "results_BayesianMCPMod_conv.rds"))


## ----fig.height=5-------------------------------------------------------------
df_plot_conv <- inner_join(results_BayesianMCPMod_conv,
                           results_MCPModPack_conv,
                           by = c("model_name", "n_sim")) %>%
  mutate(success_rate_diff = success_rate.x - success_rate.y) %>%
  select(model_name, n_sim, success_rate_diff)

ggplot(df_plot_conv, aes(x = n_sim, y = success_rate_diff, color = model_name)) +
  geom_point() +
  geom_line() +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey")+
  scale_x_continuous(breaks = unique(df_plot_conv$n_sim)[-c(2, 3)]) +
  scale_y_continuous(breaks = c(-0.05, 0, 0.05, 0.1, 0.15)) +
  labs(
    title   = "Success Rate Difference vs Number of Simulations",
    x       = "Number of Simulations",
    y       = "Success Rate Difference",
    color   = "True Model Shape",
    caption = "The success rate difference is the difference of the success rates calculated 
    with BayesianMCPMod and the power values calculated with MCPModPack.") +
  theme_minimal() + 
  theme(panel.grid.minor.x = element_blank())

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, collapse = TRUE---------------------------------------------------
library(BayesianMCPMod)
library(clinDR)
library(dplyr)

set.seed(7015)

## ----eval = FALSE-------------------------------------------------------------
# future::plan(future::multisession, workers = 4L)

## ----Historical Data----------------------------------------------------------
data("metaData")
testdata    <- as.data.frame(metaData)
dataset     <- filter(testdata, bname == "BRINTELLIX")
histcontrol <- filter(dataset, dose == 0, primtime == 8, indication == "MAJOR DEPRESSIVE DISORDER")

hist_data <- data.frame(
  trial = histcontrol$nctno,
  est   = histcontrol$rslt,
  se    = histcontrol$se,
  sd    = histcontrol$sd,
  n     = histcontrol$sampsize)

sd_tot <- with(hist_data, sum(sd * n) / sum(n))

## ----Setting Prior Without Execution, eval = FALSE----------------------------
# dose_levels <- c(0, 2.5, 5, 10, 20)
# 
# prior_list  <- getPriorList(
#   hist_data     = hist_data,
#   dose_levels   = dose_levels,
#   robust_weight = 0.3)

## ----Setting Prior, echo = FALSE----------------------------------------------
dose_levels <- c(0, 2.5, 5, 10, 20)

prior_list  <- list(
  Ctr  = RBesT::mixnorm(
    comp1  = c(w = 0.446213, m = -12.774661, s = 1.393130),
    comp1  = c(w = 0.253787, m = 3.148116,   s = 3.148116),
    robust = c(w = 0.3,      m = 9.425139,   s = 9.425139),
    sigma = sd_tot),
  DG_1 = RBesT::mixnorm(
    comp1 = c(w = 1, m = -12.816875, n = 1),
    sigma = sd_tot,
    param = "mn"),
  DG_2 = RBesT::mixnorm(
    comp1 = c(w = 1, m = -12.816875, n = 1),
    sigma = sd_tot,
    param = "mn"),
  DG_3 = RBesT::mixnorm(
    comp1 = c(w = 1, m = -12.816875, n = 1),
    sigma = sd_tot,
    param = "mn"),
  DG_4 = RBesT::mixnorm(
    comp1 = c(w = 1, m = -12.816875, n = 1),
    sigma = sd_tot,
    param = "mn")
)

## ----Setting Vague Prior Without Execution, eval = FALSE----------------------
# prior_list_vague <- rep(list(RBesT::mixnorm(comp1 = c(w = 1, m = 0, n = 1),
#                                             sigma = sd_sim, param = "mn")),
#                         times = length(dose_levels))
# names(prior_list_vague) <- c("Ctrl", "DG_1", "DG_2", "DG_3", "DG_4")

## -----------------------------------------------------------------------------
exp     <- DoseFinding::guesst(
  d     = 5,
  p     = c(0.2),
  model = "exponential",
  Maxd  = max(dose_levels))

emax    <- DoseFinding::guesst(
  d     = 2.5,
  p     = c(0.9),
  model = "emax")

sigemax <- DoseFinding::guesst(
  d     = c(2.5, 5),
  p     = c(0.1, 0.6),
  model = "sigEmax")

sigemax2 <- DoseFinding::guesst(
  d     = c(2, 4),
  p     = c(0.3, 0.8),
  model = "sigEmax")

mods <- DoseFinding::Mods(
  linear      = NULL,
  emax        = emax,
  exponential = exp,
  sigEmax     = rbind(sigemax, sigemax2),
  doses       = dose_levels,
  maxEff      = -3,
  placEff     = -12.8)

n_patients <- c(60, 80, 80, 80, 80)

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities <- assessDesign(
  n_patients  = n_patients,
  mods        = mods,
  prior_list  = prior_list,
  sd          = sd_tot,
  n_sim       = 100) # speed up example run-time

success_probabilities

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities_uneq <- assessDesign(
  n_patients  = c(80, 60, 60, 60, 120),
  mods        = mods,
  prior_list  = prior_list,
  sd          = sd_tot,
  n_sim       = 100) # speed up example run-time
success_probabilities_uneq

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities_dr <- assessDesign(
  n_patients  = c(60, 80, 80, 80, 80),
  mods        = mods,
  prior_list  = prior_list,
  sd          = sd_tot,
  dr_means    = c(-12, -14, -15, -16, -17),
  n_sim       = 100) # speed up example run-time
success_probabilities_dr

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities_med <- assessDesign(
  n_patients  = c(60, 80, 80, 80, 80),
  mods        = mods,
  prior_list  = prior_list,
  sd          = sd_tot,
  delta       = 2,
  n_sim       = 100) # speed up example run-time
success_probabilities_med

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
suppressPackageStartupMessages({
  library(BayesianMCPMod)
  library(RBesT)
  library(DoseFinding)
  library(dplyr)
})

set.seed(7015)

display_params_table <- function(named_list) {
  round_numeric <- function(x, digits = 3) if (is.numeric(x)) round(x, digits) else x
  tbl <- data.frame(
    Name  = names(named_list),
    Value = I(lapply(named_list, function(v) {
      if (inherits(v, "Date")) v <- as.character(v)
      if (!is.null(names(v))) paste0("{", paste(names(v), v, sep="=", collapse=", "), "}")
      else v
    }))
  )
  tbl$Value <- lapply(tbl$Value, round_numeric)
  knitr::kable(tbl)
}

## ----eval = FALSE-------------------------------------------------------------
# future::plan(future::multisession, workers = 4L)

## -----------------------------------------------------------------------------
trial <- c("trial_1", "trial_2", "trial_3")
n     <- c(70,  115, 147) # sample size per trial
r     <- c( 6,   16,  16) # n responders per trial

## -----------------------------------------------------------------------------
dose_levels <- c(0, 2.5, 5, 10, 20, 50, 100, 200)

# 1) Establish MAP prior (beta mixture distribution)
set.seed(7015) # re-set seed only for this example; remove in your analysis script
map <- gMAP(
  cbind(r, n - r) ~ 1 | trial,
  family     = binomial,
  tau.dist   = "HalfNormal",
  tau.prior  = 0.5,
  beta.prior = (1 / sqrt(0.1 * 0.9)),
  warmup     = 1000,
  iter       = 10000,
  chains     = 2,
  thin       = 1
)
map

prior <- automixfit(map) #fits mixture distribution from MCMC samples from above
p     <- summary(prior)[1]

# 2) Robustify prior
prior_rob <- RBesT::robustify(priormix = prior,
                              mean     = 0.5,
                              weight   = 0.4)

# 3) Translate prior to logit scale (to approximate via normal mixture model)
r                <- rmix(prior_rob, n = 1e4)
log_r            <- RBesT::logit(r)
prior_ctr        <- automixfit(log_r, type = "norm")

# Specification of reference scale (this follows the idea of [@Neuenschwander2016]).
sigma(prior_ctr) <- sqrt(1 / (p * (1 - p)))

# Specify a prior list
prior_trt <- RBesT::mixnorm(
  comp1 = c(
    w = 1,
    m = logit(summary(prior)[1]),
    n = 1
  ),
  sigma = sqrt(1 / (p * (1 - p))),
  param = "mn"
)

prior_list <- c(list(prior_ctr),
                rep(x     = list(prior_trt),
                    times = length(dose_levels[-1])))

dose_names        <- c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
names(prior_list) <- dose_names

## -----------------------------------------------------------------------------
models <- Mods(
  linear      = NULL,
  sigEmax     = c(50, 3),
  quadratic   = -1 / 250,
  logistic    = c(110, 15),
  exponential = 80,
  emax        = 10,
  doses       = dose_levels,
  placEff     = RBesT::logit(0.118),
  maxEff      = RBesT::logit(0.3) - RBesT::logit(0.118)
)

plot(models)

## -----------------------------------------------------------------------------
data("migraine") # data set from the DoseFinding package

doses_fact <- as.factor(dose_levels)
n_patients <- migraine$ntrt
resp_rate  <- migraine$painfree/n_patients

## Execution of logistic regression and readout of parameters 
## Note that estimates are automatically on the logit scale.
log_fit <- glm(resp_rate ~ doses_fact - 1, family = binomial, weights = n_patients)
mu_hat  <- coef(log_fit)
S_hat   <- vcov(log_fit)

## -----------------------------------------------------------------------------
post_logit <- getPosterior(prior_list, mu_hat = mu_hat, S_hat  = S_hat)

## -----------------------------------------------------------------------------
summary(post_logit, probability_scale = TRUE)

## -----------------------------------------------------------------------------
contr_mat_prior <- getContr(
  mods           = models,
  dose_levels    = dose_levels,
  dose_weights   = n_patients)

set.seed(7015) # re-sets seed only for this example; remove in your analysis script
crit_pval <- getCritProb(
  mods           = models,
  dose_levels    = dose_levels,
  cov_new_trial  = S_hat,
  alpha_crit_val = 0.05
)

## -----------------------------------------------------------------------------
BMCP_result <- performBayesianMCP(
  posterior_list = post_logit,
  contr          = contr_mat_prior, 
  crit_prob_adj  = crit_pval)

## -----------------------------------------------------------------------------
BMCP_result

## -----------------------------------------------------------------------------
model_fits <- getModelFits(
  models            = models,
  dose_levels       = dose_levels,
  posterior         = post_logit,
  simple            = TRUE,
  probability_scale = TRUE)

## -----------------------------------------------------------------------------
plot(model_fits, cr_bands = TRUE)

## -----------------------------------------------------------------------------
plot(model_fits, probability_scale = FALSE)

## -----------------------------------------------------------------------------
display_params_table(stats::predict(model_fits, doses = c(0, 2.5, 10,150, 200)))

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
bootstrap_quantiles <- getBootstrapQuantiles(
  model_fits = model_fits,
  quantiles  = c(0.025, 0.5, 0.975),
  doses      = dose_levels,
  n_samples  = 10)

## -----------------------------------------------------------------------------
getMED(
  delta       = 0.16, # on probability scale
  model_fits  = model_fits,
  dose_levels = seq(min(dose_levels), max(dose_levels), by = 1))

## ----eval = FALSE-------------------------------------------------------------
# BMCPMod_result <- performBayesianMCPMod(
#   posterior_list    = post_logit,
#   contr             = contr_mat_prior,
#   crit_prob_adj     = crit_pval,
#   simple            = TRUE,
#   delta             = 0.16,
#   probability_scale = TRUE
# )

## ----setup--------------------------------------------------------------------
suppressPackageStartupMessages({
  library(BayesianMCPMod)
  library(RBesT)
  library(DoseFinding)
  library(clinDR)
  library(dplyr)
  library(tibble)
  library(reactable)
})

set.seed(7015)

## ----eval = FALSE-------------------------------------------------------------
# future::plan(future::multisession, workers = 4L)

## -----------------------------------------------------------------------------
data("metaData")
testdata    <- as.data.frame(metaData)
dataset     <- filter(testdata, bname == "XELJANZ")
histcontrol <- filter(dataset, dose == 0, primtime == 12, indication == "RHEUMATOID ARTHRITIS")

hist_data <- data.frame(
  trial = histcontrol$nctno,
  est   = histcontrol$rslt,
  se    = histcontrol$se,
  sd    = histcontrol$sd,
  r     = round(histcontrol$sampsize*histcontrol$rslt),
  n     = histcontrol$sampsize)

sd_tot <- with(hist_data, sum(sd * n) / sum(n))

## -----------------------------------------------------------------------------
dose_levels <- c(0, 2.5, 5, 10,20)

# 1) Establish MAP prior (beta mixture distribution) 
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
map <- gMAP(
  cbind(hist_data$r, hist_data$n - hist_data$r) ~ 1 | histcontrol$nctno,
  family     = binomial,
  tau.dist   = "HalfNormal",
  tau.prior  = 0.5,
  beta.prior = (1 / sqrt(0.1 * 0.9)),
  warmup     = 1000,
  iter       = 10000,
  chains     = 2,
  thin       = 1
)

prior <- automixfit(map) #fits mixture distribution from MCMC samples from above

p <- summary(prior)[1]

# 2) Robustify prior
prior_rob <- RBesT::robustify(
      priormix = prior,
      mean     = 0.5,
      weight   = 0.2)

# 3) Translate prior to logit scale (to approximate via normal mixture model)
r         <- rmix(prior_rob, n = 1e4)
log_r     <- logit(r)
prior_ctr <- automixfit(log_r, type = "norm")

# Specification of reference scale (this follows the idea of [@Neuenschwander2016]). 
sigma(prior_ctr) <- sqrt(1 / (p * (1 - p)))

# Specify a prior list
prior_trt <- RBesT::mixnorm(
    comp1 = c(w = 1, m = logit(summary(prior)[1]), n = 1),
    sigma = sqrt(1/(p*(1-p))),
    param = "mn")
  
  prior_list <- c(list(prior_ctr),
                  rep(x     = list(prior_trt),
                      times = length(dose_levels[-1])))

dose_names        <- c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
names(prior_list) <- dose_names

## ----Setting Vague Prior Without Execution, eval = TRUE-----------------------
prior_list_vague <- rep(list(RBesT::mixnorm(
  comp1 = c(w = 1, m = logit(p), n = 1),
  sigma = sqrt(1 / (p * (1 - p))),
  param = "mn"
)), times = length(dose_levels))
names(prior_list_vague) <- c("Ctrl", "DG_1", "DG_2", "DG_3", "DG_4")

## -----------------------------------------------------------------------------
n_patients <- c(30, 40, 40, 40, 40)

models <- Mods(
  linear      = NULL,
  sigEmax     = c(10, 5),
  logistic    = c(11, 15),
  exponential = 10,
  emax        = 2,
  doses       = dose_levels,
  placEff     = RBesT::logit(0.18),
  maxEff      = (RBesT::logit(0.48) - RBesT::logit(0.18))
)

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities <- assessDesign(
  n_patients        = n_patients,
  mods              = models,
  prior_list        = prior_list,
  probability_scale = TRUE,
  delta             = 0.2,
  n_sim             = 100) # speed up example run-time
success_probabilities

## -----------------------------------------------------------------------------
contr_mat <- getContr(
  mods           = models,
  dose_levels    = dose_levels,
  dose_weights   = n_patients)

success_probabilities <- assessDesign(
  n_patients        = n_patients,
  mods              = models,
  prior_list        = prior_list,
  probability_scale = TRUE,
  contr             = contr_mat,
  delta             = 0.2,
  n_sim             = 100) # speed up example run-time
success_probabilities

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities_uneq <- assessDesign(
  n_patients        = c(40, 30, 30, 30, 50),
  mods              = models,
  prior_list        = prior_list,
  probability_scale = TRUE,
  delta             = 0.2,
  n_sim             = 100) # speed up example run-time
success_probabilities_uneq

## -----------------------------------------------------------------------------
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities_vague <- assessDesign(
  n_patients        = n_patients,
  mods              = models,
  prior_list        = prior_list_vague,
  probability_scale = TRUE,
  delta             = 0.2,
  n_sim             = 100) # speed up example run-time
success_probabilities_vague

#' @title assessDesign
#'
#' @description This function performs simulation based trial design evaluations for a set of specified dose-response models
#'
#' @param n_patients Vector specifying the planned number of patients per dose group. A minimum of 2 patients are required in each group.
#' @param mods An object of class `Mods` as specified in the `DoseFinding` package.
#' @param prior_list A prior_list object specifying the utilized prior for the different dose groups
#' @param sd A positive value, specification of assumed sd. Not required if either `data_sim` or `estimates_sim` is provided. Also not required in case of binary endpoint. Default NULL
#' @param contr An object of class `optContr` as created by the `DoseFinding::getContr` function. Allows specification of a fixed contrasts matrix. Default NULL.
#' @param dr_means A vector, allows specification of individual (not model based) assumed effects per dose group. Default NULL.
#' @param data_sim An optional data frame for custom simulated data. Must follow the data structure as provided by `simulateData()`. Default NULL.
#' @param estimates_sim An optional named list of 1) list of vectors for the estimated means per dose group (`estimates_sim$mu_hats`) and 2) of list of matrices for the covariance matrices specifying the (estimated) variabilities (`estimates_sim$S_hats`). Dimensions of entries must match the number of dose levels. Default NULL.
#' @param n_sim Number of simulations to be performed
#' @param alpha_crit_val (Un-adjusted) Critical value to be used for the MCP testing step. Passed to the `getCritProb` function for the calculation of adjusted critical values (on the probability scale). Default 0.05.
#' @param modeling Boolean variable defining whether the Mod part of Bayesian MCP-Mod will be performed in the assessment. More heavy on resources. Default FALSE.
#' @param simple Boolean variable defining whether simplified fit will be applied, see `?getModelFits`. Set automatically to TRUE if argument `delta` is provided. Passed to `getModelFits` Default TRUE.
#' @param avg_fit Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.
#' @param reestimate Boolean variable defining whether critical value should be calculated with re-estimated contrasts (see `getCritProb` function for more details). Default FALSE.
#' @param delta A numeric value for the threshold Delta for the MED assessment. If NULL, no MED assessment is performed. Default NULL.
#' @param evidence_level A numeric value between 0 and 1 for the evidence level gamma for the MED assessment. Only required for Bayesian MED assessment, see `?getMED` for details. If NULL, MED assessment will be performed on the fitted model according to the argument `med_selection`.  Default NULL.
#' @param n_bs_samples Number of bootstrap samples for the MED assessment if `evidence_level` is provided. Note that more extreme quantiles (i.e., quantiles closer to 0 or 1) tend to require more bootstrap samples to maintain precision. Default 1000.
#' @param med_selection A string, either `"avgFit"` or `"bestFit"`, for the method of MED selection. Default `"avgFit"`.
#' @param probability_scale A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform calculations from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.
#'
#' @details
#' For binary endpoints, if separation occurs (for example, when one treatment arm has only responders or only non‑responders), we use penalized logistic regression with Firth’s correction to prevent convergence issues and obtain more stable estimates.
#' See the references for additional details.
#'
#' @references
#' Heinze, G. & Schemper, M. (2002). A solution to the problem of separation
#' in logistic regression. *Statistics in Medicine*, 21(16), 2409–2419.
#' 
#' Liu et al. (2022). Commentary: analyzing binary data using MCPMod when zero counts are expected.
#' *arXiv*, 2202.08781, https://arxiv.org/abs/2202.08781
#'
#' @return Returns success probabilities for the different assumed dose-response shapes, attributes also includes information around average success rate (across all assumed models) and prior Effective sample size.
#'
#' @examples
#' n_patients  <- c(40, 60, 60, 60, 60)
#' dose_levels <- c(0, 0.5, 2, 4, 8)
#'  
#' mods <- DoseFinding::Mods(linear      = NULL,
#'                           emax        = c(0.5, 1.2),
#'                           exponential = 2,
#'                           betaMod     = c(1, 1),
#'                           doses       = dose_levels,
#'                           maxEff      = 6)
#'                           
#' sd <- 12
#' prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 12), sigma = 2),
#'                    DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
#'                    DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
#'                    DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2) ,
#'                    DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2))
#' 
#' success_probabilities <- assessDesign(
#'   n_patients  = n_patients,
#'   mods        = mods,
#'   prior_list  = prior_list,
#'   sd          = sd,
#'   n_sim       = 1e2) # speed up example run time
#'
#' success_probabilities
#' 
#' ## Analysis with custom data
#' data_sim <- simulateData(
#'   n_patients        = n_patients,
#'   dose_levels       = dose_levels,
#'   sd                = sd,
#'   mods              = mods,
#'   n_sim             = 10)
#'
#' success_probabilities_cd <- assessDesign(
#'   n_patients  = n_patients,
#'   mods        = mods,
#'   prior_list  = prior_list,
#'   data_sim    = data_sim,
#'   sd          = sd,
#'   n_sim       = 1e2) # speed up example run time
#'
#' success_probabilities_cd
#' 
#' ## Analysis with custom dose response relationship
#' custom_dr_means <- c(1, 2, 3, 4, 5)
#' 
#' success_probs_custom_dr <- assessDesign(
#'   n_patients  = n_patients,
#'   mods        = mods,
#'   prior_list  = prior_list,
#'   dr_means    = custom_dr_means,
#'   sd          = sd,
#'   n_sim       = 1e2) # speed up example run time
#'
#' success_probs_custom_dr
#' 
#' ## Analysis with custom estimates for means and variabilies
#' ## No simulated data, only simulated model estimates
#' estimates_sim <- list(mu_hats = replicate(1e2, list(c(1, 2, 3, 4, 5) + rnorm(5, 0, 1))),
#'                       S_hats  = list(diag(1, 5)))
#' 
#' success_probs_custom_est <- assessDesign(
#'   n_patients    = n_patients,
#'   mods          = mods,
#'   prior_list    = prior_list,
#'   estimates_sim = estimates_sim)
#'
#' success_probs_custom_est
#'
#' if (interactive()) { # takes typically > 5 seconds
#'
#' # with MED estimation without bootstrapping
#' # see ?getMED for details
#'
#' success_probabilities <- assessDesign(
#'   n_patients     = n_patients,
#'   mods           = mods,
#'   prior_list     = prior_list,
#'   sd             = sd,
#'   modeling       = TRUE,
#'   n_sim          = 10, # speed up example run time
#'   delta          = 3)
#'
#'   success_probabilities
#'
#' # with MED estimation with bootstrapping
#'
#' success_probabilities <- assessDesign(
#'   n_patients     = n_patients,
#'   mods           = mods,
#'   prior_list     = prior_list,
#'   sd             = sd,
#'   modeling       = TRUE,
#'   n_sim          = 10, # speed up example run time
#'   delta          = 3,
#'   evidence_level = 0.8)
#'
#'   success_probabilities
#'
#' }
#'
#' @export
assessDesign <- function (
    
  n_patients,
  mods,
  prior_list,
  
  sd                = NULL,
  
  contr             = NULL,
  
  dr_means          = NULL,
  data_sim          = NULL,
  estimates_sim     = NULL,
  
  n_sim             = 1e3,
  alpha_crit_val    = 0.05,
  modeling          = FALSE,
  simple            = TRUE,
  avg_fit           = TRUE,
  reestimate        = FALSE,
  
  delta             = NULL,
  evidence_level    = NULL,
  n_bs_samples      = 1000,
  med_selection     = c("avgFit", "bestFit"),
  
  probability_scale = FALSE
  
) {
  
  checkmate::assert_vector(n_patients, len = length(attr(mods, "doses")), any.missing = FALSE)
  checkmate::assert_double(n_patients, lower = 2, upper = Inf)
  checkmate::assert_class(mods, classes = "Mods")
  checkmate::assert_list(prior_list, names = "named", len = length(attr(mods, "doses")), any.missing = FALSE)
  checkmate::assert_list(estimates_sim, null.ok = TRUE)
  # sensitive to how DoseFinding labels their attributes for "Mods" class
  checkmate::assert_integerish(n_sim, lower = 1, upper = Inf)
  checkmate::assert_double(alpha_crit_val, lower = 0, upper = 1)
  checkmate::assert_flag(modeling)
  checkmate::assert_flag(probability_scale)
  
  # TODO: check that prior_list has 'sd_tot' attribute, and that it's numeric # this is not applicable at the moment
  
  stopifnot("Cannot specify both 'data_sim' and 'estimates_sim' simultaneously" = 
              is.null(data_sim)  & is.null(estimates_sim) |
              !is.null(data_sim) & is.null(estimates_sim) |
              is.null(data_sim)  & !is.null(estimates_sim))
  
  modeling <- ifelse(!is.null(delta), TRUE, modeling)
  
  dose_levels <- attr(mods, "doses")
  
  if (!is.null(estimates_sim)) {
    
    stopifnot("'estimates_sim' must be a named list of lenght 2 with list items called 'mu_hats' and 'S_hats'" =
                length(estimates_sim) ==  2L &
                all(names(estimates_sim) %in% c("mu_hats", "S_hats")))
    stopifnot("The length of 'estimates_sim$S_hats' must match 'estimates_sim$mu_hats' or 1." =
                length(estimates_sim$S_hats) == length(estimates_sim$mu_hats) |
                length(estimates_sim$S_hats) == 1L)
    stopifnot("The length of each entry of 'estimates_sim$mu_hats' must match the number of dose levels." =
                length(estimates_sim$mu_hats[[1]]) == length(dose_levels))
    stopifnot("The dimensions of each entry of 'estimates_sim$S_hats' must match the number of dose levels." =
                ncol(estimates_sim$S_hats[[1]]) == length(dose_levels))
    
  } else if (!is.null(data_sim)) {
    
    ## lazily simulate data anyway ...
    data <- simulateData(
      n_patients        = n_patients,
      dose_levels       = dose_levels,
      sd                = 1,
      mods              = mods,
      n_sim             = floor(nrow(data_sim) / sum(n_patients)), # adjust the number of simulations to data_sim
      dr_means          = dr_means,
      probability_scale = probability_scale)
    
    ## ... and then check if formats of data_sim and data match
    stopifnot("'data_sim' must follow the data structure as provided by simulateData()" =
                isTRUE(all.equal(data_sim[, 1:3], data[, 1:3])))
    stopifnot("data_sim must have named columns" = !is.null(colnames(data_sim)))
    checkmate::assert_data_frame(data_sim, types = "numeric", any.missing = FALSE)
    
    data <- data_sim
    
  } else {
    
    data <- simulateData(
      n_patients        = n_patients,
      dose_levels       = dose_levels,
      sd                = sd,
      mods              = mods,
      n_sim             = n_sim,
      dr_means          = dr_means,
      probability_scale = probability_scale)
    
  }
  
  crit_prob_adj <- getCritProb(
    mods           = mods,
    dose_levels    = dose_levels,
    dose_weights   = n_patients,
    alpha_crit_val = alpha_crit_val)
  
  if (!reestimate & is.null(contr)) {
    
    if (!is.null(data_sim) | !is.null(estimates_sim)) {
      
      message("Consider to provide 'contr' for your custom simulated data or analysis results.")
      
    }
    
    contr <- getContr(
      mods           = mods,
      dose_levels    = dose_levels,
      dose_weights   = n_patients,
      prior_list     = prior_list)
    
  }
  
  # get model names of true underlying models
  if (is.null(estimates_sim))  {
    
    model_names <- colnames(data)[-c(1, 2, 3)]
    
  } else {
    
    model_names <- "estimates_sim"
    
  }
  
  eval_design <- lapply(model_names, function (model_name) {
    
    if (is.null(estimates_sim)) {
      
      posterior_list <- getPosterior(
        data              = getModelData(data, model_name),
        prior_list        = prior_list,
        probability_scale = probability_scale)
      
    } else {
      
      posterior_list <- mapply(getPosterior,
             mu_hat   = estimates_sim$mu_hats,
             S_hat    = estimates_sim$S_hats,
             MoreArgs = list(prior_list = prior_list),
             SIMPLIFY = FALSE)
      
      names(posterior_list) <- seq_along(estimates_sim$mu_hats)
      
    }
    
    if (reestimate & is.null(contr)) {
      
      post_sds <- sapply(posterior_list, function (post) summary(post)[, 2])
      
      contr <- apply(post_sds, 2, function (post_sd) getContr(
        mods          = mods,
        dose_levels   = dose_levels,
        cov_posterior = diag(post_sd^2)))
      
    }
    
    if (modeling) {
      
      attr(contr, "direction") <- attr(mods,"direction")
      
      b_mcp_mod <- performBayesianMCPMod(
        posterior_list    = posterior_list,
        contr             = contr,
        crit_prob_adj     = crit_prob_adj,
        simple            = simple,
        avg_fit           = avg_fit,
        delta             = delta,
        evidence_level    = evidence_level,
        med_selection     = med_selection,
        n_samples         = n_bs_samples,
        probability_scale = probability_scale) # should this also be transferred via attribute?
      
    } else {
      
      b_mcp <- performBayesianMCP(
        posterior_list = posterior_list,
        contr          = contr,
        crit_prob_adj  = crit_prob_adj)
      
    }
    
  })
  
  names(eval_design) <- model_names
  
  avg_success_rate <- mean(sapply(eval_design, function (x) {
    
    ifelse(identical(modeling, FALSE),
           attr(x, "successRate"),
           attr(x$BayesianMCP, "successRate"))
    
  }))
  
  attr(eval_design, "avgSuccessRate") <- avg_success_rate
  
  if (modeling & !is.null(delta)) {
    
    avg_med_id_rate <- mean(sapply(eval_design, function (x) {
      
      mean(attr(x, "MED")[, "med_reached"])
      
    }))
    
    attr(eval_design, "avgMEDIdentificationRate") <- avg_med_id_rate
    
  }
  
  attr(eval_design, "placEff")        <- ifelse(test = is.null(dr_means),
                                                yes  = attr(mods, "placEff"),
                                                no   = dr_means[1])
  attr(eval_design, "maxEff")         <- ifelse(test = is.null(dr_means),
                                                yes  = attr(mods, "maxEff"), # this is on logit scale in case of binary endpoint
                                                no   = diff(range(dr_means)))
  attr(eval_design, "sampleSize")     <- n_patients
  attr(eval_design, "priorESS")       <- round(getESS(prior_list), 1)
  
  return (eval_design)
  
}

# Tests for assessDesign --------------------------------------------------



test_that("base case input throws no error and has correct properties", {
  expect_no_error(
    eval_design <- assessDesign(
      n_patients = n_patients,
      mods = mods,
      sd = sd,
      prior_list = prior_list,
      n_sim = n_sim,
      alpha_crit_val = alpha_crit_val,
      simple = TRUE
    )
  )
  
  # assessDesign should give results for each model in mods
  expect_equal(
    names(eval_design), names(mods)
  )
  
  # assessDesign result should have rows = n_sim
  expect_equal(
    attr(eval_design$linear, "dim")[1],
    n_sim
  )
  
  # assessDesign result (in this base case) should have crit_prob = 1 - alpha_crit_val
  expect_equal(
    attr(eval_design$linear, "critProb"),
    1 - alpha_crit_val
  )
  
  contr_mat <- getContr(
    mods = mods,
    dose_levels = dose_levels,
    dose_weights = n_patients,
    prior_list = prior_list
  )
  
  expect_no_error(
    eval_design <- assessDesign(
      n_patients = n_patients,
      mods = mods,
      sd = sd,
      prior_list = prior_list,
      n_sim = n_sim,
      alpha_crit_val = alpha_crit_val,
      simple = TRUE,
      modeling = TRUE
    )
  )
  
  # assessDesign result should have rows = n_sim
  expect_equal(
    attr(eval_design$linear$BayesianMCP, "dim")[1],
    n_sim
  )
  
  # assessDesign result (in this base case) should have crit_prob = 1 - alpha_crit_val
  expect_equal(
    attr(eval_design$linear$BayesianMCP, "critProb"),
    1 - alpha_crit_val
  )
  
  expect_no_error(
    assessDesign(
      n_patients = n_patients,
      mods = mods,
      sd = sd,
      prior_list = prior_list,
      n_sim = n_sim,
      alpha_crit_val = alpha_crit_val,
      simple = TRUE,
      reestimate = TRUE,
      contr = contr_mat
    )
  )
  
  
  sd_tot <- 9.4
  
  dose_levels <- c(0, 2.5, 5, 10, 20)
  
  prior_list <- lapply(dose_levels, function(dose_group) {
    RBesT::mixnorm(weak = c(w = 1, m = 0, s = 200), sigma = 10)
  })
  
  names(prior_list) <- c("Ctr", paste0("DG_", dose_levels[-1]))
  
  exp <- DoseFinding::guesst(
    d     = 5,
    p     = c(0.2),
    model = "exponential",
    Maxd  = max(dose_levels)
  )
  
  emax <- DoseFinding::guesst(
    d     = 2.5,
    p     = c(0.9),
    model = "emax"
  )
  
  sigemax <- DoseFinding::guesst(
    d     = c(2.5, 5),
    p     = c(0.1, 0.6),
    model = "sigEmax"
  )
  
  sigemax2 <- DoseFinding::guesst(
    d     = c(2, 4),
    p     = c(0.3, 0.8),
    model = "sigEmax"
  )
  
  mods <- DoseFinding::Mods(
    linear      = NULL,
    emax        = emax,
    exponential = exp,
    sigEmax     = rbind(sigemax, sigemax2),
    doses       = dose_levels,
    maxEff      = -3,
    placEff     = -12.8
  )
  
  n_patients <- c(60, 80, 80, 80, 80)
  
  expect_no_error(
    assessDesign(
      n_patients = n_patients,
      mods       = mods,
      prior_list = prior_list,
      sd         = sd_tot,
      n_sim      = 10,
      reestimate = TRUE
    )
  )
})


### n_patients param ###

test_that("assessDesign validates n_patients parameter input and give appropriate error messages", {
  # assertions that aren't tested here for sake of brevity
  # n_patients should be a non-NULL numeric vector
  
  expect_error(
    assessDesign(n_patients = n_patients[-1], sd = sd, mods = mods, prior_list = prior_list, n_sim = n_sim)
  )
  
  expect_error(
    assessDesign(n_patients = rep(1, length(n_patients)), sd = sd, mods = mods, prior_list = prior_list, n_sim = n_sim),
  )
})

### mods param ###

test_that("assessDesign validates mods parameter input and give appropriate error messages", {
  # assertions that aren't tested here for sake of brevity
  # mods should be non-NULL object of class "Mods" from {DoseFinding}
  
  
  # checking that DoseFinding didn't change how they named their 'doses' attribute
  expect_true(
    "doses" %in% names(attributes(mods))
  )
  
  mods2 <- mods
  attr(mods2, "doses") <- 0
  expect_error(
    assessDesign(n_patients = n_patients, mods = mods2, sd = sd, prior_list = prior_list, n_sim = n_sim)
  )
  rm(mods2)
})

## prior_list param ###

test_that("assessDesign validates prior_list parameter input and give appropriate error messages", {
  # assertions that aren't tested here for sake of brevity
  # prior_list should be a non-NULL named list with length = number of dose levels
  # length(attr(prior_list, "dose_levels")) == n_patients (see above)
  
  # checking that we didn't change how we named the 'dose_levels' attribute
  expect_true(
    "doses" %in% names(attributes(mods))
  )
})

test_that("assessDesign: input validation branches are triggered", {
  skip_if_not_installed("DoseFinding")
  skip_if_not_installed("RBesT")
  
  dose_levels <- c(0, 1, 2)
  mods <- DoseFinding::Mods(
    linear = NULL,
    doses  = dose_levels,
    maxEff = 1
  )
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 2), sigma = 2)
    }),
    c("Ctr", "DG_1", "DG_2")
  )
  
  # n_patients < 2
  expect_error(
    assessDesign(
      n_patients = c(1, 2, 2),
      mods = mods,
      prior_list = prior_list,
      n_sim = 1
    )
  )
  
  # length mismatch
  expect_error(
    assessDesign(
      n_patients = c(10, 10),
      mods = mods,
      prior_list = prior_list,
      n_sim = 1
    )
  )
  
  # conflicting inputs
  expect_error(
    assessDesign(
      n_patients = c(10, 10, 10),
      mods = mods,
      prior_list = prior_list,
      sd = 1,
      data_sim = data.frame(),
      n_sim = 1
    )
  )
  
  expect_error(
    assessDesign(
      n_patients = c(10, 10, 10),
      mods = mods,
      prior_list = prior_list,
      sd = 1,
      estimates_sim = list(),
      n_sim = 1
    )
  )
})


test_that("assessDesign: binary endpoint runs and returns per-true-model results with avgSuccessRate attribute", {
  skip_if_not_installed("DoseFinding")
  skip_if_not_installed("RBesT")
  
  set.seed(401)
  
  dose_levels <- c(0, 1, 2)
  n_patients  <- c(20, 20, 20)
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    doses  = dose_levels,
    maxEff = 1
  )
  
  # Logit-scale priors
  p_true <- c(0.10, 0.20, 0.40)
  prior_list <- setNames(
    lapply(seq_along(p_true), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(p_true[i]), s = 2), sigma = 2)
    }),
    c("Ctr", "DG_1", "DG_2")
  )
  
  res <- assessDesign(
    n_patients = n_patients,
    mods = mods,
    prior_list = prior_list,
    n_sim = 2,
    probability_scale = TRUE,
    modeling = FALSE
  )
  
  # Returned object is a list with names == true underlying model names
  expect_true(is.list(res))
  expect_true(length(res) >= 1)
  expect_true(!is.null(names(res)))
  
  # avgSuccessRate is stored as attribute on the returned list
  expect_true(!is.null(attr(res, "avgSuccessRate")))
  expect_true(is.numeric(attr(res, "avgSuccessRate")))
  expect_true(attr(res, "avgSuccessRate") >= 0 && attr(res, "avgSuccessRate") <= 1)
  
  # Each element (when modeling=FALSE) is the BayesianMCP result and should have successRate attribute
  for (nm in names(res)) {
    expect_true(!is.null(attr(res[[nm]], "successRate")))
    sr <- attr(res[[nm]], "successRate")
    expect_true(is.numeric(sr))
    expect_true(sr >= 0 && sr <= 1)
  }
})


test_that("assessDesign: custom data_sim path requires model columns and returns results; contr warning is expected", {
  skip_if_not_installed("DoseFinding")
  skip_if_not_installed("RBesT")
  
  set.seed(402)
  
  dose_levels <- c(0, 1)
  n_patients  <- c(30, 30)
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    doses  = dose_levels,
    maxEff = 1
  )
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 2), sigma = 2)
    }),
    c("Ctr", "DG_1")
  )
  
  # IMPORTANT: data_sim must match simulateData() structure:
  # first 3 columns: simulation, ptno, dose
  # then ONE OR MORE model columns (true model responses), e.g. "lin"
  data_sim <- data.frame(
    simulation = 1L,
    ptno = seq_len(sum(n_patients)),
    dose = rep(dose_levels, times = n_patients),
    lin  = rnorm(sum(n_patients))
  )
  
  # assessDesign will message about providing contr; we allow that message
  expect_message(
    res <- assessDesign(
      n_patients = n_patients,
      mods = mods,
      prior_list = prior_list,
      data_sim = data_sim,
      n_sim = 1
    ),
    regexp = "Consider to provide 'contr'"
  )
  
  expect_true(is.list(res))
  expect_true("lin" %in% names(res))
  expect_true(!is.null(attr(res, "avgSuccessRate")))
})


test_that("assessDesign: modeling + delta attaches MED in a robust form and sets avgMEDIdentificationRate", {
  skip_if_not_installed("DoseFinding")
  skip_if_not_installed("RBesT")
  
  set.seed(403)
  
  dose_levels <- c(0, 1, 2)
  n_patients  <- c(25, 25, 25)
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    doses  = dose_levels,
    maxEff = 1
  )
  
  p_flat <- c(0.25, 0.25, 0.25)
  
  prior_list <- setNames(
    lapply(seq_along(p_flat), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(p_flat[i]), s = 1.5), sigma = 2)
    }),
    c("Ctr", "DG_1", "DG_2")
  )
  
  res_avg <- assessDesign(
    n_patients = n_patients,
    mods = mods,
    prior_list = prior_list,
    n_sim = 3,
    probability_scale = TRUE,
    modeling = TRUE,
    delta = 0.2,
    med_selection = "avgFit"
  )
  
  expect_true(is.list(res_avg))
  expect_true(length(res_avg) >= 1)
  
  # Top-level identification rate attribute
  ir <- attr(res_avg, "avgMEDIdentificationRate")
  expect_true(!is.null(ir))
  expect_true(is.numeric(ir))
  expect_true(ir >= 0 && ir <= 1)
  
  # MED is stored as attribute on each element, but its structure may vary.
  # We just need it to contain a med_reached indicator (0/1) in some form.
  extract_med_reached <- function(med) {
    if (is.null(med)) return(NULL)
    
    # case 1: named atomic vector/list
    if (!is.null(names(med)) && "med_reached" %in% names(med)) {
      return(as.numeric(med[["med_reached"]]))
    }
    
    # case 2: matrix/data.frame with rowname
    if ((is.matrix(med) || is.data.frame(med)) && !is.null(rownames(med)) &&
        "med_reached" %in% rownames(med)) {
      return(as.numeric(med["med_reached", ]))
    }
    
    # case 3: matrix/data.frame with column name
    if ((is.matrix(med) || is.data.frame(med)) && "med_reached" %in% colnames(med)) {
      return(as.numeric(med[, "med_reached"]))
    }
    
    # case 4: 2-row matrix without rownames where first row is med_reached
    if (is.matrix(med) && nrow(med) >= 1) {
      # Heuristic: first row should be 0/1-like if it's med_reached
      cand <- as.numeric(med[1, ])
      if (all(cand %in% c(0, 1, NA))) return(cand)
    }
    
    NULL
  }
  
  any_has_med <- FALSE
  any_has_reached <- FALSE
  
  for (nm in names(res_avg)) {
    med <- attr(res_avg[[nm]], "MED")
    if (!is.null(med)) any_has_med <- TRUE
    
    mr <- extract_med_reached(med)
    if (!is.null(mr)) {
      any_has_reached <- TRUE
      mr <- mr[!is.na(mr)]
      expect_true(all(mr %in% c(0, 1)))
    }
  }
  
  # Invariant expectations: at least one element should carry MED,
  # and at least one should expose a med_reached indicator in some form.
  expect_true(any_has_med)
  expect_true(any_has_reached)
})

#' @title getBootstrapQuantiles
#'
#' @description A function for the calculation of bootstrapped model predictions.
#' Samples from the posterior distribution are drawn (via the RBesT function rmix()) and for every sample the simplified fitting step (see getModelFits() function) and a prediction is performed.
#' These fits are then used to identify the specified quantiles.
#' This approach can be considered as the Bayesian equivalent of the frequentist bootstrap approach described in O'Quigley et al. (2017).
#' Instead of drawing n bootstrap samples from the sampling distribution of the trial dose-response estimates, here the samples are directly taken from the posterior distribution.
#' @references O'Quigley J, Iasonos A, Bornkamp B. 2017. Handbook of Methods for Designing, Monitoring, and Analyzing Dose-Finding Trials (1st ed.). Chapman and Hall/CRC. doi:10.1201/9781315151984
#'
#' @param model_fits An object of class modelFits, i.e. information about fitted models & corresponding model coefficients as well as the posterior distribution that was the basis for the model fitting
#' @param quantiles A vector of quantiles that should be evaluated
#' @param n_samples Number of samples that should be drawn as basis for the bootstrapped quantiles
#' @param doses A vector of doses for which a prediction should be performed. If NULL, the dose levels of the model_fits will be used. Default NULL.
#' @param probability_scale A boolean variable to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform predictions from the logit scale to the probability scale, which can be desirable for a binary outcome. Default `attr(model_fits, "probability_scale")`.
#'
#' @examples
#' posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
#'                        DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
#'                        DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2) ,
#'                        DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2) ,
#'                        DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
#' models         <- c("exponential", "linear")
#' dose_levels    <- c(0, 1, 2, 4, 8)
#' model_fits     <- getModelFits(models      = models,
#'                                posterior   = posterior_list,
#'                                dose_levels = dose_levels,
#'                                simple      = TRUE)
#'
#' bs_quantiles <- getBootstrapQuantiles(model_fits = model_fits,
#'                                       quantiles  = c(0.025, 0.5, 0.8, 0.975),
#'                                       n_samples  = 10, # speeding up example run time
#'                                       doses      = c(0, 6, 8))
#'                       
#' bs_quantiles
#' @return  A tibble with columns for model, dose, and bootstrapped samples
#'
#' @export
getBootstrapQuantiles <- function (

  model_fits,
  quantiles,
  n_samples         = 1e3,
  doses             = NULL,
  probability_scale = attr(model_fits, "probability_scale")

) {
  
  checkmate::assert_flag(probability_scale, null.ok = TRUE)
  if (is.null(probability_scale)) probability_scale <- FALSE
  
  ## R CMD --as-cran appeasement
  model <- dose <- sample_q <- sample_diff_q <- q_prob <- NULL

  bs_samples <- getBootstrapSamples(
    model_fits        = model_fits,
    n_samples         = n_samples,
    doses             = doses,
    probability_scale = probability_scale)

  quantile_probs <- sort(unique(quantiles))
    
  bs_quantiles <- bs_samples |>
    dplyr::group_by(model, dose) |>
    dplyr::summarize(
      sample_q      = list(stats::quantile(abs, probs = quantile_probs)),
      sample_diff_q = list(stats::quantile(diff, probs = quantile_probs)),
      .groups = "drop") |>
    tidyr::unnest_wider(sample_q, names_sep = "_") |>
    tidyr::unnest_wider(sample_diff_q, names_sep = "_") |>
    dplyr::rename_with(~ paste0("sample_", quantile_probs, "%"), dplyr::starts_with("sample_q_")) |>
    dplyr::rename_with(~ paste0("sample_diff_", quantile_probs, "%"), dplyr::starts_with("sample_diff_q_")) |>
    tidyr::pivot_longer(
      cols          = dplyr::matches("^(sample|sample_diff)_\\d+(\\.\\d+)?%$"),
      names_to      = c("sample_type", "q_prob"),
      names_pattern = "^(sample(?:_diff)?)_(.+)%$",
      values_to     = "q_val"
    ) |>
    dplyr::mutate(q_prob      = as.numeric(gsub("%", "", q_prob)),
                  sample_type = dplyr::case_when(
                    sample_type == "sample" ~ "abs",
                    sample_type == "sample_diff" ~ "diff"))
  
  attr(bs_quantiles, "direction")         <- attr(model_fits, "direction")
  attr(bs_quantiles, "probability_scale") <- probability_scale
  
  return (bs_quantiles)

}

#' @title getBootstrapSamples
#'
#' @description A function to return bootstrap samples from the fitted dose-response models.
#' Samples from the posterior distribution are drawn (via the RBesT function rmix()) and for every sample the simplified fitting step (see getModelFits() function) and a prediction is performed. 
#' These samples are returned by this function.   
#' This approach can be considered as the Bayesian equivalent of the frequentist bootstrap approach described in O'Quigley et al. (2017).
#' Instead of drawing n bootstrap samples from the sampling distribution of the trial dose-response estimates, here the samples are directly taken from the posterior distribution.
#' @references O'Quigley J, Iasonos A, Bornkamp B. 2017. Handbook of Methods for Designing, Monitoring, and Analyzing Dose-Finding Trials (1st ed.). Chapman and Hall/CRC. doi:10.1201/9781315151984
#' @param model_fits An object of class modelFits, i.e. information about fitted models & corresponding model coefficients as well as the posterior distribution that was the basis for the model fitting 
#' @param n_samples Number of samples that should be drawn
#' @param doses A vector of doses for which a prediction should be performed
#' @param probability_scale A boolean variable to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform predictions from the logit scale to the probability scale, which can be desirable for a binary outcome. Default `attr(model_fits, "probability_scale")`.
#'
#' @examples
#' posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
#'                        DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
#'                        DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2) ,  
#'                        DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2) ,
#'                        DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
#' models         <- c("exponential", "linear")
#' dose_levels    <- c(0, 1, 2, 4, 8)
#' model_fits     <- getModelFits(models      = models,
#'                                posterior   = posterior_list,
#'                                dose_levels = dose_levels,
#'                                simple      = TRUE)
#'                            
#' bs_samples <- getBootstrapSamples(model_fits = model_fits,
#'                                   n_samples  = 10, # speeding up example run time
#'                                   doses      = c(0, 6, 8))
#'                       
#' bs_samples
#' @return  A tibble with columns for sample_id, model, dose, sample, and sample_diff
#' 
#' @export
getBootstrapSamples <- function (
    
  model_fits,
  n_samples         = 1e3,
  doses             = NULL,
  probability_scale = attr(model_fits, "probability_scale")
  
) {
  
  checkmate::assert_flag(probability_scale, null.ok = TRUE)
  if (is.null(probability_scale)) probability_scale <- FALSE
  
  ## R CMD --as-cran appeasement
  name <- dose <- sample_type <- sample_id <- NULL
  
  mu_hat_samples <- sapply(attr(model_fits, "posterior"),
                           RBesT::rmix, n = n_samples)
  sd_hat         <- summary.postList(attr(model_fits, "posterior"))[, 2]
  
  dose_levels    <- model_fits[[1]]$dose_levels
  model_names    <- names(model_fits)
  
  if (is.null(doses)) {
    
    doses <- dose_levels
    
  }
  
  avg_fit <- "avgFit" %in% model_names
  
  if (avg_fit) {
    
    model_names <- setdiff(model_names, "avgFit")
    
  }
  
  ## pre-compute constant variables across iterations
  S         <- diag(sd_hat^2)
  bnds_list <- DoseFinding::defBnds(mD = max(dose_levels))[model_names]
  
  # predictions[samples, (dose1 model1, dose1 model2, ..., dose2 model1, ...)]
  preds <- t(optPar_apply(mu_hat_samples, 1, function (mu_hat) {
    
    preds_mu_hat <- sapply(model_names, function (model) {
      
      fit <- DoseFinding::fitMod(
        dose  = dose_levels,
        resp  = mu_hat,
        S     = S,
        model = model,
        type  = "general",
        bnds  = bnds_list[[model]])
      
      preds <- stats::predict(fit, doseSeq = doses, predType = "ls-means")
      attr(preds, "gAIC") <- DoseFinding::gAIC(fit)
      
      return (preds)
      
    }, simplify = FALSE)
    
    preds_mu_mat <- do.call(rbind, preds_mu_hat)
    
    if (avg_fit) {
      
      avg_fit_indx <- which.min(sapply(preds_mu_hat, attr, "gAIC"))
      preds_mu_mat <- rbind(avgFit = preds_mu_mat[avg_fit_indx, ], preds_mu_mat)
      
    }
    
    if (probability_scale) preds_mu_mat <- RBesT::inv_logit(preds_mu_mat)
    
    preds_mu_mat_adj <- preds_mu_mat - preds_mu_mat[, 1]
    
    # predictions[models, c(doses, adj_doses)]
    return (c(preds_mu_mat, preds_mu_mat_adj))
    
  }))
  
  if (avg_fit) {
    
    model_names <- c("avgFit", model_names)
    
  }
  
  bs_samples           <- as.data.frame(preds)
  colnames(bs_samples) <- c(paste0(model_names, "_",
                                   rep(doses, each = length(model_names))),
                            paste0(model_names, "_diff_",
                                   rep(doses, each = length(model_names))))
  
  bs_samples <- cbind(sample_id = seq_len(nrow(bs_samples)), bs_samples)
  
  bs_samples_long <- bs_samples |>
    tidyr::pivot_longer(
      cols          = -sample_id,
      names_to      = c("model", "diff", "dose"),
      names_pattern = "^(avgFit|linear|exponential|logistic|emax|sigEmax|quadratic|betaMod)(_diff)?_([0-9]+(?:\\.[0-9]+)?)$",
      values_to     = "sample") |>
    dplyr::mutate(
      diff = dplyr::case_when(
        diff == "" ~ "abs",
        diff == "_diff" ~ "diff"),
      dose = as.numeric(dose)) |>
    dplyr::rename(sample_type = diff) |>
    tidyr::pivot_wider(
      names_from  = sample_type,
      values_from = sample)
  
  attr(bs_samples_long, "probability_scale") <- probability_scale
  
  return (bs_samples_long)
  
}

# Test cases
test_that("test getBootstrapBands", {

  BMCP_result <- performBayesianMCP(
    posterior_list = posterior_list,
    contr           = contr_mat,
    crit_prob_adj   = crit_pval)

  model_shapes  <- c("linear", "emax", "exponential")

  # Option b) Making use of the complete posterior distribution
  fit <- getModelFits(
    models      = model_shapes,
    dose_levels = dose_levels,
    posterior   = posterior_list,
    simple      = FALSE)

  result <- bootstrap_quantiles <- BayesianMCPMod::getBootstrapQuantiles(
    model_fits = fit,
    quantiles  = c(0.025, 0.5, 0.975),
    doses      = c(0, 2.5, 4, 5, 7, 10),
    n_samples  = 6
  )

  expect_type(result, "list")

  result_2 <- bootstrap_quantiles <- BayesianMCPMod::getBootstrapQuantiles(
    model_fits = fit,
    quantiles  = c(0.025, 0.5, 0.975),
    doses      = c(0, 2.5, 4, 5, 7, 10),
    n_samples  = 6
  )

  expect_type(result_2, "list")
  
})

test_that("Bootstrap samples/quantiles: probability_scale=TRUE returns bounded values and stable quantile handling", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("DoseFinding")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  
  set.seed(12)
  
  dose_levels <- c(0, 1, 2, 4)
  probs <- c(0.10, 0.20, 0.30, 0.55)
  
  # Build a posterior directly from mu_hat/se_hat to keep test fast and stable
  mu_hat <- qlogis(probs)
  se_hat <- rep(0.35, length(dose_levels))
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = mu_hat[i], s = 1.5), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(
    prior_list        = prior_list,
    mu_hat            = mu_hat,
    S_hat             = diag(se_hat^2),
    probability_scale = TRUE
  )
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    emax   = c(0.5, 1.2),
    doses  = dose_levels,
    maxEff = 2
  )
  
  fits <- getModelFits(
    models            = mods,
    dose_levels       = dose_levels,
    posterior         = post,
    simple            = TRUE,
    avg_fit           = TRUE,
    probability_scale = TRUE
  )
  
  bs <- getBootstrapSamples(
    model_fits        = fits,
    n_samples         = 60,
    doses             = dose_levels,
    probability_scale = TRUE
  )
  
  expect_true(all(c("model", "dose", "sample_id", "abs", "diff") %in% names(bs)))
  expect_true(all(bs$abs >= 0 & bs$abs <= 1))
  # diff is probability difference vs control => in [-1,1]
  expect_true(all(bs$diff >= -1 & bs$diff <= 1))
  
  # Quantiles: duplicates + unsorted are handled via sort(unique())
  qs_in <- c(0.5, 0.025, 0.5, 0.975)
  bq <- getBootstrapQuantiles(
    model_fits        = fits,
    quantiles         = qs_in,
    n_samples         = 60,
    doses             = dose_levels,
    probability_scale = TRUE
  )
  
  expect_true(all(c("model", "dose", "q_prob", "q_val", "sample_type") %in% names(bq)))
  expect_equal(sort(unique(qs_in)), sort(unique(bq$q_prob)))
  expect_true(all(bq$sample_type %in% c("abs", "diff")))
  
  # Bounds by sample_type
  expect_true(all(bq$q_val[bq$sample_type == "abs"]  >= 0 & bq$q_val[bq$sample_type == "abs"] <= 1))
  expect_true(all(bq$q_val[bq$sample_type == "diff"] >= -1 & bq$q_val[bq$sample_type == "diff"] <= 1))
})

## Make parallel processing optional
useFuture <- function () isTRUE(requireNamespace("future.apply", quietly = TRUE))

optPar_lapply <- function (X, FUN, ...) {
  
  if (useFuture()) {
    
    return (future.apply::future_lapply(
      X, FUN,
      future.seed       = FALSE,
      future.scheduling = 1,
      future.packages   = c("DoseFinding", "RBesT", "nloptr"),
      ...))
    
  } else {
    
    return (lapply(X, FUN, ...))
    
  }
  
}

optPar_apply <- function (X, MARGIN, FUN, ...) {
  
  if (useFuture()) {
    
    return (future.apply::future_apply(
      X, MARGIN, FUN,
      future.seed       = FALSE, 
      future.scheduling = 1,
      future.packages   = c("DoseFinding", "RBesT"),
      ...))
    
  } else {
    
    return (apply(X, MARGIN, FUN, ...))
    
  }
  
}

padStrings <- function(strings) {
  
  max_length     <- max(nchar(strings))
  padded_strings <- sapply(strings, function(x) {
    
    paste0(x, strrep(" ", max_length - nchar(x)))
    
  })
  
  return (padded_strings)
  
}

printMatrixWithPrefix <- function (
    
  mat,
  prefix = "  "
  
) {
  
  row_names <- rownames(mat)
  col_names <- colnames(mat)
  
  if (!is.null(row_names)) {
    
    if (!is.null(col_names)) {
      
      cat(rep(" ", max(nchar(row_names))), " ", sep = "")
      
    }
    
    row_names <- padStrings(row_names)
    
  }
  
  print_width <- max(nchar(round(mat, options()$digits)),
                     nchar(col_names), na.rm = TRUE)
  
  if (!is.null(col_names)) {
    
    cat(prefix)
    cat(format(col_names, width = print_width, justify = "right"))
    cat("\n")
    
  }
  
  for (i in seq_len(nrow(mat))) {
    
    cat(prefix)
    
    if (!is.null(row_names)) {
      
      cat(row_names[i], " ", sep = "")
      
    }
    
    cat(format(mat[i, ], width = print_width), "\n")
    
  }
  
}

shortenModelNames <- function (model_names, pad_string = FALSE) {
  
  model_names_out <- model_names |>
    gsub("exponential", "exp", x = _) |>
    gsub("quadratic",  "quad", x = _) |>
    gsub("linear",      "lin", x = _) |>
    gsub("logistic",    "log", x = _) |>
    gsub("sigEmax",    "sigE", x = _) |>
    gsub("betaMod",   "betaM", x = _) |>
    gsub("quadratic",  "quad", x = _)
  
  if (pad_string) {
    
    model_names_out <- padStrings(model_names_out)
    
  }
  
  return (model_names_out)
  
}

## BayesianMCPMod -----------------------------------------

#' @export
print.BayesianMCPMod <- function (
    
  x,
  n_mods = 0L,
  ...
  
) {
  
  print(x$BayesianMCP, ...)
  
  if (any(!is.na(attr(x, "MED")))) {
    
    sign_indx <- as.logical(x$BayesianMCP[, "sign"])
    med_info  <- attr(x, "MED")
    
    cat("MED Assessment")
    
    if (any(sign_indx)) {
      
      probability_scale <- attr(x$Mod[[which(sign_indx)[1]]], "probability_scale")
      
      if (probability_scale) cat(" on Probability Scale")
      
    }
    
    cat("\n")
    
    cat("  Selection Method:           ", attr(x, "MEDSelection"), "\n")
    cat("  MED reached Freq:           ", attr(x, "MEDReachedFreq"), "\n")
    
    if (!any(sign_indx)) {
      
      cat("    No model shapes are significant\n")
      
    } else {
      
      # Frequency of Test not passed
      freq_not_passed <- 1 - mean(sign_indx)
      
      # Frequency of MED not reached in passed tests
      freq_not_reached <- mean(as.numeric(!med_info[, "med_reached"]) * as.numeric(sign_indx))
      
      # MED Dose Frequencies
      dose_levels   <- x$Mod[[which(sign_indx)[1]]][[1]]$dose_levels[-1]
      med_table     <- table(factor(med_info[, "med"], levels = dose_levels), useNA = "no")
      
      med_mat <- matrix(data  = c(as.numeric(names(med_table)),
                                  as.vector(med_table) / nrow(med_info)),
                        nrow  = 2,
                        byrow = TRUE)
      rownames(med_mat) <- c("Dose Level:", "MED Freq:")
      
      printMatrixWithPrefix(med_mat, prefix = "   ")
      
      cat("  MED not reached Freq:       ", freq_not_reached, "\n")
      cat("  No success in MCP step Freq:", freq_not_passed, "\n")
      
    }
    
  } 
  
  if (length(x$Mod) == 1) {

    print(x$Mod[[1]], ...)

  } else {
    
    for (m in seq_len(n_mods)) {
      
      print(x$Mod[[m]], ...)
      
    }
    
  }
  
  invisible(x)
  
}

## BayesianMCP --------------------------------------------

#' @export
print.BayesianMCP <- function(x, ...) {
  
  n_sim <- nrow(x)
  
  cat("Bayesian Multiple Comparison Procedure\n")
  
  if (n_sim == 1L) {
    
    cat("  Significant:                  ", x[1, "sign"], "\n")
    cat("  Critical Probability:         ", x[1, "crit_prob_adj"], "\n")
    cat("  Maximum Posterior Probability:", x[1, "max_post_prob"], "\n")
    
    cat("Posterior Probabilities for Model Shapes\n")
    
    model_probs <- x[1, grep("^post_probs\\.", colnames(x))]
    model_mat   <- matrix(
      data     = model_probs,
      nrow     = 1,
      dimnames = list(
        "Posterior Prob",
        shortenModelNames(gsub(
          pattern     = "post_probs\\.",
          replacement = "",
          x           = names(model_probs)))))
    
    printMatrixWithPrefix(
      rbind(model_mat, Significant = x[, -c(1, 2, 3)] > attr(x, "critProbAdj")))
    
    if (any(!is.na(attr(x, "essAvg")))) {
      
      cat("Average Posterior ESS\n")
      
      ess_vec       <- as.vector(attr(x, "essAvg"))
      ess_vec_names <- names(attr(x, "essAvg"))
      
      print_width <- max(nchar(ess_vec), nchar(ess_vec_names), na.rm = TRUE)
      
      cat("  Dose Level:  ", format(ess_vec_names, width = print_width, justify = "right"), "\n")
      cat("  Avg Post ESS:", format(ess_vec, width = print_width), "\n")
      
    }
    
  } else {
    
    model_successes <- getModelSuccesses(x)
    
    cat("  Estimated Success Rate:", attr(x, "successRate"), "\n")
    cat("  N Simulations:         ", n_sim, "\n")
    
    m_succ       <- as.vector(model_successes)
    m_succ_names <- shortenModelNames(names(model_successes))
    
    print_width  <- max(nchar(m_succ), nchar(m_succ_names), na.rm = TRUE)
    
    cat("   Model Shape:      ", format(m_succ_names, width = print_width, justify = "right"), "\n")
    cat("   Significance Freq:", format(m_succ, width = print_width), "\n")
    
  }
  
  invisible(x)
  
}


## ModelFits ----------------------------------------------

#' @title predict.modelFits
#' @description This function performs model predictions based on the provided
#' model and dose specifications
#'
#' @param object A modelFits object containing information about the fitted
#' model coefficients
#' @param doses A vector specifying the doses for which a prediction should be
#' done
#' @param probability_scale A boolean variable to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform predictions from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.
#' @param ... Currently without function
#'
#' @examples
#' posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
#'                        DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
#'                        DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2) ,
#'                        DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2) ,
#'                        DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
#' models         <- c("emax", "exponential", "sigEmax", "linear", "betaMod")
#' dose_levels    <- c(0, 1, 2, 4, 8)
#' fit            <- getModelFits(models      = models,
#'                                posterior   = posterior_list,
#'                                dose_levels = dose_levels)
#'
#' predict(fit, doses = c(0, 1, 3, 4, 6, 8))
#'
#' @return a list with the model predictions for the specified models and doses
#'
#' @export
predict.modelFits <- function (
    
  object,
  doses             = NULL,
  probability_scale = attr(object, "probability_scale"),
  ...
  
) {
  
  checkmate::assert_double(doses, lower = 0, any.missing = FALSE, unique = TRUE,
                           sorted = TRUE, finite = TRUE, null.ok = TRUE)
  checkmate::assert_flag(probability_scale)
  
  model_fits  <- object
  model_names <- names(model_fits)
  
  warning_displayed <- FALSE
  withCallingHandlers(
  
    expr = {
      
      predictions <- lapply(model_fits[model_names != "avgFit"],
                            predictModelFit, doses = doses)
      
      if ("avgFit" %in% model_names) {
        
        preds_avg_fit <- predictAvgFit(model_fits, doses = doses) # predictAvgFit calls predict.modelFits !!
        predictions   <- c(list(avgFit = preds_avg_fit), predictions)
        
      }
      
    },
    
    warning = function(w) {
      
      if (grepl("The specified range exceeds the bounds of the original dose range.", conditionMessage(w))) {
        
        if (warning_displayed) {
          
          invokeRestart("muffleWarning")
          
        } else {
          
          warning_displayed <<- TRUE
          
        }
        
      }
      
    }
      
  )
  
  if (probability_scale) {
    
    predictions <- lapply(predictions, RBesT::inv_logit)
    
  }
  
  attr(predictions, "doses") <- doses
  
  return (predictions)
  
}

#' @export
print.modelFits <- function (
    
  x,
  n_digits          = 1,
  probability_scale = attr(x, "probability_scale"),
  ...
  
) {
  
  checkmate::assert_integerish(n_digits, lower = 0)
  checkmate::assert_flag(probability_scale)
  
  if (probability_scale & n_digits < 2) n_digits <- 2
  
  ## Model Coefficients
  
  model_names <- shortenModelNames(names(x), pad_string = TRUE)
  
  cat("Model Coefficients\n")
  
  for (i in seq_along(x)) {
    
    if (x[[i]]$model != "avgFit") {
      
      coeff_values <- x[[i]]$coeff
      coeff_names  <- names(coeff_values)
      
      cat(" ", model_names[i],
          paste(coeff_names, round(coeff_values, n_digits),
                sep      = " = ",
                collapse = ", "), "\n",
          sep = " ")
      
    }
    
  }
  
  ## Dose Levels
  
  dose_levels <- x[[1]]$dose_levels
  dose_names  <- names(attr(x, "posterior"))
  
  cat("Dose Levels\n", "", # "" is required for proper indentation
      paste(dose_names, round(dose_levels, n_digits),
            sep      = " = ",
            collapse = ", "), "\n")
  
  ## Predictions, Maximum Effect, gAIC & avgFit Model Weights
  
  cat("Predictions")
  
  if (probability_scale) {
    
    cat(" on Prob. Scale")
    
  }
  
  cat(", Maximum Effect, gAIC")
  
  predictions           <- t(sapply(x, function (y) y$pred_values))
  colnames(predictions) <- dose_names
  
  out_table <- data.frame(predictions,
                          mEff = sapply(x, function (y) y$max_effect),
                          gAIC = sapply(x, function (y) y$gAIC),
                          w    = sapply(x, function (y) y$model_weight))
  
  if (!is.null(x[[1]]$significant)) {
    
    model_sig      <- TRUE
    out_table$Sign <- sapply(x, function (y) y$significant)
    
  } else {
    
    model_sig <- FALSE
    
  }
  
  if (model_sig) {
    
    cat(", avgFit Model Weights & Significance\n")
    
  } else {
    
    cat(" & avgFit Model Weights\n")
    
  }
  
  out_table <- apply(as.matrix(out_table), 2, round, digits = n_digits)
  rownames(out_table) <- shortenModelNames(rownames(out_table))
  
  printMatrixWithPrefix(out_table)
  
  invisible(x)
  
}

## plot.ModelFits()
## see file R/plot.R

## postList -----------------------------------------------

#' @export
summary.postList <- function (
    
  object,
  probability_scale = FALSE,
  ...
  
) {
  
  checkmate::assert_flag(probability_scale)
  
  summary_list        <- lapply(object, summary, ...)
  names(summary_list) <- names(object)
  summary_tab         <- do.call(rbind, summary_list)
  
  ## Transformations for probability scale, X ~ N(mu, sigma^2), p ~ expit(X) = inv_logit(X)
  if (probability_scale) {
    
    ## quantiles - simply use expit(), i.e., inv_logit()
    q_indices                <- setdiff(colnames(summary_tab), c("mean", "sd"))
    summary_tab[, q_indices] <- apply(summary_tab[, q_indices], 2, RBesT::inv_logit)
    
    ## means and standard deviations - numerical integration
    getNormal2LogisticNormalMoments <- function (mu, sigma) {
      
      # integrand for the mean E[p]
      integrand_mean <- function (x) {
        
        p <- RBesT::inv_logit(x)
        
        return (p * stats::dnorm(x, mean = mu, sd = sigma))
        
      }
      
      # integrand for second moment E[p^2]
      integrand_second <- function (x) {
        
        p <- RBesT::inv_logit(x)
        
        return (p^2 * stats::dnorm(x, mean = mu, sd = sigma))
        
      }
      
      # numerical integration
      m1 <- stats::integrate(integrand_mean,   -Inf, Inf, rel.tol = 1e-9)$value
      m2 <- stats::integrate(integrand_second, -Inf, Inf, rel.tol = 1e-9)$value
      
      # standard deviation
      sd_p <- sqrt(m2 - m1^2)
      
      return (c(mean = m1, sd = sd_p))
      
    }
    
    summary_tab[, c("mean", "sd")] <- t(mapply(getNormal2LogisticNormalMoments,
                                               mu    = summary_tab[, "mean"],
                                               sigma = summary_tab[, "sd"]))
    
  }
  
  return (summary_tab)
  
}

#' @export
print.postList <- function (
    
  x,
  ...
  
) {
  
  getMaxDiff <- function (
    
    medians
    
  ) {
    
    diffs <- medians - medians[1]
    
    max_diff       <- max(diffs)
    max_diff_level <- which.max(diffs) - 1
    
    out <- c(max_diff, max_diff_level)
    names(out) <- c("max_diff", "DG")
    
    return (out)
    
  }
  
  summary_tab <- summary.postList(x)
  
  names(x) <- rownames(summary_tab)
  class(x) <- NULL
  
  ## removes the covariance matrices from the print out
  attr(x, "posteriorInfo") <- NULL
  
  list_out <- list(summary_tab, getMaxDiff(summary_tab[, 4]), x)
  
  names(list_out) <- c("Summary of Posterior Distributions",
                       "Maximum Difference to Control and Dose Group",
                       "Posterior Distributions")
  
  print(list_out, ...)
  cat("Note: For a more detailed posterior output including\n",
      "     covariance matricies across mixture components,\n",
      "     please use attr(x, 'posteriorInfo').")
  
}

test_that("print.BayesianMCPMod works as intented", {
  expect_error(print.BayesianMCPMod())

})


test_that("print.BayesianMCP works as intented", {
  expect_error(print.BayesianMCP())

  data <- simulateData(
    n_patients  = n_patients,
    dose_levels = dose_levels,
    sd          = sd,
    mods        = mods,
    n_sim       = n_sim
  )

  posterior_list <- getPosterior(
    data = getModelData(data, names(mods)[1]),
    prior_list = prior_list
  )

  contr_mat = getContr(
    mods = mods,
    dose_levels = dose_levels,
    dose_weights = n_patients,
    prior_list = prior_list
  )

  crit_pval = getCritProb(
    mods = mods,
    dose_levels = dose_levels,
    dose_weights = n_patients,
    alpha_crit_val = alpha_crit_val
  )

  b_mcp <- performBayesianMCP(
    posterior_list = posterior_list,
    contr = contr_mat,
    crit_prob_adj = crit_pval
  )

  expect_s3_class(b_mcp, "BayesianMCP")
  expect_no_error(print(b_mcp))
  expect_type(print(b_mcp), "double")

  mods <- DoseFinding::Mods(
  linear = NULL,
  emax = c(0.5, 1.2),
  exponential = 2,
  doses = c(0, 0.5, 2, 4, 8)
)
dose_levels <- c(0, 0.5, 2, 4, 8)
sd_posterior <- c(2.8, 3, 2.5, 3.5, 4)
contr_mat <- getContr(
  mods          = mods,
  dose_levels   = dose_levels,
  cov_posterior = diag(sd_posterior)
)
critVal <- getCritProb(
  mods           = mods,
  dose_weights   = c(50, 50, 50, 50, 50), # reflecting the planned sample size
  dose_levels    = dose_levels,
  alpha_crit_val = 0.6
)
prior_list <- list(
  Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 5), sigma = 2),
  DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
  DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2),
  DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2),
  DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2)
)
mu <- c(0, 1, 1.5, 2, 2.5)
S_hat <- diag(c(5, 4, 6, 7, 8))
posterior_list <- getPosterior(
  prior_list = prior_list,
  mu_hat = mu,
  S_hat = S_hat
)

x <- performBayesianMCPMod(
  posterior_list = posterior_list,
  contr = contr_mat,
  crit_prob_adj = critVal,
  simple = FALSE,
  delta = 1
)

expect_s3_class(x, "BayesianMCPMod")
expect_no_error(print(x))
expect_type(print(x), "list")


})

test_that("predict.ModelFits works as intented", {
  expect_error(predict.ModelFits())

})

test_that("s3 postList functions work as intented", {

  dose_levels <- c(0, 2.5, 5, 10)

  prior_list <- list(
    Ctrl = RBesT::mixnorm(comp1 = c(w = 0.4, m = 0, s = 5), comp2 = c(w = 0.6, m = 2, s = 3), sigma = 2),
    DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
    DG_2 = RBesT::mixnorm(comp1 = c(w = 0.3, m = 1.2, s = 11), comp2 = c(w = 0.7, m = 1, s = 4), sigma = 2),
    DG_3 = RBesT::mixnorm(comp1 = c(w = 0.2, m = 1.3, s = 11), comp2 = c(w = 0.5, m = 1, s = 4), comp3 = c(w = 0.3, m = 7, s = 1), sigma = 2),
    DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2)
  )

  mu_hat <- c(0, 1, 1.5, 2, 2.5)
  S_hat <- diag(c(5, 4, 6, 7, 8)^2)

  post_test_list <- getPosterior(
    prior_list = prior_list,
    mu_hat     = mu_hat,
    S_hat      = S_hat
  )
  expect_error(summary.postList())
  expect_type(summary.postList(post_test_list), "double")
  expect_error(print.postList())
  expect_no_error(print(post_test_list))
  expect_type(post_test_list, "list")
})

test_that("test modelFits s3 methods", {

  model_shapes <- colnames(contr_mat$contMat)
  dose_levels  <- as.numeric(rownames(contr_mat$contMat))

  model_fits  <- getModelFits(
    models      = model_shapes,
    dose_levels = dose_levels,
    posterior   = posterior_list,
    simple      = simple)

  pred <- predict(model_fits)
  pred_dosage <- predict(model_fits, doses = dose_levels)

  expect_type(pred, "list")
  expect_true(is.null(attr(pred, "doses")))
  expect_identical(attr(pred_dosage, "doses"), dose_levels)
  expect_type(print(model_fits), "list")
})

#' @title simulateData
#' 
#' @description
#' Function to simulate patient level data for a normally distributed endpoint
#' @param n_patients Vector containing number of patients as a numerical
#' value per dose-group.
#' @param dose_levels Vector containing the different dosage levels.
#' @param sd Standard deviation on patient level. Can be NULL if `probability_scale` is TRUE. Default NULL.
#' @param mods An object of class "Mods" as specified in the DoseFinding package. Can be NULL if ´dr_means´ is not NULL. Default NULL.
#' @param n_sim Number of simulations to be performed,
#' Default is 1000
#' @param true_model A character for model name, e.g. "emax". Assumed true underlying model.
#' If NULL, all dose-response models included in the mods input parameter will be used. Default NULL.
#' @param dr_means an optional vector, with information about assumed effects per dose group. Default NULL.
#' @param probability_scale A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform calculations from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.
#' 
#' @examples
#' models <- DoseFinding::Mods(linear      = NULL,
#'                             linlog      = NULL,
#'                             emax        = c(0.5, 1.2),
#'                             exponential = 2, 
#'                             doses       = c(0, 0.5, 2,4, 8),
#'                             maxEff      = 6)
#' dose_levels <- c(0, 0.5, 2, 4, 8)
#' sd          <- 12
#' n_patients  <- c(40, 60, 60, 60, 60)
#' 
#' sim_data <- simulateData(n_patients  = n_patients,
#'                          dose_levels = dose_levels,
#'                          sd          = sd,
#'                          mods        = models)
#' 
#' head(sim_data)
#' 
#' # custom response "model" shape
#' custom_dose_response <- c(1, 2, 3, 4, 5)
#' sim_data_custom_dr   <- simulateData(n_patients  = n_patients,
#'                                      dose_levels = dose_levels,
#'                                      sd          = sd,
#'                                      dr_means    = custom_dose_response)
#' 
#' head(sim_data_custom_dr)
#'
#' @return A list object, containing patient level simulated data for all assumed true models.
#' Also providing information about simulation iteration, patient number as well as dosage levels.
#' 
#' @export
simulateData <- function(
    
  n_patients,
  dose_levels,
  sd                = NULL,
  mods              = NULL,
  n_sim             = 1e3,
  true_model        = NULL,
  dr_means          = NULL,
  probability_scale = FALSE
  
) {
  
  checkmate::check_vector(n_patients, any.missing = FALSE, len = length(dose_levels))
  checkmate::check_double(dose_levels, lower = 0, any.missing = FALSE, len = length(n_patients))
  checkmate::check_double(sd, len = 1, null.ok = TRUE, lower = 0, upper = Inf)
  checkmate::check_class(mods, classes = "Mods", null.ok = TRUE)
  checkmate::check_numeric(n_sim, lower = 0, upper = Inf, len = 1)
  checkmate::check_string(true_model, null.ok = TRUE)
  checkmate::assert_flag(probability_scale)
  
  if (!probability_scale) stopifnot("Must provide 'sd' argument for simulation." = !is.null(sd))

  # stopifnot("Either 'dr_means' or 'mods' must be NULL." =
  #             is.null(dr_means) & !is.null(mods) |
  #             !is.null(dr_means) & is.null(mods))
  
  if (!probability_scale) stopifnot(!is.null(sd))
  
  if (!is.null(true_model)) stopifnot(!is.null(mods))
  
  if (!is.null(true_model)) {
    
    n_sim            <- 1
    mods_attr        <- attributes(mods)
    mods_attr$names  <- true_model
    mods             <- mods[true_model]
    attributes(mods) <- mods_attr
    
  }
  
  sim_info <- data.frame(
    simulation = rep(seq_len(n_sim), each = sum(n_patients)),
    ptno       = rep(seq_len(sum(n_patients)), times = n_sim),
    dose       = rep(rep(dose_levels, times = n_patients), times = n_sim))
  
  if (is.null(dr_means)) {
    
    model_responses <- DoseFinding::getResp(mods, sim_info$dose)
    
  } else {
    
    stopifnot(identical(length(dose_levels), length(dr_means)))
    
    model_responses <- matrix(
      data     = rep(unlist(mapply(rep, dr_means, n_patients)), n_sim),
      ncol     = 1L,
      dimnames = list(NULL, "dr_response"))
    
  }
  
  if (probability_scale) {
    
    # simulate responders from binomial distribution
    model_response_rates <- RBesT::inv_logit(model_responses)
    simulated_responses  <- matrix(
      stats::rbinom(length(model_response_rates), size = 1L, prob = as.vector(model_response_rates)),
      nrow = nrow(model_response_rates), ncol = ncol(model_response_rates))
    colnames(simulated_responses) <- colnames(model_response_rates)
    
  } else {
    
    # purposefully simulating random noise per patient, not per patient and model shape
    random_noise        <- stats::rnorm(nrow(sim_info), mean = 0, sd = sd) 
    simulated_responses <- model_responses + random_noise
    
  }
  
  sim_data <- cbind(sim_info, simulated_responses)
  
  rownames(sim_data)                  <- NULL
  attr(sim_data, "probability_scale") <- probability_scale
  
  return (sim_data)
  
}

getModelData <- function (
    
  sim_data,
  model_name
  
) {
  
  model_data <- sim_data[, c("simulation", "dose", model_name)]
  colnames(model_data)[3] <- "response"
  
  attr(model_data, "probability_scale") <- attr(sim_data, "probability_scale")
  
  return (model_data)
  
}

# Test for getModelData
test_that("getModelData returns a data frame with correct columns", {

  mock_sim_data <- data.frame(
    simulation = 1:5,
    dose = rnorm(5),
    model1 = rnorm(5),
    model2 = rnorm(5)
  )
  mock_model_name <- "model1"

  result <- getModelData(sim_data = mock_sim_data, model_name = mock_model_name)

  # Assert that the result is a data frame
  expect_true(is.data.frame(result))
  # Assert that the data frame has the correct number of columns
  expect_equal(ncol(result), 3)
  # Assert that the data frame has the correct column names
  expect_equal(colnames(result), c("simulation", "dose", "response"))
})

# Test for getModelData
test_that("simulateData works as expected", {

  dataset     <- dplyr::filter(testdata, bname == "BRINTELLIX")
  histcontrol <- dplyr::filter(dataset, dose == 0, primtime == 8, indication == "MAJOR DEPRESSIVE DISORDER")

  hist_data <- data.frame(
    trial = histcontrol$nctno,
    est   = histcontrol$rslt,
    se    = histcontrol$se,
    sd    = histcontrol$sd,
    n     = histcontrol$sampsize)

  sd_tot <- with(hist_data, sum(sd * n) / sum(n))

  dose_levels <- c(0, 2.5, 5, 10, 20)

  prior_list  <- getPriorList(
    hist_data     = hist_data,
    dose_levels   = dose_levels,
    robust_weight = 0.3)


  prior_list  <- list(
    Ctr  = RBesT::mixnorm(
      comp1  = c(w = 0.446213, m = -12.774661, s = 1.393130),
      comp1  = c(w = 0.253787, m = 3.148116,   s = 3.148116),
      robust = c(w = 0.3,      m = 9.425139,   s = 9.425139),
      sigma = sd_tot),
    DG_1 = RBesT::mixnorm(
      comp1 = c(w = 1, m = -12.816875, n = 1),
      sigma = sd_tot,
      param = "mn"),
    DG_2 = RBesT::mixnorm(
      comp1 = c(w = 1, m = -12.816875, n = 1),
      sigma = sd_tot,
      param = "mn"),
    DG_3 = RBesT::mixnorm(
      comp1 = c(w = 1, m = -12.816875, n = 1),
      sigma = sd_tot,
      param = "mn"),
    DG_4 = RBesT::mixnorm(
      comp1 = c(w = 1, m = -12.816875, n = 1),
      sigma = sd_tot,
      param = "mn")
  )

  exp     <- DoseFinding::guesst(
    d     = 5,
    p     = c(0.2),
    model = "exponential",
    Maxd  = max(dose_levels))

  emax    <- DoseFinding::guesst(
    d     = 2.5,
    p     = c(0.9),
    model = "emax")

  sigemax <- DoseFinding::guesst(
    d     = c(2.5, 5),
    p     = c(0.1, 0.6),
    model = "sigEmax")

  sigemax2 <- DoseFinding::guesst(
    d     = c(2, 4),
    p     = c(0.3, 0.8),
    model = "sigEmax")

  mods <- DoseFinding::Mods(
    linear      = NULL,
    emax        = emax,
    exponential = exp,
    sigEmax     = rbind(sigemax, sigemax2),
    doses       = dose_levels,
    maxEff      = -3,
    placEff     = -12.8)

  n_patients <- c(60, 80, 80, 80, 80)

  sim_data_1 <- simulateData(
    n_patients  = n_patients,
    dose_levels = dose_levels,
    sd          = sd_tot,
    mods        = mods,
    n_sim       = 10)

  expect_type(sim_data_1, "list")

  sim_data_2 <- simulateData(
    n_patients  = n_patients,
    dose_levels = dose_levels,
    sd          = sd_tot,
    mods        = mods,
    n_sim       = 10,
    true_model = "emax")

  expect_type(sim_data_2, "list")

  sim_data_3 <- simulateData(
    n_patients  = n_patients,
    dose_levels = dose_levels,
    sd          = sd_tot,
    mods        = mods,
    n_sim       = 10,
    dr_means    = c(0.2, 0.3, 0.4, 0.5, 0.6))

  expect_type(sim_data_3, "list")

})

test_that("simulateData: binary mode produces 0/1 outcomes and stores attribute", {
  skip_if_not_installed("DoseFinding")
  skip_if_not_installed("RBesT")
  
  set.seed(1)
  
  dose_levels <- c(0, 1, 2, 4)
  n_patients  <- c(30, 30, 30, 30)
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    emax   = c(0.5, 1.2),
    doses  = dose_levels,
    maxEff = 2
  )
  
  sim <- simulateData(
    n_patients        = n_patients,
    dose_levels       = dose_levels,
    sd                = NULL,
    mods              = mods,
    n_sim             = 3,
    probability_scale = TRUE
  )
  
  expect_true(is.data.frame(sim))
  expect_true(all(c("simulation", "ptno", "dose") %in% names(sim)))
  expect_true(attr(sim, "probability_scale"))
  
  # All response columns should be 0/1
  resp_cols <- setdiff(names(sim), c("simulation", "ptno", "dose"))
  expect_true(length(resp_cols) >= 1)
  
  for (cn in resp_cols) {
    expect_true(all(sim[[cn]] %in% c(0, 1)))
  }
  
  # Must error if continuous mode but sd is missing
  expect_error(
    simulateData(
      n_patients        = n_patients,
      dose_levels       = dose_levels,
      sd                = NULL,
      mods              = mods,
      n_sim             = 1,
      probability_scale = FALSE
    ),
    regexp = "Must provide 'sd'|sd"
  )
})

test_that("simulateData: binary mode works with custom dr_means and sd=NULL", {
  skip_if_not_installed("RBesT")
  
  set.seed(2)
  
  dose_levels <- c(0, 1, 2, 4)
  n_patients  <- c(25, 25, 25, 25)
  
  # dr_means are on the *logit* scale (inv_logit used internally)
  dr_means_logit <- c(-2, -1, 0, 1)
  
  sim <- simulateData(
    n_patients        = n_patients,
    dose_levels       = dose_levels,
    sd                = NULL,
    mods              = NULL,
    n_sim             = 2,
    dr_means          = dr_means_logit,
    probability_scale = TRUE
  )
  
  resp_cols <- setdiff(names(sim), c("simulation", "ptno", "dose"))
  expect_equal(length(resp_cols), 1L)
  expect_true(all(sim[[resp_cols]] %in% c(0, 1)))
})

test_that("Test plot.modelFits with different model_fits input", {
  dataset <- dplyr::filter(testdata, bname == "VIAGRA")
  histcontrol <- dplyr::filter(dataset, dose == 0, primtime == 12, indication == "ERECTILE DYSFUNCTION")

  ## Create MAP Prior
  hist_data <- data.frame(
    trial = c(1, 2, 3, 4),
    est = histcontrol$rslt,
    se = histcontrol$se,
    sd = histcontrol$sd,
    n = histcontrol$sampsize
  )

  sd_tot <- with(hist_data, sum(sd * n) / sum(n))


  gmap <- RBesT::gMAP(
    formula    = cbind(est, se) ~ 1 | trial,
    weights    = hist_data$n,
    data       = hist_data,
    family     = gaussian,
    beta.prior = cbind(0, 100 * sd_tot),
    tau.dist   = "HalfNormal",
    tau.prior  = cbind(0, sd_tot / 4)
  )

  prior_ctr <- RBesT::robustify(
    priormix = RBesT::automixfit(gmap),
    weight   = 0.5,
    mean     = 1.4,
    sigma    = sd_tot
  )

  # RBesT::ess(prior_ctr)

  ## derive prior for treatment
  ## weak informative using same parameters as for robustify component
  prior_trt <- RBesT::mixnorm(
    comp1 = c(w = 1, m = 1.4, n = 1),
    sigma = sd_tot,
    param = "mn"
  )
  dose_levels <- c(0, 50, 100, 200)
  ## combine priors in list
  prior_list <- c(list(prior_ctr), rep(list(prior_trt), times = length(dose_levels[-1])))
  names(prior_list) <- c("Ctrl", paste0("DG_", seq_along(dose_levels[-1])))

  # Pre-Specification (B)MCPMod

  ## candidate models for MCPMod
  # linear function - no guestimates needed
  exp <- DoseFinding::guesst(
    d = 50,
    p = c(0.2),
    model = "exponential",
    Maxd = max(dose_levels)
  )
  emax <- DoseFinding::guesst(
    d = 100,
    p = c(0.9),
    model = "emax"
  )


  mods <- DoseFinding::Mods(
    linear      = NULL,
    emax        = emax,
    exponential = exp,
    doses       = dose_levels,
    maxEff      = 10,
    placEff     = 1.4
  )

  # Simulation of new trial
  ## Note: This part will be simplified and direct results from one trial will be used
  mods_sim <- DoseFinding::Mods(
    emax        = emax,
    doses       = dose_levels,
    maxEff      = 12,
    placEff     = 1.4
  )

  n_patients <- c(50, 50, 50, 50)
  data <- simulateData(
    n_patients  = n_patients,
    dose_levels = dose_levels,
    sd          = sd_tot,
    mods        = mods_sim,
    n_sim       = 1
  )

  data_emax <- data[, c("simulation", "dose", "emax")]
  names(data_emax)[3] <- "response"

  posterior_emax <- getPosterior(
    data       = data_emax,
    prior_list = prior_list
  )

  # Evaluation of Bayesian MCPMod

  contr_mat <- DoseFinding::optContr(
    models = mods,
    doses  = dose_levels,
    w      = n_patients
  )
  ## Calculation of critical value can be done with critVal function
  crit_val_equal <- DoseFinding:::critVal(contr_mat$corMat, alpha = 0.05, df = 0, alternative = "one.sided")
  crit_pval <- stats::pnorm(crit_val_equal)

  ess_prior <- round(unlist(lapply(prior_list, RBesT::ess)))

  ### Evaluation of Bayesian MCPMod
  contr_mat_prior <- DoseFinding::optContr(
    models = mods,
    doses  = dose_levels,
    w      = n_patients + ess_prior
  )

  BMCP_result <- performBayesianMCP(
    posterior_list = list(posterior_emax),
    contr          = contr_mat_prior,
    crit_prob      = crit_pval
  )

  # Model fit
  # This part is currently not working
  post_observed <- posterior_emax
  model_shapes <- c("linear", "emax", "exponential")


  # Option a) Simplified approach by using frequentist idea
  fit_simple <- getModelFits(
    models      = model_shapes,
    dose_levels = dose_levels,
    posterior   = post_observed,
    simple      = TRUE
  )

  # Option b) Making use of the complete posterior distribution
  fit <- getModelFits(
    models      = model_shapes,
    dose_levels = dose_levels,
    posterior   = post_observed,
    simple      = FALSE
  )

  # Test with default parameters and more models
  plot1 <- plot.modelFits(fit)
  expect_s3_class(plot1, "ggplot")

  # Test with cr_intv = TRUE and more models
  plot2 <- plot.modelFits(fit, cr_intv = TRUE)
  expect_s3_class(plot2, "ggplot")

  # Test with gAIC = FALSE and more models
  plot3 <- plot.modelFits(fit, gAIC = FALSE)
  expect_s3_class(plot3, "ggplot")

  # Test with avg_fit = FALSE and more models
  plot4 <- plot.modelFits(fit, avg_fit = FALSE)
  expect_s3_class(plot4, "ggplot")

  # Test with all non-default parameters and more models
  plot5 <- plot.modelFits(fit, cr_intv = TRUE, gAIC = FALSE, avg_fit = FALSE)
  expect_s3_class(plot5, "ggplot")

  # Test with all non-default parameters and more models
  plot6 <- plot.modelFits(fit, cr_intv = TRUE, cr_bands = TRUE, gAIC = FALSE, avg_fit = FALSE)
  expect_s3_class(plot6, "ggplot")
  # Test with all non-default parameters and more models
  plot7 <- plot.modelFits(fit, cr_intv = TRUE, cr_bands = TRUE, alpha_CrB = c(0.025, 0.5), gAIC = FALSE, avg_fit = FALSE)
  expect_s3_class(plot7, "ggplot")
})

#' @title getModelFits
#'
#' @description Fits dose-response curves for the specified dose-response models, based on the posterior distributions.
#' For the simplified fit, multivariate normal distributions will be approximated and reduced by one-dimensional normal distributions.
#' For the default case, the Nelder-Mead algorithm is used.
#' In detail, for both approaches the mean vector \eqn{\theta^{Y}} and the covariance \eqn{\Sigma} of the (mixture) posterior distributions and the corresponding posterior weights \eqn{\tilde{\omega}_{l}} for \eqn{l \in {1,...,L}} are used as basis
#' For the full fit a GLS estimator is used to minimize the following expression for the respective dose-response models \eqn{m}
#' \deqn{ \hat{\theta}_{m}=\text{arg min}_{\theta_{m}} \sum_{l=1}^{L} \tilde{\omega}_{l}(\theta_{l_{i}}^{Y}-f(dose_{i},\hat{\theta}_{m}))'\Sigma_{l}^{-1}(\theta_{l_{i}}^{Y}-f(dose_{i},\hat{\theta}_{m}))}
#' Therefore the function nloptr of the nloptr package is utilized.
#' In the simplified case \eqn{L=1}, as the dimension of the posterior is reduced to 1 first.
#' The generalized AIC values are calculated via the formula
#' \deqn{gAIC_{m} = \sum_{l=1}^{L} \tilde{\omega}_{l} \sum_{i=0}^{K} \frac{1}{\Sigma_{l_{i,i}}} (\theta_{l_i}^Y - f(dose_{i},\hat{\theta}_{m}))^2 + 2p }
#' where \eqn{p} denotes the number of estimated parameters and \eqn{K} the number of active dose levels.
#' Here as well for the simplified case the formula reduces to one summand as \eqn{L=1}.
#' Corresponding gAIC based weights for model \eqn{M} are calculated as outlined in Schorning et al. (2016)
#' \deqn{
#' \Omega_I (M) = \frac{\exp(-0.5 gAIC_{M})}{\sum_{m=1}^{Q} \exp(-0.5 gAIC_{m})}
#' }
#' where \eqn{Q} denotes the number of models included in the averaging procedure.
#' @references Schorning K, Bornkamp B, Bretz F, Dette H. 2016. Model selection versus model averaging in dose finding studies. Stat Med; 35; 4021-4040.
#' @param models A Mods object as created with `DoseFinding::Mods()` or a vector
#' of model names for which a fit will be performed.
#' Implemented model shapes are `"linear"`, `"exponential"`, `"logistic"`,
#' `"emax"`, `"sigEmax"`, `"quadratic"`, and `"betaMod"`.
#' @param dose_levels A vector containing the different dosage levels.
#' @param posterior A getPosterior object, containing the (multivariate) posterior distribution per dosage level.
#' @param avg_fit Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.
#' @param simple Boolean variable, defining whether simplified fit will be applied. Default FALSE.
#' @param probability_scale A boolean variable to specify if the predicted dose-response should be on the logit scale or the probability scale. Setting to TRUE will transform predictions from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.
#' @examples
#' posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
#'                        DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
#'                        DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2),
#'                        DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2),
#'                        DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
#' models         <- c("emax", "exponential", "sigEmax", "linear")
#' dose_levels    <- c(0, 1, 2, 4, 8)
#'
#' fit        <- getModelFits(models      = models,
#'                            posterior   = posterior_list,
#'                            dose_levels = dose_levels)
#'                            
#' fit
#'                            
#' fit_simple <- getModelFits(models      = models,
#'                            posterior   = posterior_list,
#'                            dose_levels = dose_levels,
#'                            simple      = TRUE)
#'                            
#' fit_simple
#'
#' @return An object of class modelFits. A list containing information about the fitted model coefficients, the prediction per dose group as well as maximum effect and generalized AIC (and corresponding weight) per model.
#'
#' @export
getModelFits <- function (

  models,
  dose_levels,
  posterior,
  avg_fit           = TRUE,
  simple            = FALSE,
  probability_scale = FALSE

) {
  
  checkmate::assert(
    checkmate::check_class(models, "Mods"),
    checkmate::check_character(models, any.missing = FALSE)
  )
  checkmate::assert_double(dose_levels, lower = 0, any.missing = FALSE)
  checkmate::assert(
    checkmate::check_class(posterior, "postList"),
    checkmate::check_list(posterior) && all(sapply(posterior, inherits, c("normMix", "mix")))
  )
  checkmate::assert_flag(avg_fit)
  checkmate::assert_flag(simple)
  checkmate::assert_flag(probability_scale)
  
  if (inherits(models, "character")) {
    
    model_names <- models
    
  } else if (inherits(models, "Mods")) {
    
    model_names <- names(models)
    
  }
  
  model_names <- sort(unique(gsub("\\d", "", model_names)))     # remove duplicate model type specification
  
  add_args <- list(scal = ifelse(is.null(attr(models, "scal")), # if scal does no exist
                                 1.2 * max(dose_levels),        # use default value
                                 attr(models, "scal")))         # otherwise use scal

  getModelFit <- if (simple) getModelFitSimple else getModelFitOpt

  model_fits  <- lapply(model_names, getModelFit, dose_levels, posterior, add_args)
  model_fits  <- addModelWeights(model_fits)
  
  names(model_fits) <- model_names
  
  if (avg_fit) {
    
    model_fits <- addAvgFit(model_fits)
    
  }
  
  if (probability_scale) {
    
    for (i in seq_along(model_fits)) {
      
      model_fits[[i]]$pred_values <- RBesT::inv_logit(model_fits[[i]]$pred_values)
      model_fits[[i]]$max_effect  <- max(model_fits[[i]]$pred_values) - min(model_fits[[i]]$pred_values)
      
    }
    
  }

  attr(model_fits, "direction")         <- getDirection(models, model_fits)
  attr(model_fits, "posterior")         <- posterior
  attr(model_fits, "probability_scale") <- probability_scale
  class(model_fits)                     <- "modelFits"

  return (model_fits)

}

## ignoring mixture posterior
getModelFitSimple <- function (

  model,
  dose_levels,
  posterior,
  add_args = NULL

) {

  fit <- DoseFinding::fitMod(
    dose    = dose_levels,
    resp    = summary.postList(posterior)[, 1],
    S       = diag(summary.postList(posterior)[, 2]^2),
    model   = model,
    type    = "general",
    bnds    = DoseFinding::defBnds(mD = max(dose_levels))[[model]],
    addArgs = add_args)

  pred_vals  <- stats::predict(fit, predType = "ls-means")
  max_effect <- max(pred_vals) - min(pred_vals)
  gAIC       <- DoseFinding::gAIC(fit)

  model_fit <- list(
    model       = model,
    coeffs      = fit$coefs,
    add_args    = add_args,
    dose_levels = dose_levels,
    pred_values = pred_vals,
    max_effect  = max_effect,
    gAIC        = gAIC)
  
  if (model != "betaMod") model_fit$add_args <- NULL

  return (model_fit)

}

## taking mixture posterior into account
getModelFitOpt <- function (

  model,
  dose_levels,
  posterior,
  add_args = NULL

) {

  getOptParams <- function (

    model,
    dose_levels,
    posterior,
    add_args = NULL

  ) {
    
    switch (
      model,
      "emax" = {
        lb     <- c(-Inf, -Inf, 0.001 * max(dose_levels))
        ub     <- c(Inf, Inf, 1.5 * max(dose_levels))
        expr_i <- quote(sum((post_combs$means[i, ] - (theta[1] + (theta[2] * dose_levels) / (theta[3] + dose_levels)))^2 / (post_combs$vars[i, ])))
      },
      "sigEmax" = {
        lb     <- c(-Inf, -Inf, 0.001 * max(dose_levels), 0.5)
        ub     <- c(Inf, Inf, 1.5 * max(dose_levels), 10)
        expr_i <- quote(sum((post_combs$means[i, ] - (theta[1] + (theta[2] * dose_levels^theta[4]) / (theta[3]^theta[4] + dose_levels^theta[4])))^2 / (post_combs$vars[i, ])))
      },
      "exponential" = {
        lb     <- c(-Inf, -Inf, 0.1 * max(dose_levels))
        ub     <- c(Inf, Inf, 2 * max(dose_levels))
        expr_i <- quote(sum((post_combs$means[i, ] - (theta[1] + theta[2] * (exp(dose_levels / theta[3]) - 1)))^2 / (post_combs$vars[i, ])))
      },
      "quadratic" = {
        lb     <- NULL
        ub     <- c(Inf, Inf, Inf)
        expr_i <- quote(sum((post_combs$means[i, ] - (theta[1] + theta[2] * dose_levels + theta[3] * dose_levels^2))^2 / (post_combs$vars[i, ])))
      },
      "linear" = {
        lb     <- NULL
        ub     <- NULL
        expr_i <- quote(sum((post_combs$means[i, ] - (theta[1] + theta[2] * dose_levels))^2 / (post_combs$vars[i, ])))
      },
      "logistic" = {
        lb     <- c(-Inf, -Inf, 0.001 * max(dose_levels), 0.01 * max(dose_levels))
        ub     <- c(Inf, Inf, 1.5 * max(dose_levels), 0.5 * max(dose_levels))
        expr_i <- quote(sum((post_combs$means[i, ] - (theta[1] + theta[2] / (1 + exp((theta[3] - dose_levels) / theta[4]))))^2 / (post_combs$vars[i, ])))
      },
      "betaMod" = {
        lb     <- c(-Inf, -Inf, 0.05, 0.05)
        ub     <- c(Inf, Inf, 4, 4)
        scal   <- ifelse(is.null(add_args) | is.null(add_args[["scal"]]), 1.2 * max(dose_levels), add_args[["scal"]])
        expr_i <- substitute(sum(((post_combs$means[i, ] - (theta[1] + theta[2] * (((theta[3] + theta[4])^(theta[3] + theta[4])) / ((theta[3] ^ theta[3]) * (theta[4]^theta[4]))) * (dose_levels / scal)^(theta[3]) * ((1 - dose_levels / scal)^(theta[4]))))^2 / (post_combs$vars[i, ]))),
                             list(scal = scal))
      },
      stop(paste("error:", model, "shape not (yet) implemented"))
    )

    simple_fit <- getModelFitSimple(
      model       = model,
      dose_levels = dose_levels,
      posterior   = posterior)

    param_list <- list(
      expr_i  = expr_i,
      opts    = list("algorithm" = "NLOPT_LN_NELDERMEAD", maxeval = 1e3), #,stopval=0
      params  = list(
        x0 = simple_fit$coeffs,
        lb = lb,
        ub = ub),
      c_names = names(simple_fit$coeffs))

    return (param_list)

  }

  optFun <- function (
    
    theta,
    dose_levels,
    post_combs,
    expr_i
    
  ) {

    comps <- sapply(seq_along(post_combs$weights), function (i) eval(expr_i))

    return (sum(post_combs$weights * comps))

  }

  param_list <- getOptParams(model, dose_levels, posterior)
  post_combs <- getPostCompsI(posterior)

  fit <- nloptr::nloptr(
    eval_f      = optFun,
    x0          = param_list$params$x0,
    lb          = param_list$params$lb,
    ub          = param_list$params$ub,
    opts        = param_list$opts,
    dose_levels = dose_levels,
    post_combs  = post_combs,
    expr_i      = param_list$expr_i)

  names(fit$solution) <- param_list$c_names

  model_fit <- list(
    model       = model,
    coeffs      = fit$solution,
    add_args    = add_args,
    dose_levels = dose_levels)

  model_fit$pred_values <- predictModelFit(model_fit = model_fit,
                                           doses     = model_fit$dose_levels)
  model_fit$max_effect  <- max(model_fit$pred_values) - min(model_fit$pred_values)
  model_fit$gAIC        <- getGenAIC(model_fit, post_combs)
  
  if (model != "betaMod") model_fit$add_args <- NULL

  return (model_fit)

}

getDirection <- function (
    
  models,
  model_fits
  
) {
  
  if (inherits(models, "Mods")) {
    
    direction   <- attr(models, "direction")
    
  } else if (inherits(models, "character")) {
    
    if (!is.null(attr(models, "direction"))) {
      
      direction <- attr(models, "direction")
      
    } else {
      
      ## Try guessing the direction from the fitted model
      ## in case a character vector is provided
      
      preds <- model_fits[[1]]$pred_values
      
      if (min(preds[-1]) < preds[1]) {
        ## min non-placebo effect less than placebo
        
        direction <- "decreasing"
        
      } else if (max(preds[-1]) > preds[1]) {
        ## max non-placebo effect greater than placebo
        
        direction <- "increasing"
        
      } else {
        
        direction <- character(0)
        
        warning(paste0("The direction of the benefit ",
                       "with increasing dose levels could not be determined."))
        
      }
      
    }
    
  }
  
  return (direction)
  
}

predictModelFit <- function (

  model_fit,
  doses    = NULL

) {
  
  if (is.null(doses)) {
    
    doses <- model_fit$dose_levels
    
  } else if (min(doses) < min(model_fit$dose_levels) | max(doses) > max(model_fit$dose_levels)) {
    
    warning ("The specified range exceeds the bounds of the original dose range.")
    
  }

  ## cannot predict average values as model_fits required instead of model_fit
  pred_vals <- switch (
    model_fit$model,
    "emax" = {DoseFinding::emax(
      doses,
      model_fit$coeffs["e0"],
      model_fit$coeffs["eMax"],
      model_fit$coeffs["ed50"])},
    "sigEmax" = {DoseFinding::sigEmax(
      doses,
      model_fit$coeffs["e0"],
      model_fit$coeffs["eMax"],
      model_fit$coeffs["ed50"],
      model_fit$coeffs["h"])},
    "exponential" = {DoseFinding::exponential(
      doses,
      model_fit$coeffs["e0"],
      model_fit$coeffs["e1"],
      model_fit$coeffs["delta"])},
    "quadratic" = {DoseFinding::quadratic(
      doses,
      model_fit$coeffs["e0"],
      model_fit$coeffs["b1"],
      model_fit$coeffs["b2"])},
    "linear" = {DoseFinding::linear(
      doses,
      model_fit$coeffs["e0"],
      model_fit$coeffs["delta"])},
    "logistic" = {DoseFinding::logistic(
      doses,
      model_fit$coeffs["e0"],
      model_fit$coeffs["eMax"],
      model_fit$coeffs["ed50"],
      model_fit$coeffs["delta"])},
    "betaMod" = {DoseFinding::betaMod(
      doses,
      model_fit$coeffs["e0"],
      model_fit$coeffs["eMax"],
      model_fit$coeffs["delta1"],
      model_fit$coeffs["delta2"],
      model_fit$add_args[["scal"]])},
    "avgFit" = {stop("error: use predictAvgFit for the avgFit")},
    {stop(paste("error:", model_fit$model, "shape not yet implemented"))})

  return (pred_vals)

}

predictAvgFit <- function (
    
  model_fits,
  doses = NULL
  
) {
  
  model_fits_sub <- model_fits[names(model_fits) != "avgFit"]
  
  if (is.null(doses)) {
    
    dose_levels <- model_fits_sub[[1]]$dose_levels
    
    stopifnot(all(sapply(model_fits_sub,
                         function (x) identical(x$dose_levels, dose_levels))))
    
    mod_preds <- sapply(model_fits_sub, function (x) x$pred_values)
    
  } else {
    
    mod_preds <- do.call(cbind, predict.modelFits(model_fits_sub, doses = doses,
                                                  probability_scale = FALSE)) # transformation will be done outside this function if applicable
    
  }
  
  mod_weights <- sapply(model_fits_sub, function (x) x$model_weight)
  
  pred_vals   <- as.vector(mod_preds %*% mod_weights)
  
  return (pred_vals)
  
}

addAvgFit <- function (
    
  model_fits,
  doses = NULL
  
) {
  
  pred_vals  <- predictAvgFit(model_fits = model_fits, doses = doses)
  
  avg_fit  <- list(model        = "avgFit",
                   coeff        = NA,
                   dose_levels  = model_fits[[1]]$dose_levels,
                   pred_values  = pred_vals,
                   max_effect   = max(pred_vals) - min(pred_vals),
                   gAIC         = NA,
                   model_weight = NA)
  
  model_fits <- c(avgFit = list(avg_fit), model_fits)
  
  return (model_fits)
  
}

getGenAIC <- function (

  model_fit,
  post_combs

) {

  expr_i   <- quote(sum(1 / post_combs$vars[i, ] * (post_combs$means[i, ] - model_fit$pred_values)^2) + 2 * length(model_fit$coeffs))
  comb_aic <- sapply(seq_along(post_combs$weights), function (i) eval(expr_i))

  g_aic    <- stats::weighted.mean(comb_aic, w = post_combs$weights)

  return (g_aic)

}

addModelWeights <- function (

  model_fits

) {
  
  g_aics        <- sapply(model_fits, function (model_fit) model_fit$gAIC)
  exp_values    <- exp(-0.5 * g_aics)
  model_weights <- exp_values / sum(exp_values)

  names(model_weights) <- NULL

  model_fits_out <- lapply(seq_along(model_fits), function (i) {

    c(model_fits[[i]], model_weight = model_weights[i])

  })

  return (model_fits_out)

}

#' @title getMED
#'
#' @description This function provides information on the minimally efficacious dose (MED).
#' The MED evaluation can either be based on the fitted model shapes (model_fits) or on bootstrapped quantiles (bs_quantiles). 
#' @details
#' The function assumes that the 1st dose group is the control dose group.
#' 
#' The bootstrap approach allows for an MED based on decision rules of the form
#' \deqn{\widehat{\text{MED}} = \text{arg min}_{d\in\{d_1, \dots, d_k\}} \left\{ \text{Pr}\left(f(d, \hat\theta) - f(d_1, \hat\theta) > \Delta\right) > \gamma \right\} .}
#' The model-shape approach takes the point estimate of the model into account.
#' @param delta A numeric value for the threshold Delta.
#' @param evidence_level A numeric value between 0 and 1 for the evidence level gamma. Used for the bs_quantiles-based evaluation and not used for the model_fits-based evaluation. Default 0.5.
#' @param dose_levels A vector of numerics containing the different dosage levels. Default NULL.
#' @param model_fits An object of class modelFits as created with getModelFits(). Default NULL.
#' @param bs_quantiles A dataframe created with getBootstrapQuantiles(). Default NULL.
#' @param probability_scale A boolean variable to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform predictions from the logit scale to the probability scale, which can be desirable for a binary outcome. Default `attr(model_fits, "probability_scale")`.
#' 
#' @return  A matrix with rows for MED reached, MED, and MED index in the vector of dose levels and columns for the dose-response shapes.
#' @examples
#' posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
#'                        DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
#'                        DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2) ,
#'                        DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2) ,
#'                        DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
#' models         <- c("exponential", "linear")
#' dose_levels    <- c(0, 1, 2, 4, 8)
#' model_fits     <- getModelFits(models      = models,
#'                                posterior   = posterior_list,
#'                                dose_levels = dose_levels,
#'                                simple      = TRUE)
#' 
#' # MED based on the model_fit:
#' getMED(delta = 5, model_fits = model_fits)
#'                                
#' # MED based on bootstrapped quantiles
#' bs_quantiles <- getBootstrapQuantiles(model_fits = model_fits,
#'                                       quantiles  = c(0.025, 0.2, 0.5, 0.8),
#'                                       n_samples  = 100) # speeding up example run time
#'                                       
#' getMED(delta          = 5,
#'        evidence_level = 0.8,
#'        bs_quantiles   = bs_quantiles)
#'        
#' # MED on the probability scale
#' getMED(delta = 0.1, model_fits = model_fits, probability_scale = TRUE)
#' 
#' @export
getMED <- function (
    
  delta,
  evidence_level    = 0.5,
  dose_levels       = NULL,
  model_fits        = NULL,
  bs_quantiles      = NULL,
  probability_scale = attr(model_fits, "probability_scale")
  
) {
  
  ## R CMD --as-cran appeasement
  model <- sample_type <- NULL
  
  stopifnot("Either model_fits or bs_quantiles must be not NULL, but not both" =
              is.null(model_fits) & !is.null(bs_quantiles) |
              !is.null(model_fits) & is.null(bs_quantiles))
  
  checkmate::assert_double(delta, len = 1L)
  checkmate::assert_double(evidence_level, lower = 0, upper = 1, len = 1L)
  checkmate::assert_double(dose_levels, lower = 0, any.missing = FALSE, null.ok = TRUE)
  
  checkmate::assert_flag(probability_scale, null.ok = TRUE)
  if (is.null(probability_scale)) probability_scale <- FALSE
  
  if (probability_scale) checkmate::assert_double(delta, lower = 0, upper = 1)
  
  if (!is.null(model_fits)) {
    ## Direction not needed, because mean value
    ## (approx. evidence_level = 0.5) is used.
    
    checkmate::assert_class(model_fits, "modelFits")
    
    if (is.null(dose_levels)) dose_levels <- model_fits[[1]]$dose_levels
    
    preds <- do.call(rbind, predict.modelFits(model_fits, doses = dose_levels,
                                              probability_scale = probability_scale))
    
    abs_diffs <- abs(preds - preds[, 1])
    colnames(abs_diffs) <- dose_levels
    
  } else {
    
    # R CMD Check Appeasement
    q_prob <- dose <- q_val <- models <- NULL
    
    # Floating-point precision handling to pick up correct rows of bs_quantiles
    tolerance <- 1e-9
    
    direction_increasing <- attr(bs_quantiles, "direction") == "increasing"
    
    if (!direction_increasing) {

      stopifnot("corresponding quantile (i.e. evidence_level) not in bootstrapped quantiles matrix" = 
                  any(abs(bs_quantiles$q_prob - evidence_level) < tolerance))

    } else if (direction_increasing) {
      
      stopifnot("corresponding quantile (i.e. 1 - evidence_level) not in bootstrapped quantiles matrix" = 
                  any(abs((1 - bs_quantiles$q_prob) - evidence_level) < tolerance))
      
    }
    
    if (is.null(dose_levels)) dose_levels <- unique(bs_quantiles$dose)
    
    stopifnot("dose_levels not (all) in bootstrapped quantiles matrix" = 
                all(dose_levels %in% bs_quantiles$dose))
    
    abs_diffs <- bs_quantiles |>
      dplyr::filter(abs((if (direction_increasing) 1 - q_prob else q_prob) - evidence_level) < tolerance,
                    dose %in% dose_levels,
                    sample_type == "diff") |>
      tidyr::pivot_wider(names_from = dose, values_from = q_val) |>
      dplyr::select(-model, -sample_type, -q_prob) |>
      as.matrix() |>
      abs()

    rownames(abs_diffs) <- unique(bs_quantiles$model)
    
  }
  
  med_info <- apply(abs_diffs > delta, 1, function (med_indx_m) {
    
    if (!any(med_indx_m)) {
      
      med_reached <- 0
      med         <- NA_real_
      
    } else {
      
      med_reached <- 1
      med_indx    <- min(which(med_indx_m))
      med         <- dose_levels[med_indx]
      
    }
    
    return (c(med_reached = med_reached,
              med         = med))
    
  })
  
  return (med_info)
  
}

test_that("getModelFits: probability_scale=TRUE gives pred_values in [0,1] and predict() is consistent", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("DoseFinding")
  
  set.seed(11)
  
  dose_levels <- c(0, 1, 2, 4)
  n_per_dose  <- 60
  probs       <- c(0.10, 0.20, 0.30, 0.55)
  
  dat <- data.frame(
    simulation = 1L,
    dose       = rep(dose_levels, each = n_per_dose),
    response   = unlist(lapply(probs, function(p) rbinom(n_per_dose, 1, p)))
  )
  attr(dat, "probability_scale") <- TRUE
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(probs[i]), s = 1.5), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(prior_list = prior_list, data = dat, probability_scale = TRUE)
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    emax   = c(0.5, 1.2),
    doses  = dose_levels,
    maxEff = 2
  )
  
  fits <- getModelFits(
    models            = mods,
    dose_levels       = dose_levels,
    posterior         = post,
    simple            = TRUE,
    avg_fit           = TRUE,
    probability_scale = TRUE
  )
  
  expect_s3_class(fits, "modelFits")
  expect_true(isTRUE(attr(fits, "probability_scale")))
  
  for (nm in names(fits)) {
    pv <- fits[[nm]]$pred_values
    expect_true(is.numeric(pv))
    expect_true(all(pv >= 0 & pv <= 1))
    expect_equal(fits[[nm]]$max_effect, max(pv) - min(pv), tolerance = 1e-12)
  }
  
  # predict() should match inv_logit of logit-scale predictions
  doses_new <- sort(unique(c(0, 0.5, 1, 3, 4)))
  pred_prob <- predict(fits, doses = doses_new, probability_scale = TRUE)
  pred_log  <- predict(fits, doses = doses_new, probability_scale = FALSE)
  
  for (nm in names(pred_prob)) {
    expect_equal(
      as.numeric(pred_prob[[nm]]),
      as.numeric(RBesT::inv_logit(pred_log[[nm]])),
      tolerance = 1e-10
    )
  }
})

test_that("Binary posterior: posterior means are close to empirical logits (vague prior, large n)", {
  skip_if_not_installed("RBesT")
  
  set.seed(101)
  
  dose_levels <- c(0, 1, 2, 4)
  n_per_dose  <- 400
  p_true      <- c(0.10, 0.20, 0.35, 0.55)
  
  dat <- data.frame(
    simulation = 1L,
    dose       = rep(dose_levels, each = n_per_dose),
    response   = unlist(lapply(p_true, function(p) rbinom(n_per_dose, 1, p)))
  )
  attr(dat, "probability_scale") <- TRUE
  
  # Vague priors on logit scale (so data dominates)
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 10), sigma = 10)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(prior_list = prior_list, data = dat, probability_scale = TRUE)
  smry <- summary(post)
  
  # empirical logits (avoid +/-Inf)
  p_hat <- tapply(dat$response, dat$dose, mean)
  eps <- 1/(n_per_dose + 1)  # gentle continuity correction
  p_hat_cc <- pmin(pmax(p_hat, eps), 1 - eps)
  logit_hat <- qlogis(p_hat_cc)
  
  # posterior means should be near empirical logits when n is large and prior vague
  # Tolerance: 0.25 on logit scale is fairly tight for Monte Carlo variability
  expect_equal(
    as.numeric(smry[, "mean"]),
    as.numeric(logit_hat),
    tolerance = 0.25
  )
  
  # sanity: posterior SDs should be finite and not huge with n=400
  expect_true(all(is.finite(smry[, "sd"])))
  expect_true(all(smry[, "sd"] < 2))
})


test_that("Binary posterior: uncertainty shrinks with more data (same p_true)", {
  skip_if_not_installed("RBesT")
  
  set.seed(102)
  
  dose_levels <- c(0, 1, 2, 4)
  p_true      <- c(0.12, 0.18, 0.30, 0.50)
  
  make_dat <- function(n_per_dose) {
    d <- data.frame(
      simulation = 1L,
      dose       = rep(dose_levels, each = n_per_dose),
      response   = unlist(lapply(p_true, function(p) rbinom(n_per_dose, 1, p)))
    )
    attr(d, "probability_scale") <- TRUE
    d
  }
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 10), sigma = 10)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post_small <- getPosterior(prior_list = prior_list, data = make_dat(40), probability_scale = TRUE)
  post_big   <- getPosterior(prior_list = prior_list, data = make_dat(400), probability_scale = TRUE)
  
  sd_small <- as.numeric(summary(post_small)[, "sd"])
  sd_big   <- as.numeric(summary(post_big)[, "sd"])
  
  # Big sample should have smaller SD in aggregate (not necessarily every single dose)
  expect_true(mean(sd_big) < mean(sd_small))
})


test_that("Binary modeling: fitted probabilities are plausible vs. generating truth (strong signal)", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("DoseFinding")
  
  set.seed(103)
  
  dose_levels <- c(0, 1, 2, 4)
  n_per_dose  <- 250
  p_true      <- c(0.08, 0.18, 0.35, 0.65)
  
  dat <- data.frame(
    simulation = 1L,
    dose       = rep(dose_levels, each = n_per_dose),
    response   = unlist(lapply(p_true, function(p) rbinom(n_per_dose, 1, p)))
  )
  attr(dat, "probability_scale") <- TRUE
  
  # Mildly informative priors near truth (keeps test stable)
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(p_true[i]), s = 1.5), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(prior_list = prior_list, data = dat, probability_scale = TRUE)
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    emax   = c(0.5, 1.2),
    doses  = dose_levels,
    maxEff = 2
  )
  
  fits <- getModelFits(
    models            = mods,
    dose_levels       = dose_levels,
    posterior         = post,
    simple            = TRUE,
    avg_fit           = TRUE,
    probability_scale = TRUE
  )
  
  # avgFit should be close-ish to p_true at the observed doses when signal is strong
  avg_pred <- fits$avgFit$pred_values
  expect_true(all(avg_pred >= 0 & avg_pred <= 1))
  
  # Probability-scale tolerance: 0.08 is fairly strict with n=250
  expect_equal(as.numeric(avg_pred), as.numeric(p_true), tolerance = 0.08)
  
  # max_effect should be close to true range
  expect_equal(fits$avgFit$max_effect, max(p_true) - min(p_true), tolerance = 0.08)
})


test_that("Bootstrap quantiles: ordered, bounded, and median sits between 20% and 80%", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("DoseFinding")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  
  set.seed(104)
  
  dose_levels <- c(0, 1, 2, 4)
  p_true      <- c(0.10, 0.20, 0.33, 0.55)
  
  mu_hat <- qlogis(p_true)
  se_hat <- rep(0.35, length(dose_levels))
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = mu_hat[i], s = 1.5), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(
    prior_list        = prior_list,
    mu_hat            = mu_hat,
    S_hat             = diag(se_hat^2),
    probability_scale = TRUE
  )
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    emax   = c(0.5, 1.2),
    doses  = dose_levels,
    maxEff = 2
  )
  
  fits <- getModelFits(
    models            = mods,
    dose_levels       = dose_levels,
    posterior         = post,
    simple            = TRUE,
    avg_fit           = TRUE,
    probability_scale = TRUE
  )
  
  bq <- getBootstrapQuantiles(
    model_fits        = fits,
    quantiles         = c(0.2, 0.5, 0.8),
    n_samples         = 120,
    doses             = dose_levels,
    probability_scale = TRUE
  )
  
  # Bounds
  expect_true(all(bq$q_val[bq$sample_type == "abs"]  >= 0 & bq$q_val[bq$sample_type == "abs"] <= 1))
  expect_true(all(bq$q_val[bq$sample_type == "diff"] >= -1 & bq$q_val[bq$sample_type == "diff"] <= 1))
  
  # Quantile ordering check (within each model/dose/sample_type)
  # q(0.2) <= q(0.5) <= q(0.8)
  suppressWarnings({
    split_keys <- interaction(bq$model, bq$dose, bq$sample_type, drop = TRUE)
  })
  parts <- split(bq, split_keys)
  for (part in parts) {
    qs <- part$q_prob
    vals <- part$q_val
    o <- order(qs)
    expect_true(all(diff(vals[o]) >= -1e-10))
  }
  
  # Median between 20% and 80% everywhere
  for (part in parts) {
    q20 <- part$q_val[part$q_prob == 0.2]
    q50 <- part$q_val[part$q_prob == 0.5]
    q80 <- part$q_val[part$q_prob == 0.8]
    expect_true(all(q20 <= q50 + 1e-10))
    expect_true(all(q50 <= q80 + 1e-10))
  }
})


test_that("MED: coherent behavior when delta is above/below achievable effect (probability scale)", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("DoseFinding")
  
  set.seed(105)
  
  dose_levels <- c(0, 1, 2, 4)
  p_true      <- c(0.10, 0.20, 0.35, 0.60)
  
  mu_hat <- qlogis(p_true)
  se_hat <- rep(0.30, length(dose_levels))
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = mu_hat[i], s = 1.2), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(prior_list = prior_list, mu_hat = mu_hat, S_hat = diag(se_hat^2), probability_scale = TRUE)
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    emax   = c(0.5, 1.2),
    doses  = dose_levels,
    maxEff = 2
  )
  
  fits <- getModelFits(
    models            = mods,
    dose_levels       = dose_levels,
    posterior         = post,
    simple            = TRUE,
    avg_fit           = TRUE,
    probability_scale = TRUE
  )
  
  # Achievable effect from avgFit in probability space
  eff <- fits$avgFit$max_effect
  expect_true(eff >= 0 && eff <= 1)
  
  # delta > effect => should not be reached (for avgFit column at least)
  med_hi <- getMED(delta = min(0.99, eff + 0.15), model_fits = fits, probability_scale = TRUE)
  expect_equal(med_hi["med_reached", "avgFit"], 0)
  
  # delta < effect => should be reached
  med_lo <- getMED(delta = max(0.001, eff - 0.15), model_fits = fits, probability_scale = TRUE)
  expect_equal(med_lo["med_reached", "avgFit"], 1)
})


test_that("Edge case: rare events (0 or all events in a group) yields finite posterior summaries", {
  skip_if_not_installed("RBesT")
  
  set.seed(106)
  
  dose_levels <- c(0, 1, 2)
  n_per_dose  <- 120
  
  # Construct rare-event scenario: control has 0 events, middle has a few, high has many
  y0 <- rep(0, n_per_dose)                              # 0%
  y1 <- rbinom(n_per_dose, 1, 0.03)                     # ~3%
  y2 <- rbinom(n_per_dose, 1, 0.70)                     # high
  
  dat <- data.frame(
    simulation = 1L,
    dose       = rep(dose_levels, each = n_per_dose),
    response   = c(y0, y1, y2)
  )
  attr(dat, "probability_scale") <- TRUE
  
  # Mild prior regularization helps separation-ish scenarios remain finite
  p_center <- c(0.02, 0.05, 0.60)
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(p_center[i]), s = 2.0), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(prior_list = prior_list, data = dat, probability_scale = TRUE)
  smry <- summary(post)
  
  expect_true(all(is.finite(smry[, "mean"])))
  expect_true(all(is.finite(smry[, "sd"])))
  
  # Posterior mean ordering should reflect data direction (not necessarily strict, but likely)
  # Control logit mean should be smallest; high dose should be largest
  expect_true(smry["Ctr", "mean"] < smry["DG_2", "mean"])
})

# Test predictModelFit function
test_that("predictModelFit works correctly", {
  model_fit <- list(
    model = "emax",
    coeffs = c(e0 = 0, eMax = 1, ed50 = 2),
    dose_levels = c(0, 1, 2, 3, 4, 5)
  )

  pred_vals <- predictModelFit(model_fit)
  expect_type(pred_vals, "double")

  dose_levels <- c(0, 2.5, 5, 10)

  prior_list <- lapply(dose_levels, function(dose_group) {
    RBesT::mixnorm(weak = c(w = 1, m = 0, s = 200), sigma = 10)
  })

  names(prior_list) <- c("Ctr", paste0("DG_", dose_levels[-1]))

  trial_data <- dplyr::filter(
    dplyr::filter(tibble::tibble(testdata), bname == "BRINTELLIX"),
    primtime == 8,
    indication == "MAJOR DEPRESSIVE DISORDER",
    protid == 5
  )

  posterior <- getPosterior(
    prior_list = prior_list,
    mu_hat     = trial_data$rslt,
    S_hat      = diag(trial_data$se),
    calc_ess = TRUE
  )

  n_patients <- c(128, 124, 129, 122)

  # Guesstimate estimation
  exp_guesst <- DoseFinding::guesst(
    model = "exponential",
    d = 5, p = 0.2, Maxd = max(dose_levels)
  )
  emax_guesst <- DoseFinding::guesst(
    model = "emax",
    d = 2.5, p = 0.9
  )
  sigEmax_guesst <- DoseFinding::guesst(
    model = "sigEmax",
    d = c(2.5, 5), p = c(0.5, 0.95)
  )
  logistic_guesst <- DoseFinding::guesst(
    model = "logistic",
    d = c(5, 10), p = c(0.1, 0.85)
  )

  betaMod_params <- c(delta1 = 1, delta2 = 1)
  quadratic_params <- c(delta2 = -0.1)

  mods <- DoseFinding::Mods(
    linear      = NULL,
    # guesstimate scale
    exponential = exp_guesst,
    emax        = emax_guesst,
    sigEmax     = sigEmax_guesst,
    logistic    = logistic_guesst,
    # parameter scale
    betaMod     = betaMod_params,
    quadratic   = quadratic_params,
    # Options for all models
    doses       = dose_levels,
    maxEff      = -1,
    placEff     = -12.8
  )

  fit_simple <- getModelFits(
    models      = names(mods),
    dose_levels = dose_levels,
    posterior   = posterior,
    simple      = TRUE
  )

  expect_no_error(fit_simple)
  expect_type(fit_simple, "list")

  fit <- getModelFits(
    models      = names(mods),
    dose_levels = dose_levels,
    posterior   = posterior,
    simple      = FALSE
  )

  expect_no_error(fit)
  expect_type(fit, "list")
})

# Test for addModelWeights
test_that("addModelWeights works correctly in a simple case", {
  # Create a test case
  test_model_fits1 <- list(
    model1 = list(gAIC = 1),
    model2 = list(gAIC = 1),
    model3 = list(gAIC = 1)
  )
  expected_output1 <- list(
    list(gAIC = 1, model_weight = 1 / 3),
    list(gAIC = 1, model_weight = 1 / 3),
    list(gAIC = 1, model_weight = 1 / 3)
  )

  # Call the function with the test case
  result <- addModelWeights(model_fits = test_model_fits1)
  # Assert the expected behavior
  expect_true(is.list(result))
  expect_equal(length(result), length(test_model_fits1))
  expect_true(all(sapply(result, function(x) "model_weight" %in% names(x))))
  expect_equal(result, expected_output1)

  test_model_fits2 <- list(
    model1 = list(gAIC = 100),
    model2 = list(gAIC = 0),
    model3 = list(gAIC = 100)
  )
  expected_output2 <- list(
    list(gAIC = 100, model_weight = 0),
    list(gAIC = 0, model_weight = 1),
    list(gAIC = 100, model_weight = 0)
  )
  result <- addModelWeights(model_fits = test_model_fits2)
  expect_equal(result, expected_output2, tolerance = 1e-10)
})

# Test for getGenAIC
test_that("getGenAIC calculates AIC correctly using snapshot test and a simple example", {
  # Create a test case
  test_model_fit <- list(
    pred_values = rep(1, 3),
    coeffs = rep(1, 3)
  )
  test_post_combs <- list(
    means = matrix(rep(1, 6), nrow = 2),
    vars = matrix(rep(1, 6), nrow = 2),
    weights = c(1, 1)
  )

  result <- getGenAIC(model_fit = test_model_fit, post_combs = test_post_combs)
  # expected result determined with snapshot of behavior prior to first release on CRAN
  expected_result <- 6
  # Assert the expected behavior
  expect_equal(result, expected_result)
})

test_that("getMED: probability scale enforces delta bounds and returns expected structure", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("DoseFinding")
  skip_if_not_installed("dplyr")
  skip_if_not_installed("tidyr")
  
  set.seed(13)
  
  dose_levels <- c(0, 1, 2, 4)
  probs <- c(0.10, 0.18, 0.28, 0.45)
  
  mu_hat <- qlogis(probs)
  se_hat <- rep(0.40, length(dose_levels))
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = mu_hat[i], s = 1.2), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(
    prior_list        = prior_list,
    mu_hat            = mu_hat,
    S_hat             = diag(se_hat^2),
    probability_scale = TRUE
  )
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    emax   = c(0.5, 1.2),
    doses  = dose_levels,
    maxEff = 2
  )
  
  fits <- getModelFits(
    models            = mods,
    dose_levels       = dose_levels,
    posterior         = post,
    simple            = TRUE,
    avg_fit           = TRUE,
    probability_scale = TRUE
  )
  
  # delta bounds on probability scale
  expect_error(getMED(delta = 1.5, model_fits = fits, probability_scale = TRUE))
  expect_error(getMED(delta = -0.1, model_fits = fits, probability_scale = TRUE))
  
  med <- getMED(delta = 0.10, model_fits = fits, probability_scale = TRUE)
  expect_true(is.matrix(med))
  expect_equal(rownames(med), c("med_reached", "med"))
  expect_true(all(colnames(med) %in% names(fits)))
  
  # Bootstrap path
  bq <- getBootstrapQuantiles(
    model_fits        = fits,
    quantiles         = c(0.2, 0.8),
    n_samples         = 80,
    doses             = dose_levels,
    probability_scale = TRUE
  )
  
  med2 <- getMED(
    delta             = 0.10,
    evidence_level    = 0.8,
    bs_quantiles      = bq,
    probability_scale = TRUE
  )
  expect_true(is.matrix(med2))
  expect_equal(rownames(med2), c("med_reached", "med"))
})

addSignificance <- function (

  model_fits,
  sign_models

) {

  sign_models_fits <- sapply(names(model_fits), function (model_name) {
    
    if (model_name != "avgFit") {
      
      indx <- grepl(model_name, names(sign_models))
      
      sum(sign_models[indx]) > 0
      
    } else {
      
      NA
      
    }
    
  })


  model_fits_out <- lapply(seq_along(model_fits), function (i) {

    c(model_fits[[i]], significant = sign_models_fits[i])

  })

  attributes(model_fits_out) <- attributes(model_fits)

  return (model_fits_out)

}

BayesMCPi <- function (

  posterior_i,
  contr,
  crit_prob_adj

) {
  
  post_comps_i <- attr(posterior_i, "posteriorInfo")
  
  ## Pinheiro, José, et al.
  ## "Model‐based dose finding under model uncertainty using general parametric models." 
  ## Statistics in medicine 33.10 (2014): 1646-1661. doi:10.1002/sim.6052
  ## Section 2.2
  
  mu_mat    <- do.call(cbind, post_comps_i$means) # [N candidate models, N posterior components]
  S_list    <- post_comps_i$covMats               # list[[N posterior components]] of cov matrices
                                                  # with [N candidate models, N candidate models]
  
  # Optimal contrast matrix
  contr_mat <- contr$contMat                      # [N dose levels, N candidate models]
  
  ## Contrast estimates for each component
  contr_est_mu  <- t(contr_mat) %*% mu_mat        # [N candidate models, N posterior components]
  contr_est_cov <-
    lapply(S_list, function(S)                    # list[[N posterior components]] of cov matrices
      t(contr_mat) %*% S %*% contr_mat)           # with [N candidate models, N candidate models]
  
  ## Test statistic for each component            # [N candidate models, N posterior components]
  z_m <- contr_est_mu / do.call(cbind, lapply(contr_est_cov, diag))^0.5
  
  ## Fleischer, Frank, et al.
  ## "Bayesian MCPMod."
  ## Pharmaceutical Statistics 21.3 (2022): 654-670. doi:10.1002/pst.2193
  ## Section 2.3
  
  ## Posterior probabilities                      # [N candidate models]
  post_probs <- (stats::pnorm(z_m) %*% unlist(post_comps_i$weights))[, 1]

  res <- c(sign          = ifelse(max(post_probs) > crit_prob_adj, 1, 0),
           crit_prob_adj = crit_prob_adj,
           max_post_prob = max(post_probs),
           post_probs    = post_probs)

  return (res)
  
  # getPostProb <- function (
  #   
  #   contr_j,     # j: dose level
  #   post_combs_i # i: simulation outcome
  #   
  # ) {
  # 
  #   ## Fleischer, Frank, et al. "Bayesian MCPMod."
  #   ## Pharmaceutical Statistics 21.3 (2022): 654-670. doi:10.1002/pst.2193
  #   ## Sections 2.2 and 2.3
  #   
  #   ## Test statistic = sum over all components of
  #   ## posterior weight * normal probability distribution of
  #   ## critical values for doses * estimated mean / sqrt(product of critical values for doses)
  #   
  #   ## Calculation for each component of the posterior
  #   contr_theta   <- apply(post_combs_i$means, 1, `%*%`, contr_j)
  #   contr_var     <- apply(post_combs_i$vars, 1, `%*%`, contr_j^2)
  #   contr_weights <- post_combs_i$weights
  #   
  #   ## P(c_m * theta > 0 | Y = y) for a shape m (and dose j)
  #   post_probs <- sum(contr_weights * stats::pnorm(contr_theta / sqrt(contr_var)))
  #   
  #   return (post_probs)
  #   
  # }
  # 
  # post_combs_i <- getPostCompsI(posterior_i)
  # post_probs   <- apply(contr$contMat, 2, getPostProb, post_combs_i)
  # 
  # res <- c(sign          = ifelse(max(post_probs) > crit_prob_adj, 1, 0),
  #          crit_prob_adj = crit_prob_adj,
  #          max_post_prob = max(post_probs),
  #          post_probs    = post_probs)
  # 
  # return (res)
  
}

#' @title getContr
#'
#' @description This function calculates contrast vectors that are optimal for detecting certain alternatives via applying the function optContr() of the DoseFinding package.
#' Hereby, 4 different options can be distinguished that are automatically executed based on the input that is provided
#' 1) Bayesian approach: If dose_weights and a prior_list are provided an optimized contrasts for the posterior sample size is calculated.
#'    In detail,  in a first step the dose_weights (typically the number of patients per dose group) and the prior information is combined by calculating for
#'    each dose group a posterior effective sample. Based on this posterior effective sample sizes the allocation ratio is derived, which allows for a calculation on
#'    pseudo-optimal contrasts via regular MCPMod are calculated from the
#'    regular MCPMod for these specific weights
#' 2) Frequentist approach: If only dose_weights are provided optimal contrast vectors are calculated from the
#'    regular MCPMod for these specific weights
#' 3) Bayesian approach + re-estimation: If only a cov_posterior (i.e. variability of the posterior distribution) is provided, pseudo-optimal contrasts based on these posterior weights will be calculated
#' 4) Frequentist approach+re-estimation: If only a cov_new_trial (i.e. the estimated variability of a new trial) is provided, optimal contrast vectors are calculated from the
#'    regular MCPMod for this specific covariance matrix. 
#'
#' @param mods An object of class 'Mods' as created by the function 'DoseFinding::Mods()'
#' @param dose_levels Vector containing the different dosage levels.
#' @param dose_weights Vector specifying weights for the different doses. Please note that in case this information is provided together with a prior (i.e. Option 1) is planned these two inputs should be provided on the same scale (e.g. patient numbers).  Default NULL
#' @param prior_list A list of objects of class 'normMix' as created with 'RBesT::mixnorm()'. Only required as input for Option 1. Default NULL
#' @param cov_posterior A covariance matrix with information about the variability of the posterior distribution, only required for Option 3. Default NULL
#' @param cov_new_trial A covariance matrix with information about the observed variability, only required for Option 4. Default NULL
#'
#' @examples
#' dose_levels  <- c(0, 0.5, 2, 4, 8)
#' mods <- DoseFinding::Mods(
#'   linear      = NULL,
#'   emax        = c(0.5, 1.2),
#'   exponential = 2,
#'   doses       = dose_levels,
#'   maxEff      = 6)
#' cov_posterior <- diag(c(2.8, 3, 2.5, 3.5, 4)^2)
#'
#' contr_mat <- getContr(
#'   mods         = mods,
#'   dose_levels  = dose_levels,
#'   cov_posterior = cov_posterior)
#'
#' @return An object of class 'optContr' as provided by the function 'DoseFinding::optContr()'.
#'
#' @export
getContr <- function (
    
    mods,
    dose_levels,
    dose_weights  = NULL,
    prior_list    = NULL,
    cov_posterior = NULL,
    cov_new_trial = NULL
    
) {
  
  checkmate::assert_class(mods, "Mods")
  checkmate::assert_numeric(dose_levels, lower = 0, any.missing = FALSE, len = length(attr(mods, "doses")))

  if (!is.null(dose_weights)) {
    checkmate::assert_numeric(dose_weights, any.missing = FALSE, len = length(attr(mods, "doses")))
  }
  if (!is.null(prior_list)) {
    checkmate::assert_list(prior_list, names = "named", len = length(attr(mods, "doses")), any.missing = FALSE)
  }

  # Determine scenario
  if (!is.null(cov_new_trial) &&
      is.null(dose_weights) && is.null(prior_list) && is.null(cov_posterior)) {

    w <- NULL
    S <- cov_new_trial

  } else if (!is.null(dose_weights) &&
             is.null(cov_new_trial) && is.null(prior_list) && is.null(cov_posterior)) {

    w <- dose_weights
    S <- NULL

  } else if (!is.null(cov_posterior) &&
             is.null(cov_new_trial) && is.null(prior_list) && is.null(dose_weights)) {

    w <- NULL
    S <- cov_posterior

  } else if (!is.null(dose_weights) && !is.null(prior_list) &&
             is.null(cov_new_trial) && is.null(cov_posterior)) {

    w <- dose_weights +
      suppressMessages(round(unlist(lapply(prior_list, RBesT::ess))))
    S <- NULL

  } else {
    stop("Invalid combination of inputs. Valid combinations are:
    1. cov_new_trial only (frequentist with re-estimation)
    2. dose_weights only (frequentist without re-estimation)
    3. cov_posterior only (Bayesian with re-estimation)
    4. dose_weights + prior_list (Bayesian without re-estimation)")
  }

  # Compute contrasts
  if (is.null(w)) {
    contr <- DoseFinding::optContr(
      models = mods,
      doses  = dose_levels,
      S      = S
    )
  } else {
    contr <- DoseFinding::optContr(
      models = mods,
      doses  = dose_levels,
      w      = w
    )
  }
  
  attr(contr, "direction") <- attr(mods, "direction")

  return(contr)
  
}

#' @title getCritProb
#'
#' @description This function calculates multiplicity adjusted critical values. The critical values are calculated in such a way that
#'  when using non-informative priors the actual error level for falsely declaring a significant trial in the Bayesian MCPMod is controlled (by the specified alpha level).
#'  Hereby optimal contrasts of the frequentist MCPMod are applied and two options can be distinguished
#'  1) Frequentist approach: If only dose_weights are provided optimal contrast vectors are calculated from the
#'     regular MCPMod for these specific weights and the corresponding critical value for this set of contrasts is calculated via the critVal() function of the DoseFinding package.
#'  2) Frequentist approach + re-estimation: If only a cov_new_trial (i.e. the covariance matrix of a new trial) is provided, optimal contrast vectors are calculated from the
#'     regular MCPMod for this specific matrix. Here as well the critical value for this set of contrasts is calculated via the critVal() function of the DoseFinding package.
#'
#' @param mods An object of class "Mods" as specified in the DoseFinding package.
#' @param dose_levels Vector containing the different dosage levels.
#' @param dose_weights Vector specifying weights for the different doses, only required for Option i). Default NULL
#' @param cov_new_trial A covariance matrix, only required for Option ii). Default NULL
#' @param alpha_crit_val Significance level. Default set to 0.025.
#'
#' @examples
#' mods <- DoseFinding::Mods(linear      = NULL,
#'                           emax        = c(0.5, 1.2),
#'                           exponential = 2,
#'                           doses       = c(0, 0.5, 2,4, 8))
#' dose_levels <- c(0, 0.5, 2, 4, 8)
#' critVal <- getCritProb(
#'   mods           = mods,
#'   dose_weights   = c(50,50,50,50,50), #reflecting the planned sample size
#'   dose_levels    = dose_levels,
#'   alpha_crit_val = 0.05)
#' @return Multiplicity adjusted critical value on the probability scale.
#'
#' @export
getCritProb <- function (

  mods,
  dose_levels,
  dose_weights   = NULL,
  cov_new_trial  = NULL,
  alpha_crit_val = 0.025

) {

  checkmate::assert_class(mods, classes = "Mods")
  checkmate::assert_double(dose_levels, lower = 0, any.missing = FALSE)
  checkmate::assert_double(dose_weights, any.missing = FALSE, len = length(dose_levels), null.ok = TRUE)
  checkmate::assert_double(alpha_crit_val, lower = 0, upper = 1)

  # Get contrast using updated getContr
  contr <- getContr(
    mods          = mods,
    dose_levels   = dose_levels,
    dose_weights  = dose_weights,
    cov_new_trial = cov_new_trial
  )

  # Calculate critical probability
  crit_prob <- stats::pnorm(DoseFinding::critVal(
    corMat      = contr$corMat,
    alpha       = alpha_crit_val,
    df          = 0,
    alternative = "one.sided"
  ))

  return(crit_prob)
  
}

getModelSuccesses <- function (b_mcp) {

  stopifnot(inherits(b_mcp, "BayesianMCP"))

  model_indices <- grepl("post_probs.", colnames(b_mcp))
  model_names   <- colnames(b_mcp)[model_indices] |>
    sub(pattern = "post_probs.", replacement = "", x = _)

  model_successes <- colMeans(b_mcp[, model_indices] > b_mcp[, "crit_prob_adj"])

  names(model_successes) <- model_names

  return (model_successes)

}

#' @title performBayesianMCPMod
#'
#' @description Performs Bayesian MCP Test step and modeling in a combined fashion. See performBayesianMCP() function for MCP Test step and getModelFits() for the modeling step
#'
#' @param posterior_list An object of class 'postList' or a list of 'postList' objects as created by getPosterior() containing information about the (mixture) posterior distribution per dose group
#' @param contr An object of class 'optContr' as created by the getContr() function. It contains the contrast matrix to be used for the testing step.
#' @param crit_prob_adj A getCritProb object, specifying the critical value to be used for the testing (on the probability scale).
#' @param simple Boolean variable, defining whether simplified fit will be applied. Passed to the getModelFits() function. Default FALSE.
#' @param avg_fit Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.
#' @param delta A numeric value for the threshold Delta for the MED assessment. If NULL, no MED assessment is performed. Default NULL.
#' @param evidence_level A numeric value between 0 and 1 for the evidence level gamma for the MED assessment. Only required for Bayesian MED assessment, see ?getMED for details. Default NULL.
#' @param med_selection A string, either "avgFit" or "bestFit" based on the lowest gAIC, for the method of MED selection. Default "avgFit".
#' @param n_samples A numerical for the number of bootstrapped samples in case the Bayesian MED assessment is performed. Default 1e3.
#' @param probability_scale A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform calculations from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.
#' @examples
#' mods <- DoseFinding::Mods(linear      = NULL,
#'                           emax        = c(0.5, 1.2),
#'                           exponential = 2,
#'                           doses       = c(0, 0.5, 2,4, 8))
#' dose_levels  <- c(0, 0.5, 2, 4, 8)
#' sd_posterior <- c(2.8, 3, 2.5, 3.5, 4)
#' contr_mat <- getContr(
#'   mods          = mods,
#'   dose_levels   = dose_levels,
#'   cov_posterior = diag(sd_posterior)^2)
#' critVal <- getCritProb(
#'   mods           = mods,
#'   dose_weights   = c(50, 50, 50, 50, 50), #reflecting the planned sample size
#'   dose_levels    = dose_levels,
#'   alpha_crit_val = 0.6) # unreasonable alpha chosen for this example, rather choose 0.05
#' prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 5), sigma = 2),
#'                    DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
#'                    DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
#'                    DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2) ,
#'                    DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2))
#' mu <- c(0, 1, 1.5, 2, 2.5)
#' S_hat <- diag(c(5, 4, 6, 7, 8)^2)
#' posterior_list <- getPosterior(
#'   prior_list = prior_list,
#'   mu_hat     = mu,
#'   S_hat      = S_hat,
#'   calc_ess   = TRUE)
#'
#' performBayesianMCPMod(posterior_list = posterior_list,
#'                       contr          = contr_mat,
#'                       crit_prob_adj  = critVal,
#'                       simple         = FALSE,
#'                       delta          = 1.1)
#'
#' @return Bayesian MCP test result as well as modeling result.
#'
#' @export
performBayesianMCPMod <- function (

  posterior_list,
  contr,
  crit_prob_adj,
  simple            = FALSE,
  avg_fit           = TRUE,

  delta             = NULL,
  evidence_level    = NULL,
  med_selection     = c("avgFit", "bestFit"),
  n_samples         = 1e3,
  probability_scale = FALSE

) {

  if (inherits(posterior_list,  "postList")) {
    posterior_list <- list(posterior_list)
  }
  checkmate::assert_list(posterior_list, types = "postList")
  
  checkmate::assert_class(contr, "optContr")
  checkmate::assert_class(crit_prob_adj, "numeric")
  checkmate::assert_logical(simple)
  checkmate::assert_logical(avg_fit)

  checkmate::assert_double(delta, null.ok = TRUE)
  checkmate::assert_double(evidence_level, lower = 0, upper = 1, null.ok = TRUE)
  checkmate::assert_character(med_selection)
  checkmate::assert_double(n_samples, lower = 1)

  med_selection <- match.arg(med_selection, choices = c("avgFit", "bestFit"))
  get_med       <- ifelse(is.null(delta), FALSE, TRUE)
  
  checkmate::assert_flag(probability_scale)

  if (!is.null(evidence_level)) {
    stopifnot("delta must not be NULL if evidence_level is not NULL" = !is.null(delta))
  }

  if (inherits(contr, "optContr")) {

    model_names <- colnames(contr$contMat)
    dose_levels <- as.numeric(rownames(contr$contMat))

  } else if (length(contr) == length(posterior_list)) {

    model_names <- colnames(contr[[1]]$contMat)
    dose_levels <- as.numeric(rownames(contr[[1]]$contMat))

  } else {

    stop ("Argument 'contr' must be of type 'optContr'")

  }
  
  if (is.null(attr(contr, "direction"))) {
    
    attr(model_names, "direction") <- "increasing"
    message("'contr' does not have attribute 'direction' and is assumed to be 'increasing'")
    
  } else {
    
    attr(model_names, "direction") <- attr(contr, "direction")
    
  }

  b_mcp <- performBayesianMCP(
    posterior_list = posterior_list,
    contr          = contr,
    crit_prob_adj  = crit_prob_adj)

  b_mod <- performBayesianMod(
    b_mcp             = b_mcp,
    posterior_list    = posterior_list,
    model_names       = model_names,
    dose_levels       = dose_levels,
    simple            = simple,
    avg_fit           = avg_fit,
    probability_scale = probability_scale)

  b_mcp_mod <- list(BayesianMCP = b_mcp, Mod = b_mod)

  if (get_med) {

    med_info <- t(sapply(b_mcp_mod$Mod, function (model_fits) {

      if (!is.null(model_fits)) { # model shape is significant

        if (!is.null(evidence_level)) { # && abs(evidence_level - 0.5) < 1e-9 # in case MED assessment with 0.5 should result in using the modelFit
          
          bs_quantile <- ifelse(test = attr(model_fits, "direction") == "increasing",
                                yes  = 1 - evidence_level,
                                no   = evidence_level)

          bs_quantiles <- getBootstrapQuantiles(
            model_fits        = model_fits,
            quantiles         = bs_quantile,
            n_samples         = n_samples,
            probability_scale = probability_scale)

          med_info <- getMED(
            delta             = delta,
            evidence_level    = evidence_level,
            bs_quantiles      = bs_quantiles,
            probability_scale = probability_scale)

        } else {

          med_info <- getMED(
            delta             = delta,
            model_fits        = model_fits,
            probability_scale = probability_scale)

        }

        if (med_selection == "avgFit") {

          return (med_info[, "avgFit"])

        } else if (med_selection == "bestFit") {

          model_min <- which.min(sapply(model_fits, function (x) x$gAIC))

          return (med_info[, model_min])

        }

      } else {

        return (c(med_reached  = 0, med = NA))

      }

    }))

    attr(b_mcp_mod, "MED")            <- med_info
    attr(b_mcp_mod, "MEDReachedFreq") <- mean(med_info[, "med_reached"])
    attr(b_mcp_mod, "MEDSelection")   <- med_selection

  }

  class(b_mcp_mod) <- "BayesianMCPMod"

  return (b_mcp_mod)

}

#' @title performBayesianMCP
#'
#' @description Performs Bayesian MCP Test step, as described in Fleischer et al. (2022).
#' Tests for a dose-response effect using a model-based multiple contrast test based on the (provided) posterior distribution. In particular for every dose-response candidate the posterior probability is calculated that the contrast is bigger than 0 (based on the posterior distribution of the dose groups).
#' In order to obtain significant test decision we consider the maximum of the posterior probabilities across the different models. This maximum is compared with a (multiplicity adjusted) critical value (on the probability scale).
#' @references Fleischer F, Bossert S, Deng Q, Loley C, Gierse J. 2022. Bayesian MCPMod. Pharmaceutical Statistics. 21(3): 654-670. doi:10.1002/pst.2193
#' @param posterior_list An object derived with getPosterior with information about the (mixture) posterior distribution per dose group
#' @param contr An object of class 'optContr' as created by the getContr() function. It contains the contrast matrix to be used for the testing step.
#' @param crit_prob_adj A getCritProb object, specifying the critical value to be used for the testing (on the probability scale)
#'
#' @examples
#' mods <- DoseFinding::Mods(linear      = NULL,
#'                           emax        = c(0.5, 1.2),
#'                           exponential = 2,
#'                           doses       = c(0, 0.5, 2,4, 8))
#' dose_levels  <- c(0, 0.5, 2, 4, 8)
#' sd_posterior <- c(2.8,3,2.5,3.5,4)
#' contr_mat <- getContr(
#'   mods          = mods,
#'   dose_levels   = dose_levels,
#'   cov_posterior = diag(sd_posterior)^2)
#' critVal <- getCritProb(
#'   mods           = mods,
#'   dose_weights   = c(50, 50, 50, 50, 50), #reflecting the planned sample size
#'   dose_levels    = dose_levels,
#'   alpha_crit_val = 0.05)
#' prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 5), sigma = 2),
#'                    DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
#'                    DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
#'                    DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2) ,
#'                    DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2))
#' mu <- c(0, 1, 1.5, 2, 2.5)
#' S_hat <- diag(c(5, 4, 6, 7, 8)^2)
#' posterior_list <- getPosterior(
#'   prior_list = prior_list,
#'   mu_hat     = mu,
#'   S_hat      = S_hat,
#'   calc_ess   = TRUE)
#'
#' performBayesianMCP(posterior_list = posterior_list,
#'                    contr          = contr_mat,
#'                    crit_prob_adj  = critVal)
#'
#' @return Bayesian MCP test result, with information about p-values for the individual dose-response shapes and overall significance
#'
#' @export
performBayesianMCP <- function(

  posterior_list,
  contr,
  crit_prob_adj

) {
  
  if (inherits(posterior_list,  "postList")) {
    
    posterior_list <- list(posterior_list)
    
  }
  checkmate::assert_list(posterior_list, types = "postList")
  
  checkmate::assert(
    checkmate::check_class(contr, "optContr"),
    checkmate::check_list(contr, types = "optContr"),
    combine = "or")
  
  checkmate::assert_class(crit_prob_adj, "numeric")
  checkmate::assert_numeric(crit_prob_adj, lower = 0, upper = Inf)

  ## What case is covered by the IF clause?
  ## -> if contrasts are re-evaluated, contr becomes a list of contrasts
  if (inherits(contr, "optContr")) {

    b_mcp <- t(sapply(posterior_list, BayesMCPi, contr, crit_prob_adj))

  } else {

    b_mcp <- t(mapply(BayesMCPi, posterior_list, contr, crit_prob_adj))

  }

  class(b_mcp)               <- "BayesianMCP"
  attr(b_mcp, "critProbAdj") <- crit_prob_adj
  attr(b_mcp, "successRate") <- mean(b_mcp[, 1])
  attr(b_mcp, "essAvg")      <- ifelse(
    test = is.na(attr(posterior_list[[1]], "ess")),
    yes  = numeric(0),
    no   = rowMeans(sapply(posterior_list,
                           function (posteriors) attr(posteriors, "ess"))))

  return (b_mcp)

}

performBayesianMod <- function (

  b_mcp,
  posterior_list,
  model_names,
  dose_levels,
  simple,
  avg_fit,
  probability_scale = FALSE

) {

  fits_list <- optPar_lapply(seq_along(posterior_list), function (i) {

    if (b_mcp[i, 1]) {

      model_fits <- getModelFits(
        models            = model_names,
        dose_levels       = dose_levels,
        posterior         = posterior_list[[i]],
        avg_fit           = avg_fit,
        simple            = simple,
        probability_scale = probability_scale)

      sign_models <- b_mcp[i, -c(1, 2)] > attr(b_mcp, "critProbAdj")
      model_fits  <- addSignificance(model_fits, sign_models)

    } else {

      NULL

    }

  })
  
  # requireNamespace("DoseFinding", quietly = TRUE)
  # requireNamespace("RBesT",       quietly = TRUE)
  # requireNamespace("nloptr",      quietly = TRUE)
  # 
  # 
  # fits_list <- parallel::mclapply(seq_along(posterior_list), function (i) {
  #   
  #   if (b_mcp[i, 1]) {
  #     
  #     model_fits <- getModelFits(
  #       models            = model_names,
  #       dose_levels       = dose_levels,
  #       posterior         = posterior_list[[i]],
  #       avg_fit           = avg_fit,
  #       simple            = simple,
  #       probability_scale = probability_scale)
  #     
  #     sign_models <- b_mcp[i, -c(1, 2)] > attr(b_mcp, "critProbAdj")
  #     model_fits  <- addSignificance(model_fits, sign_models)
  #     
  #   } else {
  #     
  #     NULL
  #     
  #   }
  #   
  # },
  # mc.cores       = 32L,
  # mc.preschedule = TRUE,
  # mc.set.seed    = FALSE)

}

test_that("performBayesianMCPMod: binary end-to-end returns MCP + fitted models (simple, fast)", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("DoseFinding")
  
  set.seed(14)
  
  dose_levels <- c(0, 1, 2, 4)
  probs <- c(0.10, 0.20, 0.30, 0.55)
  
  mu_hat <- qlogis(probs)
  se_hat <- rep(0.40, length(dose_levels))
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = mu_hat[i], s = 1.4), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(
    prior_list        = prior_list,
    mu_hat            = mu_hat,
    S_hat             = diag(se_hat^2),
    probability_scale = TRUE
  )
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    emax   = c(0.5, 1.2),
    doses  = dose_levels,
    maxEff = 2
  )
  
  n_patients <- c(50, 50, 50, 50)
  
  contr <- getContr(
    mods         = mods,
    dose_levels  = dose_levels,
    dose_weights = n_patients,
    prior_list   = prior_list
  )
  
  # NOTE: your exported entry point is performBayesianMCPMod()
  res <- performBayesianMCPMod(
    posterior_list    = post,
    contr             = contr,
    crit_prob_adj     = 0.5,
    simple            = TRUE,
    avg_fit           = TRUE,
    delta             = 0.10,
    med_selection     = "avgFit",
    n_samples         = 50,
    probability_scale = TRUE
  )
  
  expect_true(is.list(res))
  expect_true(all(c("BayesianMCP", "Mod") %in% names(res)))
  expect_true(is.matrix(res$BayesianMCP) || is.data.frame(res$BayesianMCP))
  expect_true(is.list(res$Mod))
  
  # If delta provided, your function attaches MED as attribute
  expect_true(!is.null(attr(res, "MED")))
})

# Tests for getCritProb ---------------------------------------------------



# getCritProb relies on DoseFinding, which we assumes works correctly, so the tests here are minimal

test_that("getCritProb returns the right type of value under normal case", {
  crit_pval <- getCritProb(
    mods = mods,
    dose_levels = dose_levels,
    dose_weights = n_patients,
    alpha_crit_val = alpha_crit_val
  )

  expect_type(
    crit_pval, "double"
  )

  expect_true(
    crit_pval >= 0 & crit_pval <= 1
  )
})


# Tests for getContrMat ---------------------------------------------------



# getContrMat relies on DoseFinding, which we assumes works correctly, so the tests here are minimal

test_that("getContrMat returns the right type of object under normal case", {
  contr_mat <- getContr(
    mods = mods,
    dose_levels = dose_levels,
    dose_weights = n_patients,
    prior_list = prior_list
  )

  expect_s3_class(
    contr_mat, "optContr"
  )
})

test_that("getContrMat works as expected", {
  cov_posterior <- diag(sd^2)

  contr_mat_post_sd <- getContr(
    mods          = mods,
    dose_levels   = dose_levels,
    cov_posterior = cov_posterior
  )

  se_new_trial <- c(0.3, 0.7, 0.9, 2.1)
  se_new_trial <- se_new_trial[1:2]

  contr_mat_se_new <- getContr(
    mods = mods,
    dose_levels = dose_levels,
    cov_new_trial = diag(se_new_trial^2)
  )

  # Length mismatch for se_new_trial should error
  expect_error(
    getContr(
      mods = mods,
      dose_levels = dose_levels,
      se_new_trial = se_new_trial[-1]
    )
  )


  expect_s3_class(
    contr_mat_post_sd, "optContr"
  )

  expect_no_error(contr_mat_post_sd)

  expect_s3_class(
    contr_mat_se_new, "optContr"
  )

  expect_no_error(contr_mat_se_new)


  expect_error(
    getContr(
      mods = mods,
      dose_levels = dose_levels
    )
  )
})


# Tests for performBayesianMCP --------------------------------------------



test_that("performBayesianMCP returns the right type of object under normal case", {
  data <- simulateData(
    n_patients  = n_patients,
    dose_levels = dose_levels,
    sd          = sd,
    mods        = mods,
    n_sim       = n_sim
  )

  posterior_list <- getPosterior(
    data = getModelData(data, names(mods)[1]),
    prior_list = prior_list
  )

  contr_mat <- getContr(
    mods = mods,
    dose_levels = dose_levels,
    dose_weights = n_patients,
    prior_list = prior_list
  )

  crit_pval <- getCritProb(
    mods = mods,
    dose_levels = dose_levels,
    dose_weights = n_patients,
    alpha_crit_val = alpha_crit_val
  )

  b_mcp <- performBayesianMCP(
    posterior_list = posterior_list,
    contr = contr_mat,
    crit_prob_adj = crit_pval
  )

  expect_s3_class(
    b_mcp,
    "BayesianMCP"
  )

  expect_true(
    attr(b_mcp, "critProbAdj") == crit_pval
  )

  expect_type(
    attr(b_mcp, "essAvg"), "logical"
  )

  expect_type(
    attr(b_mcp, "successRate"), "double"
  )


  expect_type(b_mcp, "double")
})


# Tests for performBayesianMCPMod -----------------------------------------



test_that("performBayesianMCPMod returns the right type of object under normal case", {
  b_mcp_mod <- performBayesianMCPMod(
    posterior_list = posterior_list,
    contr = contr_mat,
    crit_prob_adj = crit_pval
  )

  expect_s3_class(
    b_mcp_mod,
    "BayesianMCPMod"
  )

  expect_true(
    all(names(b_mcp_mod) == c("BayesianMCP", "Mod"))
  )
})


# Tests for addSignificance -----------------------------------------------


test_that("addSignificance attaches flags per model and validates input length", {
  addSignificance_fn <- tryCatch(
    getFromNamespace("addSignificance", "BayesianMCPMod"),
    error = function(e) NULL
  )
  skip_if(is.null(addSignificance_fn), "addSignificance not exported/available")

  models <- c("emax", "linear")
  dose_levels <- c(0, 1, 2, 4, 8)

  # Strongly convex pattern: tiny effects at low/mid doses, big jump at the top dose.
  posterior_list <- list(
    Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0.0, s = 0.7),  sigma = 1.2),
    DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 0.6, s = 0.7),  sigma = 1.2),
    DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.4, s = 0.7),  sigma = 1.2),
    DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 3.4, s = 0.7),  sigma = 1.2),
    DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 7.8, s = 0.7),  sigma = 1.2)
  )


  fit <- getModelFits(
    models     = models,
    posterior  = posterior_list,
    dose_levels = dose_levels
  )


  # Flags length matches -> flags should be attached per entry
  out <- addSignificance_fn(fit, c(TRUE, FALSE))
  expect_true(is.list(out) && all(names(out) == names(fit)))
  expect_false(out$linear$significant.linear)
  expect_false(out$emax$significant.emax)

  # Mismatched length should raise an error
  expect_error(addSignificance_fn(fit, list(TRUE)))
})

test_that("MCP step sanity: low type-I error under null and decent power under monotone alternative (binary)", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("DoseFinding")
  
  set.seed(202)
  
  dose_levels <- c(0, 1, 2, 4)
  n_per_dose  <- 90
  R_null <- 40
  R_alt  <- 40
  
  mods <- DoseFinding::Mods(
    linear = NULL,
    emax   = c(0.5, 1.2),
    doses  = dose_levels,
    maxEff = 2
  )
  
  # Mildly regularizing prior centered near a reasonable baseline.
  # This keeps separation-ish cases from blowing up while still letting data drive.
  make_prior <- function(p_center) {
    setNames(
      lapply(seq_along(dose_levels), function(i) {
        RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(p_center[i]), s = 2.0), sigma = 2)
      }),
      c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
    )
  }
  
  # Extract "significant" robustly.
  # Prefer structure if present; fall back to parsing printed output.
  extract_significant <- function(bmcp_obj) {
    # 1) Common possibilities
    if (is.list(bmcp_obj)) {
      for (nm in c("Significant", "significant", "sig")) {
        if (!is.null(bmcp_obj[[nm]])) return(as.integer(bmcp_obj[[nm]]))
      }
    }
    if (is.matrix(bmcp_obj) || is.data.frame(bmcp_obj)) {
      rn <- rownames(bmcp_obj)
      if (!is.null(rn) && "Significant" %in% rn) {
        return(as.integer(bmcp_obj["Significant", 1]))
      }
      if ("Significant" %in% colnames(bmcp_obj)) {
        return(as.integer(bmcp_obj[1, "Significant"]))
      }
    }
    
    # 2) Parse printed output (S3 print shows "Significant: <0/1>")
    out <- paste(capture.output(print(bmcp_obj)), collapse = "\n")
    m <- regexec("Significant:\\s*([01])", out)
    reg <- regmatches(out, m)[[1]]
    if (length(reg) >= 2) return(as.integer(reg[2]))
    
    stop("Could not extract 'Significant' indicator from BayesianMCP object")
  }
  
  run_one <- function(p_vec, crit_prob_adj = 0.95) {
    dat <- data.frame(
      simulation = 1L,
      dose       = rep(dose_levels, each = n_per_dose),
      response   = unlist(lapply(p_vec, function(p) rbinom(n_per_dose, 1, p)))
    )
    attr(dat, "probability_scale") <- TRUE
    
    prior_list <- make_prior(p_center = p_vec)
    
    post <- getPosterior(
      prior_list        = prior_list,
      data              = dat,
      probability_scale = TRUE
    )
    
    contr <- getContr(
      mods         = mods,
      dose_levels  = dose_levels,
      dose_weights = rep(n_per_dose, length(dose_levels)),
      prior_list   = prior_list
    )
    
    res <- performBayesianMCPMod(
      posterior_list    = post,
      contr             = contr,
      crit_prob_adj     = crit_prob_adj,
      simple            = TRUE,
      avg_fit           = TRUE,
      probability_scale = TRUE
    )
    
    extract_significant(res$BayesianMCP)
  }
  
  # ---- Null (flat) ----
  p_null <- rep(0.25, length(dose_levels))
  sig_null <- replicate(R_null, run_one(p_null, crit_prob_adj = 0.95))
  
  type1 <- mean(sig_null == 1)
  
  # Type-I error should be "low".
  # With only 40 reps, allow stochastic wiggle, but it should not be huge.
  expect_true(type1 <= 0.20)
  
  # ---- Alternative (monotone increasing) ----
  p_alt <- c(0.10, 0.18, 0.32, 0.55)
  sig_alt <- replicate(R_alt, run_one(p_alt, crit_prob_adj = 0.95))
  
  power <- mean(sig_alt == 1)
  
  # With n_per_dose=90 and a pretty clear signal, we expect decent power.
  expect_true(power >= 0.55)
})

#' @title getPosterior
#'
#' @description Either the patient level data or both mu_hat as well as S_hat must to be provided.
#' If patient level data is provided mu_hat and S_hat are calculated within the function using a linear model.
#' This function calculates the posterior distribution. Depending on the input for S_hat this step is either performed for every dose group independently via the RBesT function postmix() or the mvpostmix() function of the DoseFinding package is utilized.
#' In the latter case conjugate posterior mixture of multivariate normals are calculated (DeGroot 1970, Bernardo and Smith 1994)
#'
#' @param prior_list a prior list with information about the prior to be used for every dose group
#' @param data dataframe containing the information of dose and response. Also a simulateData object can be provided. Default NULL.
#' @param mu_hat vector of estimated mean values (per dose group). Default NULL.
#' @param S_hat covariance matrix specifying the (estimated) variability.
#' The variance-covariance matrix should be provided and the dimension of the matrix needs to match the number of dose groups. Default NULL.
#' @param calc_ess boolean variable, indicating whether effective sample size should be calculated. Default FALSE.
#' @param probability_scale A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform calculations from the logit scale to the probability scale, which can be desirable for a binary outcome. Default `attr(data, "probability_scale")`.
#' 
#' @references BERNARDO, Jl. M., and Smith, AFM (1994). Bayesian Theory. 81.
#' @return posterior_list, a posterior list object is returned with information about (mixture) posterior distribution per dose group (more detailed information about the conjugate posterior in case of covariance input for S_hat is provided in the attributes)
#' @details Kindly note that one can sample from the `posterior_list` with `lapply(posterior_list, RBesT::rmix, n = 10)`.
#' 
#' For binary endpoints, if separation occurs (for example, when one treatment arm has only responders or only non‑responders), we use penalized logistic regression with Firth’s correction to prevent convergence issues and obtain more stable estimates.
#' See the references for additional details.
#' 
#' @references
#' Heinze, G. & Schemper, M. (2002). A solution to the problem of separation
#' in logistic regression. *Statistics in Medicine*, 21(16), 2409–2419.
#' 
#' Liu et al. (2022). Commentary: analyzing binary data using MCPMod when zero counts are expected.
#' *arXiv*, 2202.08781, https://arxiv.org/abs/2202.08781
#' 
#' @examples
#' prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 5), sigma = 2),
#'                    DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
#'                    DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
#'                    DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2) ,
#'                    DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2))
#'                    
#' mu_hat <- c(0, 1, 1.5, 2, 2.5)
#' S_hat  <- diag(c(5, 4, 6, 7, 8)^2)
#'
#' posterior_list <- getPosterior(
#'    prior_list = prior_list,
#'    mu_hat     = mu_hat,
#'    S_hat      = S_hat)
#'
#' summary(posterior_list)
#'
#' @export
getPosterior <- function(

  prior_list,
  data              = NULL,
  mu_hat            = NULL,
  S_hat             = NULL,
  calc_ess          = FALSE,
  probability_scale = attr(data, "probability_scale")

) {

  checkmate::assert_data_frame(data, null.ok = TRUE)
  checkmate::assert_list(prior_list, names = "named", any.missing = FALSE)
  checkmate::assert_vector(mu_hat, any.missing = FALSE, null.ok = TRUE)
  checkmate::assert_double(mu_hat, null.ok = TRUE, lower = -Inf, upper = Inf)
  checkmate::assert_matrix(S_hat, mode = "numeric", any.missing = FALSE, null.ok = TRUE)
  checkmate::assert_flag(probability_scale, null.ok = TRUE)
  
  if (is.null(probability_scale)) probability_scale <- FALSE

  stopifnot("prior_list must be an object of RBesT package" =
              all(sapply(prior_list, function(x)
                methods::is(x, "normMix") |
                  methods::is(x, "betaMix") |
                  methods::is(x, "mix"))))

  if (!is.null(mu_hat) && !is.null(S_hat) && is.null(data)) {

    stopifnot("S_hat has to be a symmetrical matrix" =
              isSymmetric(S_hat))
  
    if (!isOffDiagonalZero(S_hat)) {
      
      stopifnot("dim of S_hat must equal length of prior list" =
                  dim(S_hat)[2] == length(prior_list))
      
      prior_mix <- priorList2priorMix(prior_list)
      
      post_mix <- DoseFinding::mvpostmix(
        priormix = prior_mix,
        mu_hat   = mu_hat,
        S_hat    = S_hat)
      
      posterior_list <- postMix2posteriorList(
        post_mix   = post_mix,
        prior_list = prior_list,
        calc_ess   = calc_ess)
      
    } else {
      
      ## TODO: Do we need posterior derivation based on RBesT,
      ## or should we just use DoseFinding::mvpostmix?
      ## Also interesting in terms of dependency on RBesT for validation?
      # all.equal(posterior_list2, posterior_list)
      posterior_list <- getPosteriorI(
        prior_list = prior_list,
        mu_hat     = mu_hat,
        se_hat     = diag(sqrt(S_hat)),
        calc_ess   = calc_ess)
      
    }

  } else if (is.null(mu_hat) && is.null(S_hat) && !is.null(data)) {

    posterior_list <- lapply(split(data, data$simulation), getPosteriorI,
                             prior_list        = prior_list,
                             calc_ess          = calc_ess,
                             probability_scale = probability_scale)

  } else {

    stop ("Either 'data' or 'mu_hat' and 'S_hat' must not be NULL.")

  }

  if (length(posterior_list) == 1) {

    posterior_list <- posterior_list[[1]]

  }

  return (posterior_list)

}

getPosteriorI <- function(
    
  data_i   = NULL,
  prior_list,
  mu_hat   = NULL,
  se_hat   = NULL,
  calc_ess = FALSE,
  probability_scale = attr(data_i, "probability_scale")
  
) {
  
  checkmate::check_data_frame(data_i, null.ok = TRUE)
  checkmate::check_list(prior_list, names = "named", any.missing = FALSE)
  checkmate::check_vector(mu_hat, any.missing = FALSE, null.ok = TRUE)
  checkmate::check_double(mu_hat, null.ok = TRUE, lower = -Inf, upper = Inf)
  checkmate::check_vector(se_hat, any.missing = FALSE, null.ok = TRUE)
  checkmate::check_double(se_hat, null.ok = TRUE, lower = 0, upper = Inf)
  checkmate::assert_flag(probability_scale, null.ok = TRUE)
  
  if (is.null(probability_scale)) probability_scale <- FALSE
  
  if (is.null(mu_hat) && is.null(se_hat) && !is.null(data_i)) {
    
    checkmate::check_data_frame(data_i, null.ok = FALSE)
    checkmate::assert_names(names(data_i), must.include = "response")
    checkmate::assert_names(names(data_i), must.include = "dose")
    
    if (probability_scale) {
      
      separation <- with(data_i, any(tapply(response, dose, function(y) {
        all(y == 0) | all(y == 1)
      })))
      
      if (separation) {
        
        fit_method <- "firth"
        logit_fit  <- logistf::logistf(
          formula = response ~ factor(dose) - 1,
          data    = data_i)
        
      } else {
        
        fit_method <- "glm"
        logit_fit  <- stats::glm(
          formula = response ~ factor(dose) - 1,
          data    = data_i,
          family  = stats::binomial)
        
      }
      
      mu_hat <- stats::coef(logit_fit)
      se_hat <- diag(sqrt(stats::vcov(logit_fit)))
      
    } else {
      
      fit_method <- "lm"
      anova_res  <- stats::lm(
        formula = response ~ factor(dose) - 1,
        data    = data_i)
      
      mu_hat     <- summary(anova_res)$coefficients[, 1]
      se_hat     <- summary(anova_res)$coefficients[, 2]
      
    }
    
  } else if (!is.null(mu_hat) && !is.null(se_hat) && is.null(data_i)) {
    
    fit_method <- "extern"
    
    stopifnot("m_hat length must match number of dose levels" =
                length(prior_list) == length(mu_hat))
    
  } else {
    
    stop ("Both mu_hat and se_hat or data_i must be provided.")
    
  }
  
  post_list <- mapply(RBesT::postmix, prior_list, m = mu_hat, se = se_hat,
                      SIMPLIFY = FALSE)
  
  if (is.null(names(prior_list))) {
    names(prior_list) <- c("Ctr", paste0("DG_", seq_along(post_list[-1])))
  }
  
  names(post_list) <- names(prior_list)
  class(post_list) <- "postList"
  
  attr(post_list, "ess")           <- if (calc_ess) getESS(post_list) else numeric(0)
  attr(post_list, "posteriorInfo") <- priorList2priorMix(post_list)
  attr(post_list, "fitMethod")     <- fit_method
  
  return(post_list)
  
}

#' @title getESS
#'
#' @description This function calculates the effective sample size for every dose group via `RBesT::ess()`.
#' @param post_list A posterior list object, for which the effective sample size for each dose group should be calculated
#' @param method A string specifying the method of ESS calculation, see `?RBesT::ess()`.
#' @param n_digits An integer for the number of digits the result should be rounded to.
#' @param ... Optional arguments applicable to specific methods, see `?RBesT::ess()`.
#' @return A vector of the effective sample sizes for each dose group
#'
#' @export
getESS <- function (

  post_list,
  method   = c("elir", "moment", "morita"),
  n_digits = 1,
  ...

) {
  
  checkmate::assert_list(post_list)
  checkmate::assert_integerish(n_digits, lower = 0, len = 1L)
  stopifnot(all(sapply(post_list, inherits, c("normMix", "mix"))))
  
  suppressMessages(round(sapply(post_list, RBesT::ess, method = method, ... = ...), 1))

}

getPostCompsI <- function (

  posterior_i

) {

  post_params <- list(
    weights = lapply(posterior_i, function (x) x[1, ]),
    means   = lapply(posterior_i, function (x) x[2, ]),
    vars    = lapply(posterior_i, function (x) x[3, ]^2))

  post_comps         <- lapply(post_params, expand.grid)
  post_comps$weights <- apply(post_comps$weights, 1, prod)

  return (post_comps)

}

priorList2priorMix <- function (prior_list) {

  checkmate::assert_list(prior_list, names = "named", any.missing = FALSE, null.ok = FALSE)

  # create mapping
  args          <- createMapping(prior_list)
  comp_ind      <- do.call("expand.grid", args)
  n_comps_prior <- nrow(comp_ind)

  # map information -> mapping function?
  prior_weight <- matrix(
    sapply(1:length(prior_list), function (x) sapply(1:n_comps_prior, function (y)
      prior_list[[x]][1, comp_ind[y, x]])), nrow = n_comps_prior)
  
  prior_mean   <- matrix(sapply(1:length(prior_list), function (x)
    sapply(1:n_comps_prior, function (y)
      prior_list[[x]][2, comp_ind[y, x]])), nrow = n_comps_prior)
  
  prior_sd     <- matrix(sapply(1:length(prior_list), function (x)
    sapply(1:n_comps_prior, function (y)
      prior_list[[x]][3, comp_ind[y, x]])), nrow = n_comps_prior)
  
  prior_weight <- apply(prior_weight, 1, prod)

  # create prior_mix object
  prior_weight <- as.list(prior_weight)
  prior_mean   <- lapply(asplit(prior_mean, 1), as.matrix)
  prior_vc     <- lapply(asplit(prior_sd^2, 1), diag)
  
  prior_mix    <- list(weights = prior_weight,
                       means   = prior_mean,
                       covMats = prior_vc)
  
  for (i in seq_len(3)) {
    
    names(prior_mix[[i]]) <- paste0("Comp", seq_along(prior_weight))
    
  }

  return (prior_mix)

}

postMix2posteriorList <- function (

  post_mix,
  prior_list,
  calc_ess = FALSE

) {

  checkmate::assert_list(post_mix, names = "named", any.missing = FALSE, null.ok = FALSE)
  checkmate::assert_list(prior_list, names = "named", any.missing = FALSE, null.ok = FALSE)

  getIndx <- function (x, y) which(comp_mat_ind[, x] == comp_indx[[x]][y])

  # create mapping
  comp_indx    <- createMapping(prior_list)
  comp_mat_ind <- do.call("expand.grid", comp_indx)

  # map posterior information
  posterior_weight <- lapply(seq_along(prior_list), function (x)
    sapply(seq_along(comp_indx[[x]]), function (y)
      sum(unlist(post_mix$weights[getIndx(x, y)]))))

  posterior_mean <- lapply(seq_along(prior_list), function (x)
    sapply(seq_along(comp_indx[[x]]), function (y)
      mean(do.call(cbind, post_mix$mean[getIndx(x, y)])[x, ])))

  posterior_sd <- lapply(seq_along(prior_list), function (x)
    sapply(seq_along(comp_indx[[x]]), function (y)
      mean(do.call(rbind, lapply(post_mix$covmat, diag)[getIndx(x, y)])[, x])))

  combined_vectors <- mapply(function (x, y, z)
    Map(c, x, y, z), posterior_weight, posterior_mean, lapply(posterior_sd, sqrt),
    SIMPLIFY = FALSE)

  # create posterior list
  post_list <- lapply(seq_along(combined_vectors), function (x)
    do.call(RBesT::mixnorm,
            c(combined_vectors[[x]], sigma = stats::sigma(prior_list[[x]]))))

  ## fix component names
  names(post_list) <- names(prior_list)
  comp_names <- lapply(prior_list, colnames)
  
  for (i in seq_along(post_list)) {

    colnames(post_list[[i]]) <- comp_names[[i]]

  }
  rm(i)

  ## set attributes
  class(post_list) <- "postList"
  
  attr(post_list, "ess") <- if (calc_ess) getESS(post_list) else numeric(0)

  names(post_mix) <- c("weights", "means", "covMats")
  attr(post_list, "posteriorInfo") <- post_mix

  return (post_list)

}

createMapping <- function (prior_list) {

  n_comps   <- unlist(lapply(prior_list, ncol))
  comp_indx <- lapply(seq_along(prior_list), function (x) seq_len(n_comps[x]))

  return (comp_indx)

}

isOffDiagonalZero <- function (mat) {
  
  is.matrix(mat) && all(!is.na(mat)) &&
    all(abs(mat[row(mat) != col(mat)]) < .Machine$double.eps)
  
}

test_that("Binary internal calibration: posterior mean ranks with empirical logits; SD tracks binomial information", {
  skip_if_not_installed("RBesT")
  
  set.seed(310)
  
  dose_levels <- c(0, 1, 2, 4)
  
  # Use separated but not extreme probabilities to avoid separation artifacts
  p_true <- c(0.12, 0.20, 0.33, 0.52)
  theta_true <- qlogis(p_true)
  
  # Mild priors centered near truth to stabilize CI; still allows data to dominate
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = theta_true[i], s = 2.0), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  run_once <- function(n_per_dose) {
    dat <- data.frame(
      simulation = 1L,
      dose       = rep(dose_levels, each = n_per_dose),
      response   = unlist(lapply(p_true, function(p) rbinom(n_per_dose, 1, p)))
    )
    attr(dat, "probability_scale") <- TRUE
    
    post <- getPosterior(prior_list = prior_list, data = dat, probability_scale = TRUE)
    smry <- summary(post)
    
    # Empirical proportions/logits with continuity correction to avoid +/-Inf
    p_hat <- tapply(dat$response, dat$dose, mean)
    eps <- 1/(n_per_dose + 1)
    p_hat_cc <- pmin(pmax(as.numeric(p_hat), eps), 1 - eps)
    logit_hat <- qlogis(p_hat_cc)
    
    list(
      mean = as.numeric(smry[, "mean"]),
      sd   = as.numeric(smry[, "sd"]),
      logit_hat = as.numeric(logit_hat),
      p_hat = as.numeric(p_hat_cc)
    )
  }
  
  # Compare two sample sizes to test shrinkage + scaling
  small_n <- 40
  big_n   <- 200
  
  out_small <- run_once(small_n)
  out_big   <- run_once(big_n)
  
  # --- Means: plausible & monotone (ranking) ---
  # rank(posterior mean) should match rank(empirical logit)
  expect_equal(order(out_small$mean), order(out_small$logit_hat))
  expect_equal(order(out_big$mean),   order(out_big$logit_hat))
  
  # Means should be reasonably close to empirical logits (looser at small n)
  expect_equal(out_small$mean, out_small$logit_hat, tolerance = 0.60)
  expect_equal(out_big$mean,   out_big$logit_hat,   tolerance = 0.35)
  
  # --- SD sanity ---
  expect_true(all(is.finite(out_small$sd)))
  expect_true(all(is.finite(out_big$sd)))
  expect_true(all(out_small$sd > 0))
  expect_true(all(out_big$sd > 0))
  
  # --- Shrinkage with n (must hold in aggregate) ---
  expect_true(mean(out_big$sd) < mean(out_small$sd))
  
  # --- Binomial information scaling on logit scale ---
  # For Bernoulli with logit link, Fisher info ~ n p (1-p), so sd(theta) ~ 1/sqrt(n p(1-p))
  approx_sd <- function(n, p) 1 / sqrt(n * p * (1 - p))
  
  # Compare ratio of SDs between small and big n: should be ~sqrt(big/small) up to slack
  ratio_obs <- out_small$sd / out_big$sd
  ratio_exp <- sqrt(big_n / small_n) * approx_sd(small_n, out_small$p_hat) / approx_sd(big_n, out_big$p_hat)
  # ratio_exp simplifies close to sqrt(big/small) if p's match; keep robust by computing anyway.
  
  # Require positive association between observed and expected scaling across doses
  expect_true(stats::cor(ratio_obs, ratio_exp) > 0.3)
  
  # Also require median ratio in a plausible band around sqrt(big/small) (~2.236)
  target <- sqrt(big_n / small_n)
  expect_true(stats::median(ratio_obs) > 0.9 * target)
  expect_true(stats::median(ratio_obs) < 1.6 * target)
})

# Function to test if covariance matrix is symmerical
test_covmatrix_symmetry <- function (

    posterior,
    mu_hat

  ) {

  lapply(1:(length(mu_hat)-1), function(x) {

    expect_equal(posterior$covmat$Comp1[row(posterior$covmat$Comp1) + x == col(posterior$covmat$Comp1)],
                 posterior$covmat$Comp1[row(posterior$covmat$Comp1)     == col(posterior$covmat$Comp1) + x])

    expect_equal(length(posterior$covmat$Comp1[row(posterior$covmat$Comp1) + x == col(posterior$covmat$Comp1)]),
                 length(posterior$covmat$Comp1[row(posterior$covmat$Comp1) + x       == col(posterior$covmat$Comp1)]))

  })

}

test_that("getPosterior works correctly", {
  dummy_data <- getModelData(data, names(mods)[1])
  prior_list_noRBesT <- list(1,2,3,4)

  expect_error(getPosterior(
    prior_list = prior_list_noRBesT,
    mu_hat = mu_hat,
    S_hat = se_hat_matrix,
    calc_ess = FALSE
  ))

  # Test getPosterior function
  posterior_list <- getPosterior(
    prior_list = prior_list_matrix,
    mu_hat = mu_hat,
    S_hat = diag(se_hat_vector),
    calc_ess = FALSE
  )
  expect_type(posterior_list, "list")
  expect_s3_class(posterior_list, "postList")
})

test_that("getPriorList input parameters do work as intented", {
  set.seed(8080)
  dataset     <- dplyr::filter(testdata, bname == "BRINTELLIX")
  histcontrol <- dplyr::filter(dataset, dose == 0, primtime == 8, indication == "MAJOR DEPRESSIVE DISORDER",protid!=6)

  ##Create MAP Prior
  hist_data <- data.frame(
    trial = histcontrol$nctno,
    est   = histcontrol$rslt,
    se    = histcontrol$se,
    sd    = histcontrol$sd,
    n     = histcontrol$sampsize)

  dose_levels <- c(0, 2.5, 5, 10)

  # call without parameter
  expect_error(getPriorList())
  # only passing the historical data
  expect_error(getPriorList(hist_data = hist_data))
  # passing both needed parameters
  expect_type(getPriorList(
    hist_data = hist_data,
    dose_levels = dose_levels,
    robust_weight = 0.5
  ), "list")
  # passing wrong format for hist_data
  expect_error(getPriorList(
    hist_data = testdata,
    dose_levels = dose_levels,
    robust_weight = 0.5
  ))
  # passing wrong format for dose_levels
  expect_error(getPriorList(
    hist_data = hist_data,
    dose_levels = c("hello", "world"),
    robust_weight = 0.5
  ))

})

test_that("getPosteriorI works correctly", {

  #prior_list <- list(1, 2, 3, 4)
  #mu_hat <- c(10, 20, 30, 40)
  #se_hat <- matrix(c(1, 2, 3, 4), nrow = 4, ncol = 1)

  # Test getPosteriorI function
  post_list <- getPosteriorI(data_i = NULL, prior_list = prior_list_matrix, mu_hat = mu_hat, se_hat = se_hat_vector)
  expect_type(post_list, "list")
  expect_s3_class(post_list, "postList")
  
  # Prepare test data and parameters
  data_i <- data.frame(
    dose = c(0, 1, 2, 3, 4),
    response = c(10, 20, 30, 40, 50)
  )

  # Test mu_hat and sd_hat both null branch
  post_list <- getPosteriorI(data_i, prior_list_matrix, NULL, NULL)
  expect_type(post_list, "list")
  expect_s3_class(post_list, "postList")
})

test_that("summary.postList works correctly", {
  # Prepare test data
  post_list <- list(
    Ctr = matrix(c(0.25, 10, 1), nrow = 3, ncol = 1),
    DG_1 = matrix(c(0.25, 20, 2), nrow = 3, ncol = 1),
    DG_2 = matrix(c(0.25, 30, 3), nrow = 3, ncol = 1),
    DG_3 = matrix(c(0.25, 40, 4), nrow = 3, ncol = 1)
  )
  class(post_list) <- "postList"

  # Test summary.postList function
  summary_tab <- summary.postList(post_list)
  expect_type(summary_tab, "character")
})

test_that("getPostCompsI returns an object with correct attributes", {
  posterior_i <- list(
    matrix(c(2, 1, 2), nrow = 3),
    matrix(c(2, 1, 2), nrow = 3)
  )
  result <- getPostCompsI(posterior_i)

  expect_true(is.list(result))
  expect_equal(length(result), 3)
  expect_equal(names(result), c("weights", "means", "vars"))
  expect_equal(result$weights, 4)
  expect_true(all(result$vars == c(4, 4)))
})

test_that("priorList2priorMix works correctly", {

  # function call without parameters
  expect_error(priorList2priorMix())

  # test priorList2priorMix function
  prior_mix <- priorList2priorMix(prior_list_matrix)
  expect_type(prior_mix, "list")
  expect_length(prior_mix, 3)

})

test_that("postMix2posteriorList works correctly", {

  # create posterior with matrix
  prior_mix <- priorList2priorMix(prior_list_matrix)

  posterior <- DoseFinding::mvpostmix(
    priormix = prior_mix,
    mu_hat   = mu_hat,
    S_hat    = se_hat_matrix)

  ## TODO: obsolete, as no option for vector is present any more?
  ## or re-purpose & rename test to check if both implementations yiel the same results?
  # create posterior with vector
  posterior_vector <- getPosterior(
    prior_list = prior_list_matrix,
    mu_hat = mu_hat,
    S_hat = diag(se_hat_vector^2),
    calc_ess = FALSE
  )

  ### test postMix2posteriorList function - se_hat_matrix only zeros
  posterior_list <- postMix2posteriorList(posterior, prior_list_matrix, calc_ess = FALSE)
  expect_type(posterior_list, "list")
  expect_s3_class(posterior_list, "postList")

  test_covmatrix_symmetry(posterior, mu_hat)

  # compare posterior result object with matrix to object with vector
  expect_length(posterior_list, length(posterior_vector))
  expect_length(attr(posterior_list, "covariance matrices"), length(attr(posterior_vector, "full covariance matrices")))
  expect_type(posterior_list, typeof(posterior_vector))
  expect_s3_class(posterior_list, S3Class(posterior_vector))


  ### test postMix2posteriorList function - se_hat_matrix not only zeros
  mvpostmix_noZero <- DoseFinding::mvpostmix(
    priormix = prior_mix,
    mu_hat   = mu_hat,
    S_hat    = se_hat_matrix2)

  posterior_noZero <- getPosterior(
  prior_list = prior_list_matrix,
  mu_hat     = mu_hat,
  S_hat      = se_hat_matrix2,
  calc_ess   = FALSE
  )

  expect_type(posterior_noZero, "list")
  expect_s3_class(posterior_noZero, "postList")

  test_covmatrix_symmetry(mvpostmix_noZero, mu_hat)

  # compare posterior result object with matrix to object with vector
  expect_length(posterior_noZero, length(posterior_vector))
  expect_length(attr(posterior_noZero, "covariance matrices"), length(attr(posterior_vector, "full covariance matrices")))
  expect_type(posterior_noZero, typeof(posterior_vector))
  expect_s3_class(posterior_noZero, S3Class(posterior_vector))


  ### test similarity of results for posterior from a matrix compared to posterior from a vector with square roots
  se_hat <- c(1, 2, 3, 4, 5)
  S_hat  <- diag(se_hat^2)

  posterior_matrix_S <- getPosterior(
    prior_list = prior_list_matrix,
    mu_hat     = mu_hat,
    S_hat      = S_hat,
    calc_ess   = FALSE
  )

  posterior_vector_se <- getPosterior_vec(
    prior_list = prior_list_matrix,
    mu_hat     = mu_hat,
    S_hat      = se_hat,
    calc_ess   = FALSE
  )

  expect_equal(posterior_matrix_S$Ctr, posterior_vector_se$Ctr, tolerance = 1e-3)
  expect_equal(posterior_matrix_S$DG_1, posterior_vector_se$DG_1, tolerance = 1e-3)
  expect_equal(posterior_matrix_S$DG_2, posterior_vector_se$DG_2, tolerance = 1e-3)
  expect_equal(posterior_matrix_S$DG_3, posterior_vector_se$DG_3, tolerance = 1e-3)
  expect_equal(posterior_matrix_S$DG_4, posterior_vector_se$DG_4, tolerance = 1e-3)




})

test_that("getPosterior: binary mode uses binomial GLM and returns postList", {
  skip_if_not_installed("RBesT")
  
  set.seed(10)
  
  dose_levels <- c(0, 1, 2, 4)
  n_per_dose  <- 50
  probs       <- c(0.10, 0.20, 0.35, 0.55)
  
  dat <- data.frame(
    simulation = 1L,
    dose       = rep(dose_levels, each = n_per_dose),
    response   = unlist(lapply(probs, function(p) rbinom(n_per_dose, 1, p)))
  )
  attr(dat, "probability_scale") <- TRUE
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(probs[i]), s = 1.5), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(
    prior_list        = prior_list,
    data              = dat,
    calc_ess          = TRUE,
    probability_scale = TRUE
  )
  
  expect_s3_class(post, "postList")
  expect_equal(length(post), length(dose_levels))
  expect_true(is.list(attr(post, "posteriorInfo")))
  # ess can be numeric(0) in your current implementation depending on path
  expect_true(is.null(attr(post, "ess")) || is.numeric(attr(post, "ess")))
})

test_that("getPosterior: binary mode uses Firth penalized regression in case of separation", {
  skip_if_not_installed("RBesT")
  
  set.seed(10)
  
  dose_levels <- c(0, 1, 2, 4)
  n_per_dose  <- 50
  probs       <- c(0.10, 0.20, 0.35, 0.55)
  
  dat <- data.frame(
    simulation = 1L,
    dose       = rep(dose_levels, each = n_per_dose),
    response   = unlist(lapply(probs, function(p) rbinom(n_per_dose, 1, p)))
  )
  attr(dat, "probability_scale") <- TRUE
  
  # hard-code separation
  dat[1:n_per_dose, 3] <- 0
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(probs[i]), s = 1.5), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(
    prior_list        = prior_list,
    data              = dat,
    calc_ess          = TRUE,
    probability_scale = TRUE
  )
  
  expect_s3_class(post, "postList")
  expect_equal(length(post), length(dose_levels))
  expect_true(is.list(attr(post, "posteriorInfo")))
  # ess can be numeric(0) in your current implementation depending on path
  expect_true(is.null(attr(post, "ess")) || is.numeric(attr(post, "ess")))
})

test_that("getPosterior: binary mode rejects malformed data (still needs simulation column)", {
  skip_if_not_installed("RBesT")
  
  dose_levels <- c(0, 1, 2, 4)
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 2), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  bad_dat <- data.frame(simulation = 1L, dose = c(0, 1), y = c(0, 1))
  attr(bad_dat, "probability_scale") <- TRUE
  
  expect_error(
    getPosterior(prior_list = prior_list, data = bad_dat, probability_scale = TRUE),
    regexp = "response"
  )
  
  bad_dat2 <- data.frame(simulation = 1L, dose = c(0, 0, 1, 1), response = c(0, 2, 1, 0))
  attr(bad_dat2, "probability_scale") <- TRUE
  
  # glm(binomial) should error when response is not 0/1 (or produce warnings then error downstream)
  expect_error(
    getPosterior(prior_list = prior_list, data = bad_dat2, probability_scale = TRUE)
  )
})

test_that("getPosterior: binary no-separation case uses glm branch", {
  skip_if_not_installed("RBesT")
  
  set.seed(201)
  
  dose_levels <- c(0, 1, 2, 4)
  n_per_dose  <- 80
  probs       <- c(0.10, 0.20, 0.35, 0.55)
  
  dat <- data.frame(
    simulation = 1L,
    dose       = rep(dose_levels, each = n_per_dose),
    response   = unlist(lapply(probs, function(p) rbinom(n_per_dose, 1, p)))
  )
  attr(dat, "probability_scale") <- TRUE
  
  # Ensure test setup really is non-separated
  by_group <- split(dat$response, dat$dose)
  expect_true(all(vapply(by_group, function(y) any(y == 0) && any(y == 1), logical(1))))
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(probs[i]), s = 1.5), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(
    prior_list        = prior_list,
    data              = dat,
    calc_ess          = TRUE,
    probability_scale = TRUE
  )
  
  s <- summary(post)
  
  expect_s3_class(post, "postList")
  expect_equal(length(post), length(dose_levels))
  expect_equal(attr(post, "fitMethod"), "glm")
  expect_true(is.list(attr(post, "posteriorInfo")))
  expect_true(is.numeric(attr(post, "ess")))
  expect_true(all(is.finite(s[, "mean"])))
  expect_true(all(is.finite(s[, "sd"])))
})

test_that("getPosterior: binary separation case uses firth branch", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("logistf")
  
  dose_levels <- c(0, 1, 2, 4)
  n_per_dose  <- 50
  probs       <- c(0.10, 0.20, 0.35, 0.55)
  
  dat <- data.frame(
    simulation = 1L,
    dose       = rep(dose_levels, each = n_per_dose),
    response   = unlist(lapply(probs, function(p) rbinom(n_per_dose, 1, p)))
  )
  attr(dat, "probability_scale") <- TRUE
  
  # Force separation in control arm
  dat$response[dat$dose == 0] <- 0
  
  # Ensure test setup really is separated in at least one group
  by_group <- split(dat$response, dat$dose)
  expect_true(any(vapply(by_group, function(y) all(y == 0) || all(y == 1), logical(1))))
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(probs[i]), s = 1.5), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(
    prior_list        = prior_list,
    data              = dat,
    calc_ess          = TRUE,
    probability_scale = TRUE
  )
  
  s <- summary(post)
  
  expect_s3_class(post, "postList")
  expect_equal(length(post), length(dose_levels))
  expect_equal(attr(post, "fitMethod"), "firth")
  expect_true(is.list(attr(post, "posteriorInfo")))
  expect_true(is.numeric(attr(post, "ess")))
  expect_true(all(is.finite(s[, "mean"])))
  expect_true(all(is.finite(s[, "sd"])))
})

test_that("getPosterior: binary path handles mixed simulation branches", {
  skip_if_not_installed("RBesT")
  skip_if_not_installed("logistf")
  
  dose_levels <- c(0, 1)
  n_per_dose  <- 30
  
  dat1 <- data.frame(
    simulation = 1L,
    dose = rep(dose_levels, each = n_per_dose),
    response = c(rbinom(n_per_dose, 1, 0.2), rbinom(n_per_dose, 1, 0.5))
  )
  
  dat2 <- data.frame(
    simulation = 2L,
    dose = rep(dose_levels, each = n_per_dose),
    response = c(rep(0, n_per_dose), rep(1, n_per_dose))
  )
  
  dat <- rbind(dat1, dat2)
  attr(dat, "probability_scale") <- TRUE
  
  prior_list <- setNames(
    list(
      RBesT::mixnorm(comp1 = c(w = 1, m = -1, s = 2), sigma = 2),
      RBesT::mixnorm(comp1 = c(w = 1, m =  1, s = 2), sigma = 2)
    ),
    c("Ctr", "DG_1")
  )
  
  post <- getPosterior(prior_list = prior_list, data = dat, probability_scale = TRUE)
  
  expect_type(post, "list")
  expect_equal(length(post), 2)
  expect_s3_class(post[[1]], "postList")
  expect_s3_class(post[[2]], "postList")
  expect_equal(attr(post[[1]], "fitMethod"), "glm")
  expect_equal(attr(post[[2]], "fitMethod"), "firth")
})

test_that("Binary PPC: observed counts not extreme under posterior predictive (normal approx on logit)", {
  skip_if_not_installed("RBesT")
  
  set.seed(311)
  
  dose_levels <- c(0, 1, 2, 4)
  p_true      <- c(0.10, 0.20, 0.35, 0.55)
  n_per_dose  <- 120
  
  dat <- data.frame(
    simulation = 1L,
    dose       = rep(dose_levels, each = n_per_dose),
    response   = unlist(lapply(p_true, function(p) rbinom(n_per_dose, 1, p)))
  )
  attr(dat, "probability_scale") <- TRUE
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = qlogis(p_true[i]), s = 2.0), sigma = 2)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  post <- getPosterior(prior_list = prior_list, data = dat, probability_scale = TRUE)
  
  y_obs <- as.numeric(tapply(dat$response, dat$dose, sum))
  
  smry <- summary(post)
  theta_mean <- as.numeric(smry[, "mean"])
  theta_sd   <- as.numeric(smry[, "sd"])
  
  expect_true(all(is.finite(theta_mean)))
  expect_true(all(is.finite(theta_sd)))
  expect_true(all(theta_sd > 0))
  
  ndraw <- 4000
  ppp <- numeric(length(dose_levels))
  
  for (j in seq_along(dose_levels)) {
    theta_draw <- rnorm(ndraw, mean = theta_mean[j], sd = theta_sd[j])
    p_draw <- plogis(theta_draw)
    y_rep <- rbinom(ndraw, size = n_per_dose, prob = p_draw)
    
    lo_tail <- mean(y_rep <= y_obs[j])
    hi_tail <- mean(y_rep >= y_obs[j])
    ppp[j] <- 2 * min(lo_tail, hi_tail)
  }
  
  # Not "essentially impossible under own posterior predictive"
  expect_true(all(ppp > 1e-4))
  # At least one dose looks reasonably typical
  expect_true(any(ppp > 0.02))
})

test_that("Binary posterior intervals are finite, shrink with n, and coverage is non-degenerate", {
  skip_if_not_installed("RBesT")
  
  set.seed(201)
  
  dose_levels <- c(0, 1, 2, 4)
  p_true      <- c(0.10, 0.20, 0.35, 0.55)
  theta_true  <- qlogis(p_true)
  
  prior_list <- setNames(
    lapply(seq_along(dose_levels), function(i) {
      RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 3), sigma = 3)
    }),
    c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  )
  
  run_replications <- function(n_per_dose, R) {
    covered <- matrix(FALSE, nrow = R, ncol = length(dose_levels))
    widths  <- matrix(NA_real_, nrow = R, ncol = length(dose_levels))
    
    for (r in seq_len(R)) {
      dat <- data.frame(
        simulation = 1L,
        dose       = rep(dose_levels, each = n_per_dose),
        response   = unlist(lapply(p_true, function(p) rbinom(n_per_dose, 1, p)))
      )
      attr(dat, "probability_scale") <- TRUE
      
      post <- getPosterior(
        prior_list        = prior_list,
        data              = dat,
        probability_scale = TRUE
      )
      
      lo <- as.numeric(sapply(post, function(m) stats::quantile(m, probs = 0.025)))
      hi <- as.numeric(sapply(post, function(m) stats::quantile(m, probs = 0.975)))
      
      # Always-required sanity
      if (!all(is.finite(lo)) || !all(is.finite(hi)) || !all(lo <= hi)) {
        return(list(ok = FALSE))
      }
      
      widths[r, ]  <- (hi - lo)
      covered[r, ] <- (theta_true >= lo) & (theta_true <= hi)
    }
    
    cov_rate <- colMeans(covered)
    width    <- colMeans(widths)
    
    list(
      ok = TRUE,
      cov_rate = cov_rate,
      width = width,
      cov_mean = mean(cov_rate),
      width_mean = mean(width),
      cov_min = min(cov_rate),
      cov_max = max(cov_rate)
    )
  }
  
  # Keep runtime reasonable but stable
  res_small <- run_replications(n_per_dose = 40,  R = 40)
  res_big   <- run_replications(n_per_dose = 200, R = 40)
  
  expect_true(res_small$ok)
  expect_true(res_big$ok)
  
  # 1) Width must shrink with more data (core statistical plausibility)
  expect_true(res_big$width_mean < res_small$width_mean)
  
  # 2) Coverage must be non-degenerate overall (not ~0, not identically 1)
  #    These are intentionally loose to accommodate approximate posteriors.
  expect_true(res_small$cov_mean > 0.05)
  expect_true(res_small$cov_mean < 0.999)
  
  expect_true(res_big$cov_mean > 0.05)
  expect_true(res_big$cov_mean < 0.999)
  
  # 3) It should not get dramatically worse with more data in aggregate
  #    (This catches regressions where CI computation breaks asymptotically.)
  expect_true(res_big$cov_mean + 0.20 >= res_small$cov_mean)
  
  # 4) Guard against "everything always covers" for all doses at large n
  #    If max coverage is 1.0 that's fine, but not all doses should be ~1 simultaneously.
  expect_true(mean(res_big$cov_rate > 0.999) < 0.75)
})

#' @title getPriorList
#'
#' @param hist_data historical trial summary level data,
#' needs to be provided as a dataframe. Including information of the
#' estimates and variability.
#' @param dose_levels vector of the different doseage levels
#' @param dose_names character vector of dose levels,
#' default NULL and will be automatically created
#' based on the dose levels parameter.
#' @param robust_weight needs to be provided as a numeric
#' value for the weight of the robustification component
#'
getPriorList <- function (

  hist_data,
  dose_levels,
  dose_names       = NULL,
  robust_weight

) {

  checkmate::check_data_frame(hist_data)
  checkmate::assert_double(dose_levels, lower = 0, any.missing = FALSE)
  checkmate::check_string(dose_names, null.ok = TRUE)
  checkmate::check_vector(dose_names, null.ok = TRUE, len = length(dose_levels))
  checkmate::check_numeric(robust_weight, len = 1, null.ok = FALSE)

  sd_tot <- with(hist_data, sum(sd * n) / sum(n))

  gmap <- RBesT::gMAP(
    formula    = cbind(est, se) ~ 1 | trial,
    family     = gaussian,
    weights    = hist_data$n,
    data       = hist_data,
    beta.prior = cbind(0, 100 * sd_tot),
    tau.dist   = "HalfNormal",
    tau.prior  = cbind(0, sd_tot / 4))

  prior_ctr <- RBesT::automixfit(gmap)

  prior_ctr <- suppressMessages(RBesT::robustify(
    priormix = prior_ctr,
    weight   = robust_weight,
    sigma    = sd_tot))


  prior_trt <- RBesT::mixnorm(
    comp1 = c(w = 1, m = summary(prior_ctr)[1], n = 1),
    sigma = sd_tot,
    param = "mn")

  prior_list <- c(list(prior_ctr),
                  rep(x     = list(prior_trt),
                      times = length(dose_levels[-1])))

  if (is.null(dose_names)) {
    dose_names <- c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
  }

  names(prior_list) <- dose_names

  return (prior_list)

}


# read in testdata --------------------------------------------------------

# testdata <- readRDS(file.path(getwd(), "tests/testthat/data/testdata.RDS"))
testdata <- readRDS("data/testdata.RDS")

# further setup -----------------------------------------------------------



# Create minimal test case
n_hist_trials = 2

hist_data <- data.frame(
  trial = seq(1, n_hist_trials, 1),
  est   = rep(1, n_hist_trials),
  se    = rep(1, n_hist_trials),
  sd    = rep(1, n_hist_trials),
  n     = rep(1, n_hist_trials)
)

n_patients <- c(2, 3)
dose_levels <- c(0, 2.5)
mean <- c(8, 12)
sd <- c(0.5, 0.8)

mods <- DoseFinding::Mods(
  linear = NULL,
  doses = dose_levels
)


prior_list <- getPriorList(
  hist_data   = hist_data,
  dose_levels = dose_levels,
  robust_weight = 0.5
)

n_sim = 1
alpha_crit_val = 0.05
simple = TRUE

data <- simulateData(
  n_patients  = n_patients,
  dose_levels = dose_levels,
  sd          = sd,
  mods        = mods,
  n_sim       = n_sim
)

posterior_list <- getPosterior(
  data = getModelData(data, names(mods)[1]),
  prior_list = prior_list
)

contr_mat = getContr(
  mods = mods,
  dose_levels = dose_levels,
  dose_weights = n_patients,
  prior_list = prior_list
)

crit_pval = getCritProb(
  mods = mods,
  dose_levels = dose_levels,
  dose_weights = n_patients,
  alpha_crit_val = alpha_crit_val
)

# eval_design <- assessDesign(
#   n_patients = n_patients,
#   mods = mods,
#   prior_list = prior_list,
#   n_sim = n_sim,
#   alpha_crit_val = alpha_crit_val,
#   simple = TRUE
# )

# Create covmat test case
mixnorm_test <- RBesT::mixnorm(comp1=c(0.2,2,3), comp2=c(0.2,5,6), comp3=c(0.2,8,9), comp4=c(0.2,11,12), robust=c(0.2,14,15), sigma=9.734)

mixnorm_DG1  <- RBesT::mixnorm(comp1=c(0.5,2,3), comp2=c(0.5,3,6), sigma=9.651)

mixnorm_DG2  <- RBesT::mixnorm(comp1=c(1,8,2), sigma=9.932)

mixnorm_DG3  <- RBesT::mixnorm(comp1=c(1/3,6,8), comp2=c(1/3,7,9), comp3=c(1/3,0.5,1), sigma=9.134)

mixnorm_DG4  <- RBesT::mixnorm(comp1=c(1/3,10,3), comp2=c(1/3,3,6), comp3=c(1/3,4,7), sigma=9.236)

prior_list_matrix <- vector("list", 5)
prior_list_matrix[[1]] <- mixnorm_test
prior_list_matrix[[2]] <- mixnorm_DG1
prior_list_matrix[[3]] <- mixnorm_DG2
prior_list_matrix[[4]] <- mixnorm_DG3
prior_list_matrix[[5]] <- mixnorm_DG4

names(prior_list_matrix) <- c("Ctr","DG_1","DG_2","DG_3","DG_4")

mu_hat <- c(10, 20, 30, 40, 50)
se_hat_vector <- c(1.0, 3.0, 5.0, 9.0, 6.0)
se_hat_vector_sqrt <- sqrt(se_hat_vector)

se_hat_matrix <- matrix(c(1.00, 0.00, 0.00, 0.00, 0.00,
                          0.00, 3.00, 0.00, 0.00, 0.00,
                          0.00, 0.00, 5.00, 0.00, 0.00,
                          0.00, 0.00, 0.00, 9.00, 0.00,
                          0.00, 0.00, 0.00, 0.00, 6.00), nrow = 5, ncol = 5)

se_hat_matrix2 <- matrix(c(1.00, 0.10, 0.20, 0.30, 0.40,
                           0.10, 3.00, 0.10, 0.20, 0.30,
                           0.20, 0.10, 5.00, 0.10, 0.20,
                           0.30, 0.20, 0.10, 9.00, 0.10,
                           0.40, 0.30, 0.20, 0.10, 6.00), nrow = 5, ncol = 5)

posterior <- getPosterior(
  prior_list = prior_list_matrix,
  mu_hat     = mu_hat,
  S_hat      = se_hat_matrix,
  calc_ess   = FALSE
)

posterior_noZero <- getPosterior(
  prior_list = prior_list_matrix,
  mu_hat     = mu_hat,
  S_hat      = se_hat_matrix2,
  calc_ess   = FALSE
)


# include vector based old getPosterior Function --------------------------

getPosterior_vec <- function(

  prior_list,
  data     = NULL,
  mu_hat   = NULL,
  S_hat    = NULL,
  calc_ess = FALSE

) {

  checkmate::check_data_frame(data, null.ok = TRUE)
  checkmate::check_list(prior_list, names = "named", any.missing = FALSE)
  checkmate::check_vector(mu_hat, any.missing = FALSE, null.ok = TRUE)
  checkmate::check_double(mu_hat, null.ok = TRUE, lower = -Inf, upper = Inf)
  checkmate::check_vector(S_hat, any.missing = FALSE, null.ok = TRUE)
  checkmate::check_double(S_hat, null.ok = TRUE, lower = 0, upper = Inf)

  stopifnot("prior_list must be an object of RBesT package" =
              all(sapply(prior_list, function(x)
                methods::is(x, "normMix") |
                  methods::is(x, "betaMix") |
                  methods::is(x, "mix"))))

  if (!is.null(mu_hat) && !is.null(S_hat) && is.null(data)) {

    if (is.matrix(S_hat)) {

      is_matrix_S_hat <- TRUE

    } else if (is.vector(S_hat)) {

      is_matrix_S_hat <- FALSE

      se_hat <- S_hat
      rm(S_hat)

    } else {

      stop("S_hat has to be either a vector or matrix")

    }

    if (is_matrix_S_hat) {

      stopifnot("dim of S_hat must equal length of prior list" =
                  dim(S_hat)[2] == length(prior_list))

      prior_mix <- priorList2priorMix_vec(prior_list)

      posterior <- DoseFinding::mvpostmix(
        priormix = prior_mix,
        mu_hat   = mu_hat,
        S_hat    = S_hat)

      posterior_list <- postMix2posteriorList_vec(
        posterior_list = posterior,
        prior_list     = prior_list,
        calc_ess       = calc_ess)

    } else {

      posterior_list <- getPosteriorI_vec(
        prior_list = prior_list,
        mu_hat     = mu_hat,
        se_hat     = se_hat,
        calc_ess   = calc_ess)

    }

  } else if (is.null(mu_hat) && is.null(S_hat) && !is.null(data)) {

    posterior_list <- lapply(split(data, data$simulation), getPosteriorI_vec,
                             prior_list = prior_list, calc_ess = calc_ess)

  } else {

    stop ("Either 'data' or 'mu_hat' and 'se_hat' must not be NULL.")

  }

  if (length(posterior_list) == 1) {

    posterior_list <- posterior_list[[1]]

  }

  return (posterior_list)

}

getPosteriorI_vec <- function(

  data_i   = NULL,
  prior_list,
  mu_hat   = NULL,
  se_hat   = NULL,
  calc_ess = FALSE

) {

  checkmate::check_data_frame(data_i, null.ok = TRUE)
  checkmate::check_list(prior_list, names = "named", any.missing = FALSE)
  checkmate::check_vector(mu_hat, any.missing = FALSE, null.ok = TRUE)
  checkmate::check_double(mu_hat, null.ok = TRUE, lower = -Inf, upper = Inf)
  checkmate::check_vector(se_hat, any.missing = FALSE, null.ok = TRUE)
  checkmate::check_double(se_hat, null.ok = TRUE, lower = 0, upper = Inf)

  if (is.null(mu_hat) && is.null(se_hat)) {

    checkmate::check_data_frame(data_i, null.ok = FALSE)
    checkmate::assert_names(names(data_i), must.include = "response")
    checkmate::assert_names(names(data_i), must.include = "dose")

    anova_res <- stats::lm(data_i$response ~ factor(data_i$dose) - 1)
    mu_hat    <- summary(anova_res)$coefficients[, 1]
    se_hat    <- summary(anova_res)$coefficients[, 2]

  } else if (!is.null(mu_hat) && !is.null(se_hat)) {

    stopifnot("m_hat length must match number of dose levels" =
                length(prior_list) == length(mu_hat))
    # ,
    #           "se_hat length must match number of dose levels" =
    #             length(prior_list) == length(se_hat))

  } else {

    stop ("Both mu_hat and se_hat must be provided.")

  }

  post_list <- mapply(RBesT::postmix, prior_list, m = mu_hat, se = se_hat,
                      SIMPLIFY = FALSE)

  if (is.null(names(prior_list))) {

    names(prior_list) <- c("Ctr", paste0("DG_", seq_along(post_list[-1])))

  }

  names(post_list) <- names(prior_list)
  class(post_list) <- "postList"

  comp_indx    <- createMapping(prior_list)
  comp_mat_ind <- do.call("expand.grid", comp_indx)

  attr(post_list, "ess") <- if (calc_ess) getESS(post_list) else numeric(0)

  diagonals <- lapply(seq_along(comp_mat_ind[, 1]), function(x) {

    lapply(seq_along(comp_mat_ind[x, ]), function(y) return(post_list[[y]]["s", comp_mat_ind[x,y]]))

  })

  attr(post_list, "full covariance matrices") <- lapply(seq_along(diagonals), function(x) diag(diagonals[[x]]))

  return (post_list)

}
priorList2priorMix_vec <- function (prior_list) {

  checkmate::assert_list(prior_list, names = "named", any.missing = FALSE, null.ok = FALSE)

  # create mapping
  args <- createMapping(prior_list)
  comp_ind <- do.call("expand.grid", args)
  n_comps_prior <- nrow(comp_ind)

  # map information -> mapping function?
  prior_weight <- matrix(
    sapply(1:length(prior_list), function (x) sapply(1:n_comps_prior,
                                                     function (y) prior_list[[x]][1, comp_ind[y, x]])), nrow = n_comps_prior)

  prior_mean   <- matrix(sapply(1:length(prior_list), function (x) sapply(1:n_comps_prior, function (y) prior_list[[x]][2, comp_ind[y, x]])), nrow = n_comps_prior)
  prior_sd     <- matrix(sapply(1:length(prior_list), function (x) sapply(1:n_comps_prior, function (y) prior_list[[x]][3, comp_ind[y, x]])), nrow = n_comps_prior)

  prior_weight <- apply(prior_weight, 1, prod)

  # create prior_mix object
  prior_weight <- as.list(prior_weight)
  prior_mean   <- asplit(prior_mean, 1)
  prior_vc     <- lapply(asplit(prior_sd^2, 1), diag)
  prior_mix    <- list(prior_weight, prior_mean, prior_vc)

  return (prior_mix)

}

postMix2posteriorList_vec <- function (

  posterior_list,
  prior_list,
  calc_ess = FALSE

) {

  checkmate::assert_list(posterior_list, names = "named", any.missing = FALSE, null.ok = FALSE)
  checkmate::assert_list(prior_list, names = "named", any.missing = FALSE, null.ok = FALSE)

  getIndx       <- function (x, y) which(comp_mat_ind[, x] == comp_indx[[x]][y])

  # create mapping
  comp_indx    <- createMapping(prior_list)
  comp_mat_ind <- do.call("expand.grid", comp_indx)

  # map posterior information
  posterior_weight <- lapply(seq_along(prior_list), function (x)
    sapply(seq_along(comp_indx[[x]]), function (y)
      sum(unlist(posterior_list$weights[getIndx(x, y)]))))

  posterior_mean <- lapply(seq_along(prior_list), function (x)
    sapply(seq_along(comp_indx[[x]]), function (y)
      mean(do.call(cbind, posterior_list$mean[getIndx(x, y)])[x, ])))

  posterior_sd <- lapply(seq_along(prior_list), function (x)
    sapply(seq_along(comp_indx[[x]]), function (y)
      mean(do.call(rbind, lapply(posterior_list$covmat, diag)[getIndx(x, y)])[, x])))

  combined_vectors <- mapply(function (x, y, z)
    Map(c, x, y, z), posterior_weight, posterior_mean, lapply(posterior_sd, sqrt),
    SIMPLIFY = FALSE)

  # create posterior list
  posterior_list_RBesT <- lapply(seq_along(combined_vectors), function (x)
    do.call(RBesT::mixnorm,
            c(combined_vectors[[x]], sigma = stats::sigma(prior_list[[x]]))))

  ## fix component names
  names(posterior_list_RBesT) <- names(prior_list)
  comp_names <- lapply(prior_list, colnames)
  for (i in seq_along(posterior_list_RBesT)) {

    colnames(posterior_list_RBesT[[i]]) <- comp_names[[i]]

  }
  rm(i)

  ## set attributes
  class(posterior_list_RBesT) <- "postList"
  attr(posterior_list_RBesT, "ess") <- calcEss(calc_ess, posterior_list_RBesT)

  diagonals <- lapply(seq_along(comp_mat_ind[, 1]), function(x) {

    lapply(seq_along(comp_mat_ind[x, ]), function(y) return(posterior_list_RBesT[[y]]["s", comp_mat_ind[x,y]]))

  })

  attr(posterior_list_RBesT, "covariance matrices") <- lapply(seq_along(diagonals), function(x) diag(diagonals[[x]]))

  return (posterior_list_RBesT)

}

calcEss_vec <- function(calc_ess, posterior_output) {

  checkmate::assert_logical(calc_ess, null.ok = FALSE, len = 1)
  checkmate::assert_list(posterior_output, names = "named", any.missing = FALSE, null.ok = FALSE)

  if (calc_ess) {

    post_ESS <- getESS(posterior_output)

  } else {

    post_ESS <- numeric(0)

  }

  return(post_ESS)

}

createMapping_vec <- function (prior_list) {

  n_comps   <- unlist(lapply(prior_list, ncol))
  comp_indx <- lapply(seq_along(prior_list), function (x) seq_len(n_comps[x]))

  return (comp_indx)

}

#' @title plot.modelFits
#'
#' @description Plot function based on the ggplot2 package. Providing visualizations for each model and a average Fit.
#' Black lines show the fitted dose response models and an AIC based average model.
#' Dots indicate the posterior median and vertical lines show corresponding credible intervals (i.e. the variability of the posterior distribution of the respective dose group).
#' To assess the uncertainty of the model fit one can in addition visualize credible bands (default coloring as orange shaded areas).
#' The calculation of these bands is performed via the `getBootstrapQuantiles()` function.
#' The default setting is that these credible bands are not calculated.
#' @param x An object of type `modelFits`
#' @param probability_scale A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform the output from the logit scale to the probability scale, which can be desirable for a binary outcome. Default `attr(x, "probability_scale")`.
#' @param gAIC Logical value indicating whether gAIC values are shown in the plot. Default TRUE
#' @param cr_intv Logical value indicating whether credible intervals are included in the plot. Default TRUE
#' @param alpha_CrI Numerical value of the width of the credible intervals. Default is set to 0.05 (i.e 95% CI are shown).
#' @param cr_bands Logical value indicating whether bootstrapped based credible bands are shown in the plot. Default FALSE
#' @param alpha_CrB Numerical vector of the width of the credible bands. Default is set to `c(0.05, 0.2, 1)`, i.e, the 95% CB, 80% CB and bootstrapped median are shown.
#' @param n_bs_smpl Number of bootstrap samples being used. Default 1000.
#' @param acc_color Color of the credible bands. Default `"orange"`.
#' @param plot_res Number of plotted doses within the range of the dose levels, i.e., the resolution of the plot. Default 100.
#' @param plot_diffs Logical flag. Plot differences to the control group. Default FALSE.
#' @param ... optional parameter to be passed to `plot()`.
#' @examples
#' posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
#'                        DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
#'                        DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2) ,
#'                        DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2) ,
#'                        DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
#' models <- c("exponential", "linear", "emax")
#' dose_levels <- c(0, 1, 2, 4, 8)
#' model_fits <- getModelFits(models      = models,
#'                            posterior   = posterior_list,
#'                            dose_levels = dose_levels,
#'                            simple      = TRUE)
#'
#' plot(model_fits)
#' 
#' # plot with credible bands
#' plot(model_fits,
#'      cr_bands          = TRUE,
#'      plot_diffs        = FALSE,
#'      probability_scale = FALSE,
#'      n_bs_smpl         = 1e2) # speeding up example run-time
#' 
#' @return A ggplot2 object
#' @export
plot.modelFits <- function (

  x,
  probability_scale = attr(x, "probability_scale"),
  gAIC              = TRUE,
  cr_intv           = TRUE,
  alpha_CrI         = 0.05,
  cr_bands          = FALSE,
  alpha_CrB         = c(0.05, 0.2, 1),
  n_bs_smpl         = 1e3,
  acc_color         = "orange",
  plot_res          = 1e2,
  plot_diffs        = FALSE,
  ...

) {
  
  ## R CMD --as-cran appeasement
  .data <- q_prob <- q_val <- model <- sample_type <- NULL

  checkmate::assert_flag(gAIC)
  checkmate::assert_flag(cr_intv)
  checkmate::assert_double(alpha_CrI, lower = 0, upper = 1, len = 1L)
  checkmate::assert_flag(cr_bands)
  checkmate::assert_double(alpha_CrB, lower = 0, upper = 1, min.len = 1L)
  checkmate::assert_integerish(n_bs_smpl, lower = 1, upper = Inf)
  checkmate::assert_string(acc_color, na.ok = TRUE)
  checkmate::assert_integerish(plot_res, lower = 1, upper = Inf)
  checkmate::assert_flag(probability_scale, null.ok = TRUE)
  checkmate::assert_flag(plot_diffs)
  
  if (is.null(probability_scale)) probability_scale <- FALSE

  model_fits  <- x
  model_names <- names(model_fits)
  avg_fit     <- "avgFit" %in% model_names

  dose_levels <- model_fits[[1]]$dose_levels
  dose_seq    <- seq(from       = min(dose_levels),
                     to         = max(dose_levels),
                     length.out = plot_res)
  
  preds_models <- do.call(c, lapply(
    predict.modelFits(model_fits, doses = dose_seq,
                      probability_scale = probability_scale),
    function (preds) if (plot_diffs) preds - preds[1] else preds))

  quantile_probs <- c(alpha_CrI / 2, 0.5, 1 - alpha_CrI / 2)
  if (plot_diffs) {
    
    if (probability_scale) {
      
      ## Sampling from posterior distributions
      post_samples   <- RBesT::inv_logit(sapply(attr(model_fits, "posterior"), RBesT::rmix, n = n_bs_smpl))
      post_quantiles <- apply(post_samples - post_samples[, 1], 2, stats::quantile, probs = quantile_probs)
      
    } else {
      
      post_quantiles <- sapply(attr(model_fits, "posterior"),
                               RBesT::qmixdiff,
                               mix2 = attr(model_fits, "posterior")[[1]],
                               p    = quantile_probs)
      post_quantiles[c(1, 3), 1] <- 0
      
    }
    
  } else {
    
    post_quantiles <- sapply(attr(model_fits, "posterior"),
                             RBesT::qmix,
                             p = quantile_probs)
    
    if (probability_scale) post_quantiles <- RBesT::inv_logit(post_quantiles)
    
  }
  
  ## ggplot data frama
  gg_data <- data.frame(
    doses  = rep(dose_seq, length(model_names)),
    fits   = as.vector(preds_models),
    models = rep(factor(model_names, levels = model_names),
                 each = plot_res))

  if (gAIC) {

    g_AICs     <- sapply(model_fits, function (x) x$gAIC)
    label_gAUC <- paste("AIC:", round(g_AICs, digits = 1))

    if (avg_fit) {
      
      mod_weights <- sapply(model_fits, function (y) y$model_weight)[-1]

      mod_weights <- sort(mod_weights, decreasing = TRUE)
      paste_names <- shortenModelNames(names(mod_weights))

      label_avg  <- paste0(paste_names, "=", round(mod_weights, 1),
                           collapse = ", ")
      label_gAUC <- paste0(" ", c(label_avg, label_gAUC[-1]))

    }

  }

  if (cr_bands) {
  
    quantile_pairs <- matrix(
      data = c(alpha_CrB / 2, rep(1, times = length(alpha_CrB))),
      ncol = 2)
    quantile_pairs[, 2] <- quantile_pairs[, 2] - quantile_pairs[, 1]

    crB_data <- getBootstrapQuantiles(
      model_fits        = model_fits,
      n_samples         = n_bs_smpl,
      quantiles         = sort(unique(c(as.vector(quantile_pairs)))),
      doses             = dose_seq,
      probability_scale = probability_scale) |>
      dplyr::filter(sample_type == dplyr::if_else(plot_diffs, "diff", "abs")) |>
      tidyr::pivot_wider(names_from = q_prob, values_from = q_val) |>
      dplyr::rename(models = model)

  }

  plts <- ggplot2::ggplot() +
    ## Layout etc.
    ggplot2::theme_bw() +
    ggplot2::labs(x = "Dose",
                  y = "Model Fits") +
    # ggplot2::scale_x_continuous(breaks = c(dose_levels)) +
    ggplot2::theme(panel.grid.major = ggplot2::element_blank(),
                   panel.grid.minor = ggplot2::element_blank()) +
    ## gAIC
    {if (gAIC) {

      ggplot2::geom_text(
        data    = data.frame(
          models = unique(gg_data$models),
          label  = label_gAUC),
        mapping = ggplot2::aes(label = label_gAUC),
        x = -Inf, y = Inf, hjust = "inward", vjust = "inward",
        size = 3)

      }
    }

  if (cr_bands) {
    
    ## Bootstrapped Credible Bands
    for (i in seq_len(nrow(quantile_pairs))) {

      q_pair <- quantile_pairs[i, ]

      loop_txt <- paste0(
        "ggplot2::geom_ribbon(
          data    = crB_data,
          mapping = ggplot2::aes(x    = .data$dose,
                                 ymin = .data$'", as.character(q_pair[1]),"',
                                 ymax = .data$'", as.character(q_pair[2]),"'),
          fill    = acc_color,
          alpha   = 0.25)")

      plts <- plts + eval(parse(text = loop_txt))

    }
    rm(i)
    
    if (0.5 %in% quantile_pairs) {
      
      ## Bootstrapped Fit
      plts <- plts +
        ggplot2::geom_line(
          data    = crB_data,
          mapping = ggplot2::aes(.data$dose, .data$`0.5`),
          color   = acc_color)
      
    }

  }

  plts <- plts +
    ## Posterior Credible Intervals
    {if (cr_intv) {

      ggplot2::geom_errorbar(
        data    = data.frame(x    = dose_levels,
                             ymin = post_quantiles[1, ],
                             ymax = post_quantiles[3, ]),
        mapping = ggplot2::aes(x    = .data$x,
                               ymin = .data$ymin,
                               ymax = .data$ymax),
        width   = 0,
        alpha   = 0.5)

    }
    } +
    ## Posterior Medians
    ggplot2::geom_point(
      data    = data.frame(
        dose_levels = dose_levels,
        medians     = post_quantiles[2, ]),
      mapping = ggplot2::aes(.data$dose_levels, .data$medians),
      size    = 2) +
    ## Fitted Models
    ggplot2::geom_line(
      data    = gg_data,
      mapping = ggplot2::aes(.data$doses, .data$fits)) +
    ## Faceting
    ggplot2::facet_wrap(~ models)

  return (plts)

}

---
title: "Trial Analysis Example of Bayesian MCPMod for Binary Data"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Trial Analysis Example of Bayesian MCPMod for Binary Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: references.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

<details>
<summary><strong>Show code</strong></summary>
```{r setup}
#| message: false
#| warning: false

suppressPackageStartupMessages({
  library(BayesianMCPMod)
  library(RBesT)
  library(DoseFinding)
  library(dplyr)
})

set.seed(7015)

display_params_table <- function(named_list) {
  round_numeric <- function(x, digits = 3) if (is.numeric(x)) round(x, digits) else x
  tbl <- data.frame(
    Name  = names(named_list),
    Value = I(lapply(named_list, function(v) {
      if (inherits(v, "Date")) v <- as.character(v)
      if (!is.null(names(v))) paste0("{", paste(names(v), v, sep="=", collapse=", "), "}")
      else v
    }))
  )
  tbl$Value <- lapply(tbl$Value, round_numeric)
  knitr::kable(tbl)
}
```
</details>

# Introduction

This vignette demonstrates the application of the `BayesianMCPMod` package for 
a binary endpoint. A more detailed introduction is provided for the setting of a continuous endpoint ([analysis example vignette](analysis_normal.html)).

Binary endpoints require modeling on the logit scale.
We will use the migraine dataset from the `DoseFinding` package as our working example, which contains response rates after migraine treatment. The prior (for the control group) will be based on historical trial data.

This package makes use of the [future](https://cran.r-project.org/package=future) framework for parallel processing, which can be set up for example as follows:
```{r, eval = FALSE}
future::plan(future::multisession, workers = 4L)
```
Kindly note that due to overhead a reduced number of worker nodes can be preferable and that for short calculations sequential execution can be faster.

## Scale Conventions in `BayesianMCPMod` 

<div class="alert alert-info">

- Internally, `BayesianMCPMod` fits binary endpoints on the **logit scale**.
- The function argument `probability_scale` controls whether outputs (summaries, predictions, plots)
  are back-transformed to **probabilities**.
- In `simulateData()`, outcomes are simulated on the response scale if `probability_scale = TRUE`.
- In `getMED()`, `delta` is interpreted on the **probability scale** if `probability_scale = TRUE`.

</div>


# Calculation of a MAP Prior
For this example, in a first step, a meta analytic prior will be calculated.
This prior is based on trials results for @Diener2011, @Ho2008 and @Hewitt2011.
Here, we assume the following historical results for the control group.
Please note that only information from the control group will be integrated, leading to an informative mixture prior for the control group, while a non-informative prior will be specified for the active groups.

<details>
<summary><strong>Show code</strong></summary>
```{r}
trial <- c("trial_1", "trial_2", "trial_3")
n     <- c(70,  115, 147) # sample size per trial
r     <- c( 6,   16,  16) # n responders per trial
```
</details>

Our approach to establish a MAP prior is conducted in 3 steps. 
First the information from the historical trials is used to establish a beta mixture MAP prior (family=binomial).  
In a next step this prior is robustified.
Finally, since the `BayesianMCPMod` procedures for binary endpoints require a prior on the logit scale, we translate this prior to this scale via sampling from the distribution, translating the results to the logit scale and approximating via fitting of normal mixtures of conjugate distributions. 
Please note that there would be other options to establish a reasonable informative prior in this setting. 

<details>
<summary><strong>Show code</strong></summary>
```{r}
dose_levels <- c(0, 2.5, 5, 10, 20, 50, 100, 200)

# 1) Establish MAP prior (beta mixture distribution)
set.seed(7015) # re-set seed only for this example; remove in your analysis script
map <- gMAP(
  cbind(r, n - r) ~ 1 | trial,
  family     = binomial,
  tau.dist   = "HalfNormal",
  tau.prior  = 0.5,
  beta.prior = (1 / sqrt(0.1 * 0.9)),
  warmup     = 1000,
  iter       = 10000,
  chains     = 2,
  thin       = 1
)
map

prior <- automixfit(map) #fits mixture distribution from MCMC samples from above
p     <- summary(prior)[1]

# 2) Robustify prior
prior_rob <- RBesT::robustify(priormix = prior,
                              mean     = 0.5,
                              weight   = 0.4)

# 3) Translate prior to logit scale (to approximate via normal mixture model)
r                <- rmix(prior_rob, n = 1e4)
log_r            <- RBesT::logit(r)
prior_ctr        <- automixfit(log_r, type = "norm")

# Specification of reference scale (this follows the idea of [@Neuenschwander2016]).
sigma(prior_ctr) <- sqrt(1 / (p * (1 - p)))

# Specify a prior list
prior_trt <- RBesT::mixnorm(
  comp1 = c(
    w = 1,
    m = logit(summary(prior)[1]),
    n = 1
  ),
  sigma = sqrt(1 / (p * (1 - p))),
  param = "mn"
)

prior_list <- c(list(prior_ctr),
                rep(x     = list(prior_trt),
                    times = length(dose_levels[-1])))

dose_names        <- c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
names(prior_list) <- dose_names
```
</details>

# Dose-Response Model Shapes

Candidate models are specified on the parameter scale using the {DoseFinding} package.
We will create a `Mods` object, which will be used in the remainder of the vignette.
Please note that the models are specified on the **logit scale**.

```{r}
models <- Mods(
  linear      = NULL,
  sigEmax     = c(50, 3),
  quadratic   = -1 / 250,
  logistic    = c(110, 15),
  exponential = 80,
  emax        = 10,
  doses       = dose_levels,
  placEff     = RBesT::logit(0.118),
  maxEff      = RBesT::logit(0.3) - RBesT::logit(0.118)
)

plot(models)
```

## Trial Data

We will use the trial data from the migraine data set available in the `DoseFinding` package as our phase 2 trial data.
We will apply a logistic regression (without any additional covariates) to get estimates on the logit scale.

```{r}
data("migraine") # data set from the DoseFinding package

doses_fact <- as.factor(dose_levels)
n_patients <- migraine$ntrt
resp_rate  <- migraine$painfree/n_patients

## Execution of logistic regression and readout of parameters 
## Note that estimates are automatically on the logit scale.
log_fit <- glm(resp_rate ~ doses_fact - 1, family = binomial, weights = n_patients)
mu_hat  <- coef(log_fit)
S_hat   <- vcov(log_fit)
```

# Posterior Calculation

In the first step of Bayesian MCPMod, the posterior is calculated by combining the prior information with the estimated results of the trial [@fleischer_2022]. 
```{r}
post_logit <- getPosterior(prior_list, mu_hat = mu_hat, S_hat  = S_hat)
```
The summary of the posterior can be provided on the probability scale.
```{r}
summary(post_logit, probability_scale = TRUE)
```

# Bayesian MCPMod Test Step

The testing step of Bayesian MCPMod is executed using a critical value on the probability scale and a pseudo-optimal contrast matrix. 

A contrast matrix is generated based on the number of patients per dose group, see @fleischer_2022 for more details.
This choice is motivated by the assessments reported in @liu2022commentaryanalyzingbinarydata, which demonstrate that this type of contrast is robust and provides good statistical power across a broad range of potential outcome scenarios.
Please note that also other options would be possible, e.g. using weight based on the observed variability.

The critical value is calculated using (re-estimated) contrasts for frequentist MCPMod to ensure error control when using weakly-informative priors.

```{r}
contr_mat_prior <- getContr(
  mods           = models,
  dose_levels    = dose_levels,
  dose_weights   = n_patients)

set.seed(7015) # re-sets seed only for this example; remove in your analysis script
crit_pval <- getCritProb(
  mods           = models,
  dose_levels    = dose_levels,
  cov_new_trial  = S_hat,
  alpha_crit_val = 0.05
)
```

The Bayesian MCP testing step is then executed:  
```{r}
BMCP_result <- performBayesianMCP(
  posterior_list = post_logit,
  contr          = contr_mat_prior, 
  crit_prob_adj  = crit_pval)
```

Here as well it should be noted that this evaluation happens on the logit scale.
```{r}
BMCP_result
```
The testing step is significant, indicating a non-flat dose-response shape.
All model shapes are significant.

# Model Fitting and Visualization

In the model fitting step the posterior distribution is used as basis.

Both simplified and full fitting can be performed. Here we are focusing on the simplified fit. Furthermore we specify that the fit should be provided on the probability scale for easier interpretation of results.

The output of the fit includes information about the predicted effects for the included dose levels, the generalized AIC, and the corresponding weights.
```{r}
model_fits <- getModelFits(
  models            = models,
  dose_levels       = dose_levels,
  posterior         = post_logit,
  simple            = TRUE,
  probability_scale = TRUE)
```

Plots of fitted dose-response models and an AIC-based average model including 80% and 95% credible bands on the probability scale:
```{r}
plot(model_fits, cr_bands = TRUE)
```

In case models should be shown on the logit scale this can be done in the following way:
```{r}
plot(model_fits, probability_scale = FALSE)
```

Estimates including predictions can be shown via:
```{r}
display_params_table(stats::predict(model_fits, doses = c(0, 2.5, 10,150, 200)))
```

The bootstrap-based quantiles can also be directly calculated via the 
`getBootstrapQuantiles()` function and a sample from the model fits can be bootstrapped using `getBootstrapSamples()`.

For this example, only 10 samples are bootstrapped for each model fit.
```{r}
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
bootstrap_quantiles <- getBootstrapQuantiles(
  model_fits = model_fits,
  quantiles  = c(0.025, 0.5, 0.975),
  doses      = dose_levels,
  n_samples  = 10)
```

# Assessment of the Minimally Efficacious Dose

The Minimally Efficacious Dose (MED) per model shape can be assessed with the function `getMED()`.
The effect needs to be specified on the probability scale.
```{r}
getMED(
  delta       = 0.16, # on probability scale
  model_fits  = model_fits,
  dose_levels = seq(min(dose_levels), max(dose_levels), by = 1))
```

## Additional Note

Testing, modeling, and MED assessment can also be combined via `performBayesianMCPMod()`:

```{r, eval = FALSE}
BMCPMod_result <- performBayesianMCPMod(
  posterior_list    = post_logit,
  contr             = contr_mat_prior,
  crit_prob_adj     = crit_pval,
  simple            = TRUE,
  delta             = 0.16,
  probability_scale = TRUE
)
```

---
title: "Simulation Example of Bayesian MCPMod for Continuous Data"
output: rmarkdown::html_vignette
number_sections: true
vignette: >
  %\VignetteIndexEntry{Simulation Example of Bayesian MCPMod for Continuous Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(BayesianMCPMod)
library(clinDR)
library(dplyr)

set.seed(7015)
```

# Background and data

In this vignette, we will show the use of the Bayesian MCPMod package for trial planning for continuous distributed data. 
As in [the analysis example vignette](analysis_normal.html), we focus on the indication MDD and make use of historical data that is included in the clinDR package. 
More specifically, trial results for BRINTELLIX will be utilized to establish an informative prior for the control group.

# Calculation of a MAP prior
In a first step, a meta analytic predictive prior will be calculated using historical data from 5 trials with main endpoint Change from baseline in MADRS score after 8 weeks.
Please note that only information from the control group will be integrated leading to an informative mixture prior for the control group, while for the active groups a non-informative prior will be specified.

```{r Historical Data}
data("metaData")
testdata    <- as.data.frame(metaData)
dataset     <- filter(testdata, bname == "BRINTELLIX")
histcontrol <- filter(dataset, dose == 0, primtime == 8, indication == "MAJOR DEPRESSIVE DISORDER")

hist_data <- data.frame(
  trial = histcontrol$nctno,
  est   = histcontrol$rslt,
  se    = histcontrol$se,
  sd    = histcontrol$sd,
  n     = histcontrol$sampsize)

sd_tot <- with(hist_data, sum(sd * n) / sum(n))
```
We will make use of the same getPriorList() function as in the [analysis example vignette](analysis_normal.html) to create a MAP prior.
```{r Setting Prior without execution, eval = FALSE}
dose_levels <- c(0, 2.5, 5, 10, 20)

prior_list  <- getPriorList(
  hist_data     = hist_data,
  dose_levels   = dose_levels,
  robust_weight = 0.3)
```
```{r Setting Prior, echo = FALSE}
dose_levels <- c(0, 2.5, 5, 10, 20)

prior_list  <- list(
  Ctr  = RBesT::mixnorm(
    comp1  = c(w = 0.446213, m = -12.774661, s = 1.393130),
    comp1  = c(w = 0.253787, m = 3.148116,   s = 3.148116),
    robust = c(w = 0.3,      m = 9.425139,   s = 9.425139),
    sigma = sd_tot),
  DG_1 = RBesT::mixnorm(
    comp1 = c(w = 1, m = -12.816875, n = 1),
    sigma = sd_tot,
    param = "mn"),
  DG_2 = RBesT::mixnorm(
    comp1 = c(w = 1, m = -12.816875, n = 1),
    sigma = sd_tot,
    param = "mn"),
  DG_3 = RBesT::mixnorm(
    comp1 = c(w = 1, m = -12.816875, n = 1),
    sigma = sd_tot,
    param = "mn"),
  DG_4 = RBesT::mixnorm(
    comp1 = c(w = 1, m = -12.816875, n = 1),
    sigma = sd_tot,
    param = "mn")
)
```

# Specification of new trial design 

For the hypothetical new trial, we plan with 4 active dose levels \eqn{2.5, 5, 10, 20} and we specify a broad set of potential dose-response relationships, including a linear, an exponential, an emax and 2 sigEMax models.  
Furthermore, we assume a maximum effect of -3 on top of control (i.e. assuming that active treatment can reduce the MADRS score after 8 weeks by up to 15.8) and plan a trial with 80 patients for all active groups and 60 patients for control.
```{r}
exp     <- DoseFinding::guesst(
  d     = 5,
  p     = c(0.2),
  model = "exponential",
  Maxd  = max(dose_levels))

emax    <- DoseFinding::guesst(
  d     = 2.5,
  p     = c(0.9),
  model = "emax")

sigemax <- DoseFinding::guesst(
  d     = c(2.5, 5),
  p     = c(0.1, 0.6),
  model = "sigEmax")

sigemax2 <- DoseFinding::guesst(
  d     = c(2, 4),
  p     = c(0.3, 0.8),
  model = "sigEmax")

mods <- DoseFinding::Mods(
  linear      = NULL,
  emax        = emax,
  exponential = exp,
  sigEmax     = rbind(sigemax, sigemax2),
  doses       = dose_levels,
  maxEff      = -3,
  placEff     = -12.8)

n_patients <- c(60, 80, 80, 80, 80)
```

# Calculation of success probabilities

To calculate success probabilities for the different assumed dose-response models and the specified trial design we will apply the assessDesign function. 
For illustration purposes, the number of simulated trial results is reduced to 100 in this example.
```{r}
success_probabilities <- assessDesign(
  n_patients  = n_patients,
  mods        = mods,
  prior_list  = prior_list,
  sd          = sd_tot,
  n_sim       = 100) # speed up example run-time

success_probabilities
```
As an alternative, we will evaluate a design with the same overall sample size but allocating more patients on the highest dose group and control.
```{r}
success_probabilities_uneq <- assessDesign(
  n_patients  = c(80, 60, 60, 60, 120),
  mods        = mods,
  prior_list  = prior_list,
  sd          = sd_tot,
  n_sim       = 100) # speed up example run-time
success_probabilities_uneq
```
For this specific trial setting the adapted allocation ratio leads to increased success probabilities under all assumed dose response relationships.

Instead of specifying the assumed effects via the models it is also possible to directly specify the effects for the individual dose levels via the dr_means input.
This allows e.g. also the simulation of scenarios with a prior-data conflict.
```{r}
success_probabilities <- assessDesign(
  n_patients  = c(60, 80, 80, 80, 80),
  mods        = mods,
  prior_list  = prior_list,
  sd          = sd_tot,
  dr_means    = c(-12, -14, -15, -16, -17),
  n_sim       = 100) # speed up example run-time
success_probabilities
```

---
title: "Comparison of Bayesian MCPMod and MCPMod"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Comparison of Bayesian MCPMod and MCPMod}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, collapse=TRUE}
#| code-summary: Setup
#| code-fold: true
#| message: false
#| warning: false

library(BayesianMCPMod)
library(DoseFinding)
library(MCPModPack)

library(tidyr)
library(dplyr)

library(kableExtra)
library(ggplot2)

library(doFuture)
```

```{r, echo = FALSE}

load_path <- file.path(getwd(), "data", "simulation_comparison")

```

# Introduction

This vignette demonstrates the application of the [BayesianMCPMod](https://CRAN.R-project.org/package=BayesianMCPMod) package for sample size calculations and the comparison with the [MCPModPack](https://CRAN.R-project.org/package=MCPModPack) package.
Bayesian MCPMod is set up in a way that it  mimics the results (and operating characteristics) of the frequentist MCPMod for vague priors.
This characteristic is illustrated in the following sections focusing on the trial planning. 

The following dose-finding scenario is considered to compare Bayesian MCPMod and MCPModPack success probabilities:

-   Four dose levels plus placebo (0 mg, 1 mg, 2 mg, 4 mg, 8 mg)

-   Total sample size of N = 200 with an equal allocation ratio for each dose group, i.e., 40 per group

-   Standard deviation of 0.4 for every dose group

-   Alpha level of 5%

```{r, collapse=TRUE}
#| code-summary: Scenario Parameters
#| code-fold: true
doses_sim     <- c(0, 1, 2, 4, 8) 
n_sample      <- c(40, 40, 40, 40, 40)
sd_sim        <- 0.4

max_dose      <- max(doses_sim)    
plc_eff_guess <- 0    

alpha         <- 0.05
```

Simulations are performed with 10000 runs. 
Given the number of simulations and applying the law of large numbers, the difference in success probabilities should be in the range of 1% - 3%.

```{r, collapse=TRUE}
#| code-summary: Simulation Parameters
#| code-fold: true
set.seed(7015)
n_sim <- 10000

plan(multisession)
registerDoFuture()
```

```{r, echo = FALSE}

n_sim <- 10

```

In the following figure shows the considered candidate models.

```{r fig.height=5, collapse=TRUE}
#| code-summary: Candidate Models
#| code-fold: true
emax_guess     <- guesst(d = doses_sim[2], p = 0.6, "emax") 
exp_guess      <- guesst(d = doses_sim[2], p = 0.05, model = "exponential", Maxd = max_dose)
logit_guess    <- guesst(d = c(doses_sim[2], doses_sim[3]), p = c(0.1, 0.9), "logistic", Maxd = max_dose) 
sig_emax_guess <- guesst(d = c(doses_sim[2], doses_sim[3]), p = c(0.15, 0.75), model = "sigEmax") 

plot(Mods(linear      = NULL,
          exponential = exp_guess,
          emax        = emax_guess,
          logistic    = logit_guess,
          sigEmax     = sig_emax_guess,
          doses       = doses_sim,
          placEff     = plc_eff_guess,
          direction   = "increasing"),
     main = "Candidate Models")
```

# Varying the Expected Effect for Maximum Dose

The following expected effects are studied:

-   Expected effect for maximum dose of 0.0001, 0.05, 0.1, 0.2, 0.3, and 0.5

```{r, collapse=TRUE}
#| code-summary: Expected Effects
#| code-fold: true
exp_eff <- c(0.0001, 0.05, 0.1, 0.2, 0.3, 0.5)
```

The value of 0.0001 was chosen instead of 0 due to technical reasons.
This case should mimic the null scenario (where we expect a success probability close to the alpha level).

```{r, collapse=TRUE}
#| code-summary: MCPModPack Implementation
#| code-fold: true
#| warning: false
# Simulation parameters
sim_parameters <- list(n            = n_sample,
                       doses        = doses_sim,
                       dropout_rate = 0.0,
                       go_threshold = 0.1,
                       nsims        = n_sim)

# Candidate dose - response models
models_MCPModPack = list(linear      = NA,
                         exponential = exp_guess,
                         emax        = emax_guess,
                         logistic    = logit_guess,
                         sigemax     = sig_emax_guess)

# Assumed dose - response models (models will be added in loop)
sim_models_part <- list(max_effect     = exp_eff,
                        sd             = rep(sd_sim, length(doses_sim)),
                        placebo_effect = plc_eff_guess)

# Parallelization across assumed dose - response models
powers_MCPModPack_eff <- foreach(
  k = seq_along(models_MCPModPack),
  .combine = cbind, 
  .options.future = list(seed = TRUE)) %dofuture% {
    
    sim_model_k <- c(models_MCPModPack[k], sim_models_part)
    
    MCPModSimulation(endpoint_type   = "Normal",
                     models          = models_MCPModPack,
                     alpha           = alpha,
                     direction       = "increasing",
                     model_selection = "aveAIC",
                     Delta           = 0.1,
                     sim_models      = sim_model_k,
                     sim_parameters  = sim_parameters)$sim_results$power
  
  }

# Post-processing result for printing
colnames(powers_MCPModPack_eff) <- names(models_MCPModPack)
results_MCPModPack_eff <- cbind(max_eff = round(exp_eff, digits = 2),
                                powers_MCPModPack_eff) %>%
  data.frame() %>%
  rename(sigEmax = sigemax) %>%
  mutate(average = rowMeans(select(., linear:sigEmax)))
```

```{r, echo = FALSE}

results_MCPModPack_eff <- readRDS(file.path(load_path, "results_MCPModPack_eff.rds"))

```


```{r, collapse=TRUE}
#| code-summary: Bayesian MCPMod Implementation
#| code-fold: true
# Vague prior specification
prior_list_vague <- rep(list(RBesT::mixnorm(comp1 = c(w = 1, m = 0, n = 1),
                                            sigma = sd_sim, param = "mn")),
                        times = length(doses_sim))
names(prior_list_vague) <- c("Ctrl", "DG_1", "DG_2", "DG_3", "DG_4")

# Parallelization across the expected effects for maximum dose
success_rates_BayesianMCPMod_eff <- foreach(
  k               = seq_along(exp_eff),
  .combine        = rbind, 
  .options.future = list(seed = TRUE)) %dofuture% {
    
    exp_eff_k <- exp_eff[k]
    
    models_BayesianMCPMod <- Mods(linear      = NULL,
                                  exponential = exp_guess,
                                  emax        = emax_guess,
                                  logistic    = logit_guess,
                                  sigEmax     = sig_emax_guess,
                                  doses       = doses_sim,
                                  placEff     = plc_eff_guess,
                                  maxEff      = exp_eff_k,
                                  direction   = "increasing")
       
    # Optimal contrasts
    contr <- getContr(mods         = models_BayesianMCPMod,
                      dose_levels  = doses_sim,
                      prior_list   = prior_list_vague,
                      dose_weights = rep(1, length(doses_sim)))
  
    # Perform Simulations
    sim_result <- assessDesign(n_patients     = n_sample,
                               mods           = models_BayesianMCPMod,
                               prior_list     = prior_list_vague,
                               sd             = sd_sim,
                               n_sim          = n_sim,
                               alpha_crit_val = alpha,
                               contr          = contr)
    
    c(sapply(sim_result, attr, "successRate"),
      average = attr(sim_result, "avgSuccessRate"))
  
  }

# Post-processing result for printing
rownames(success_rates_BayesianMCPMod_eff) <- NULL
results_BayesianMCPMod_eff <- data.frame(cbind(max_eff = round(exp_eff, digits = 2),
                                           success_rates_BayesianMCPMod_eff))
```

```{r, echo = FALSE}

results_BayesianMCPMod_eff <- readRDS(file.path(load_path, "results_BayesianMCPMod_eff.rds"))

```

The following plot shows the differences between the results obtained with MCPModPack and BayesianMCPMod.
The results of BayesianMCPMod are shown as horizontal lines and the differences to the results of MCPModPack are presented as vertical lines.
As the results are close to one another, the vertical lines are barely visible.
The colours indicate the different assumed true dose-response models, which were the basis for simulating the data.

```{r fig.height=5}
#| code-summary: Comparing Power and Success Rates
## pre-processing the data
df_plot_eff <- rbind(results_MCPModPack_eff %>% 
                     mutate(package_name = "MCPModPack"),
                     results_BayesianMCPMod_eff %>% 
                     mutate(package_name = "BayesianMCPMod")) %>%
  pivot_longer(cols      = names(results_BayesianMCPMod_eff)[-1],
               names_to  = "model_shape",
               values_to = "success_rate") %>%
  filter(model_shape != "average") %>%
  spread(key = package_name, value = success_rate)  %>%
  mutate(model_shape = as.factor(model_shape),
         max_eff     = as.factor(max_eff)) %>%
  group_by(max_eff) %>%
  mutate(offset = 0.1 * (seq_along(model_shape) - ceiling(length(model_shape) / 2)))

# Plot with short horizontal dashes for each model_shape
ggplot(df_plot_eff, aes(x = as.numeric(max_eff) + offset, y = BayesianMCPMod, color = model_shape)) +
  geom_segment(aes(x = as.numeric(max_eff) + offset - 0.05, xend = as.numeric(max_eff) + offset + 0.05,
                   y = BayesianMCPMod, yend = BayesianMCPMod)) +
  geom_segment(aes(xend = as.numeric(max_eff) + offset, yend = MCPModPack)) +
  scale_x_continuous(breaks = unique(as.numeric(df_plot_eff$max_eff)), labels = levels(df_plot_eff$max_eff)) +
  labs(title    = "Comparing Power and Success Rate",
       subtitle = "For Different Expected Effects for Maximum Dose",
       x        = "Expected Effect for Maximum Dose",
       y        = "Success Rate / Power",
       color    = "Assumed True Model Shapes",
       linetype = "Type",
       caption  = "Horizontal lines represent the BayesianMCPMod success rates and vertical lines mark the distance to the MCPModPack power.") +
  theme_minimal() + 
  theme(legend.position = "bottom")
```

As expected, the operating characteristics of BayesianMCPMod with vague priors match the operating characteristics of frequentist MCPMod.

Numerical results are shown in the two tables below.

```{r}
#| code-summary:  MCPModPack Power
kable(results_MCPModPack_eff) %>%
  kable_classic() %>%
    add_header_above(c("Power Values Across Different Expected Effects" = 7),
                     font_size = 15, bold = TRUE) %>%
    add_header_above(c("MCPModPack" = 7), font_size = 15, bold = TRUE)
```

```{r}
#| code-summary: BayesianMCPMod Success Rates
kable(results_BayesianMCPMod_eff) %>%
  kable_classic(full_width = TRUE) %>%
    add_header_above(c("Success Rates Across Different Expected Effects" = 7),
                     font_size = 15, bold = TRUE) %>%
    add_header_above(c("BayesianMCPMod" = 7), font_size = 15, bold = TRUE) 
```



# Convergence of Power Values

In the following simulations, we examine the convergence of power and success rate values for an increasing number of simulations.
For this, the expected maximum effect is fixed at 0.2. 

```{r}
#| code-summary: Fixed expected maximum effect
exp_eff_fix <- 0.2
```

The R package MCPModPack provides a final power result for a given number of simulations and does not allow accessing intermediate power values.

```{r}
#| code-summary: Numbers of simulations for power values
n_sim_vec <- seq(2, 10, 1)^4
```

```{r, echo = FALSE}
n_sim_vec <- c(5, 10)
```

This is why the simulation is repeated for different values of number of simulations for the implementation with MCPModPack.

```{r}
#| code-summary: MCPModPack Implementation
#| warning: false
# Updating the assumed dose - response models for the fixed expected effect (models will be added in loop)
sim_models_part$max_effect <- exp_eff_fix

# Parallelization across assumed dose - response models & number of simulations
## Reversing the numbers speeds up parallelization in case there are not enough workers
sim_grid <- expand.grid(names(models_MCPModPack), rev(n_sim_vec))
powers_MCPModPack_conv <- foreach(
  k = seq_len(nrow(sim_grid)),
  .combine = c,
  .options.future = list(seed = TRUE)) %dofuture% {
    
    sim_model_k <- c(models_MCPModPack[sim_grid[k, 1]], sim_models_part)
    
    sim_params_k <- sim_parameters
    sim_params_k$nsims <- sim_grid[k, 2]
    
    MCPModSimulation(endpoint_type   = "Normal",
                     models          = models_MCPModPack,
                     alpha           = alpha,
                     direction       = "increasing",
                     model_selection = "aveAIC",
                     Delta           = 0.1,
                     sim_models      = sim_model_k,
                     sim_parameters  = sim_params_k)$sim_results$power
  
  }

#Post-processing
results_MCPModPack_conv <- cbind(sim_grid, powers_MCPModPack_conv) %>%
  arrange(desc(row_number())) %>%
  rename(model_name   = Var1,
         n_sim        = Var2,
         success_rate = powers_MCPModPack_conv) %>%
  mutate(model_name = if_else(model_name == "sigemax",
                             "sigEmax", model_name)) %>%
  mutate(package_name = "MCPModPack")
```

```{r, echo = FALSE}

results_MCPModPack_conv <- readRDS(file.path(load_path, "results_MCPModPack_conv.rds"))

```

In the BayesianMCPMod implementation, only one simulation run is required to access the estimated success rate values at different numbers of simulations.

```{r}
#| code-summary: BayesianMCPMod Implementation
# Model specifications with fixed expected effect
models_BayesianMCPMod <- Mods(linear      = NULL,
                              exponential = exp_guess,
                              emax        = emax_guess,
                              logistic    = logit_guess,
                              sigEmax     = sig_emax_guess,
                              doses       = doses_sim,
                              placEff     = plc_eff_guess,
                              maxEff      = exp_eff_fix,
                              direction   = "increasing")
       
# Optimal contrasts
contr <- getContr(mods         = models_BayesianMCPMod,
                  dose_levels  = doses_sim,
                  prior_list   = prior_list_vague,
                  dose_weights = rep(1, length(doses_sim)))

# Perform Simulations
sim_result <- assessDesign(n_patients     = n_sample,
                           mods           = models_BayesianMCPMod,
                           prior_list     = prior_list_vague,
                           sd             = sd_sim,
                           n_sim          = max(n_sim_vec),
                           alpha_crit_val = alpha,
                           contr          = contr)

# Getting success rates at different numbers of simulations
results_BayesianMCPMod_conv <- sapply(sim_result, function (model_result) {
  
  sapply(n_sim_vec, function (n_sim_x) {
    
    success_rate_x <- mean(model_result[seq_len(n_sim_x), 1])
    
  })
  
}) %>%
  # Post-processing
  as.data.frame %>%
  mutate(n_sim = n_sim_vec) %>%
  pivot_longer(cols = -n_sim,
               names_to = "model_name",
               values_to = "success_rate") %>%
  mutate(package_name = "BayesianMCPMod")
```

```{r, echo = FALSE}

results_BayesianMCPMod_conv <- readRDS(file.path(load_path, "results_BayesianMCPMod_conv.rds"))

```

The figure below shows the convergence of the power and success rate values for an increasing number of simulations.

```{r fig.height=5}
#| code-summary: BayesianMCPMod Implementation
df_plot_conv <- inner_join(results_BayesianMCPMod_conv,
                           results_MCPModPack_conv,
                           by = c("model_name", "n_sim")) %>%
  mutate(success_rate_diff = success_rate.x - success_rate.y) %>%
  select(model_name, n_sim, success_rate_diff)

ggplot(df_plot_conv, aes(x = n_sim, y = success_rate_diff, color = model_name)) +
  geom_point() +
  geom_line() +
  geom_hline(yintercept = 0, linetype = "dashed", color = "grey")+
  scale_x_continuous(breaks = unique(df_plot_conv$n_sim)[-c(2, 3)]) +
  scale_y_continuous(breaks = c(-0.05, 0, 0.05, 0.1, 0.15)) +
  labs(
    title   = "Success Rate Difference vs Number of Simulations",
    x       = "Number of Simulations",
    y       = "Success Rate Difference",
    color   = "True Model Shape",
    caption = "The success rate difference is the difference of the success rates calculated 
    with BayesianMCPMod and the power values calculated with MCPModPack.") +
  theme_minimal() + 
  theme(panel.grid.minor.x = element_blank())
```

As expected, the differences are within the range of 1% - 3%.

---
title: "Trial Analysis Example of Bayesian MCPMod for Continuous Data"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Trial Analysis Example of Bayesian MCPMod for Continuous Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: references.bib
---

```{r setup}
#| code-summary: setup
#| code-fold: true
#| message: false
#| warning: false

library(BayesianMCPMod)
library(RBesT)
library(clinDR)
library(dplyr)
library(tibble)
library(reactable)

set.seed(7015)
```

<details>
<summary><strong>Show code</strong></summary>
```{r table_function, collapse=TRUE}
display_params_table <- function(named_list) {
  round_numeric <- function(x, digits = 3) if (is.numeric(x)) round(x, digits) else x
  tbl <- data.frame(
    Name  = names(named_list),
    Value = I(lapply(named_list, function(v) {
      if (inherits(v, "Date")) v <- as.character(v)
      if (!is.null(names(v))) paste0("{", paste(names(v), v, sep="=", collapse=", "), "}")
      else v
    }))
  )
  tbl$Value <- lapply(tbl$Value, round_numeric)
  knitr::kable(tbl)
}
```
</details>

# Introduction 

This vignette demonstrates the application of the `BayesianMCPMod` package for 
analyzing a phase 2 dose-finding trial using the Bayesian MCPMod approach.

A more general overview of the R package was provided with a [poster](https://github.com/Boehringer-Ingelheim/BayesianMCPMod/blob/36763ee5325955ca5d76a6c140ea3881c74e5fda/inst/PSI_Poster_2024.pdf) presented during the PSI 2024 Conference.

This package makes use of the [future](https://cran.r-project.org/package=future) framework for parallel processing, which can be set up for example as follows:
```{r, eval = FALSE}
future::plan(future::multisession, workers = 4L)
```
Kindly note that due to overhead a reduced number of worker nodes can be preferable and that for short calculations sequential execution can be faster.

# Calculation of a MAP Prior

In a first step, a meta analytic prior will be calculated using historical data from 4 trials with main endpoint Change from baseline in MADRS score after 8 weeks.
Please note that only information from the control group will be integrated leading to an informative mixture prior for the control group, while for the active groups a non-informative prior will be specified.
```{r Historical Data for Control Arm}
data("metaData")
dataset     <- filter(as.data.frame(metaData), bname == "BRINTELLIX")
histcontrol <- filter(
  dataset,
  dose       == 0,
  primtime   == 8,
  indication == "MAJOR DEPRESSIVE DISORDER",
  protid     != 5)

hist_data   <- data.frame(
  trial = histcontrol$nctno,
  est   = histcontrol$rslt,
  se    = histcontrol$se,
  sd    = histcontrol$sd,
  n     = histcontrol$sampsize)
```
Here, we suggest a function to construct a list of prior distributions for the different dose groups.
This function is adapted to the needs of this example.
Other applications may need a different way to construct prior distributions.
```{r Defining MAP prior function}
getPriorList <- function (
  
  hist_data,
  dose_levels,
  dose_names    = NULL,
  robust_weight = 0.5
  
) {
  
  sd_tot <- with(hist_data, sum(sd * n) / sum(n))
  
  gmap <- RBesT::gMAP(
    formula    = cbind(est, se) ~ 1 | trial,
    weights    = hist_data$n,
    data       = hist_data,
    family     = gaussian,
    beta.prior = cbind(0, 100 * sd_tot),
    tau.dist   = "HalfNormal",
    tau.prior  = cbind(0, sd_tot / 4))
  
  prior_ctr <- RBesT::automixfit(gmap)
  
  if (!is.null(robust_weight)) {
    
    prior_ctr <- suppressMessages(RBesT::robustify(
      priormix = prior_ctr,
      weight   = robust_weight,
      sigma    = sd_tot))
    
  }
  
  prior_trt <- RBesT::mixnorm(
    comp1 = c(w = 1, m = summary(prior_ctr)[1], n = 1),
    sigma = sd_tot,
    param = "mn")
  
  prior_list <- c(list(prior_ctr),
                  rep(x     = list(prior_trt),
                      times = length(dose_levels[-1])))
  
  if (is.null(dose_names)) {
    
    dose_names <- c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
    
  }
  
  names(prior_list) <- dose_names
  
  return (prior_list)
  
}
```
With the dose levels to be investigated, the prior distribution can be constructed.
```{r Getting the MAP prior}
dose_levels <- c(0, 2.5, 5, 10)

set.seed(7015) # re-sets seed only for this example; remove in your analysis script
prior_list  <- getPriorList(
  hist_data     = hist_data,
  dose_levels   = dose_levels,
  robust_weight = 0.3)

getESS(prior_list)
```

# Dose-Response Model Shapes

Candidate models are specified using the {DoseFinding} package. Models can be 
parameterized using guesstimates or by directly providing distribution parameters. 
Note that the linear candidate model does not require parameterization.

[**Note:** The LinLog model is rarely used and not currently supported by `BayesianMCPMod`.]{.aside}

In the code below, the models are "guesstimated" using the `DoseFinding::guesst` function. 
The `d` option usually takes a single value (a dose level), and the corresponding `p` 
for the maximum effect achieved at `d`. 


```{r}
# Guesstimate estimation
exp_guesst  <- DoseFinding::guesst(
  model = "exponential", 
  d = 5, p = 0.2, Maxd = max(dose_levels)
)
emax_guesst <- DoseFinding::guesst(
  model = "emax",
  d = 2.5, p = 0.9
)
sigEmax_guesst <- DoseFinding::guesst(
  model = "sigEmax",
  d = c(2.5, 5), p = c(0.5, 0.95)
)
logistic_guesst <- DoseFinding::guesst(
  model = "logistic",
  d = c(5, 10), p = c(0.1, 0.85)
)
```

In some cases, you need to provide more information. For instance, `sigEmax` 
requires a pair of `d` and `p` values, and `exponential` requires the specification of 
the maximum dose for the trial (`Maxd`). 

[See the help files for model specifications by typing `?DoseFinding::guesst` in your console]{.aside}



Of course, you can also specify the models directly on the parameter scale (without using `DoseFinding::guesst`).

For example, you can get a betaMod model by specifying `delta1` and `delta2` 
parameters (`scale` is assumed to be `1.2` of the maximum dose), or a quadratic model with the `delta2` parameter.

```{r}
betaMod_params   <- c(delta1 = 1, delta2 = 1)
quadratic_params <- c(delta2 = -0.1)
```

Now, we can go ahead and create a `Mods` object, which will be used in the remainder
of the vignette.

```{r}
mods <- DoseFinding::Mods(
  linear      = NULL,
  # guesstimate scale
  exponential = exp_guesst,
  emax        = emax_guesst,
  sigEmax     = sigEmax_guesst,
  logistic    = logistic_guesst,
  # parameter scale
  betaMod     = betaMod_params,
  quadratic   = quadratic_params,
  # Options for all models
  doses       = dose_levels,
  maxEff      = -1,
  placEff     = -12.8
)

plot(mods)
```

The `mods` object we just created above contains the full model parameters, which can be helpful for 
understanding how the guesstimates are translated onto the parameter scale. 

```{r}
display_params_table(mods)
```

And we can see the assumed treatment effects for the specified dose groups below:

```{r}
knitr::kable(DoseFinding::getResp(mods, doses = dose_levels))
```

## Trial Data

We will use the trial with ct.gov number NCT00735709 as our phase 2 trial data,
available in the `clinDR` package [@nct00735709_2024a].

```{r}
data("metaData")

trial_data <- dplyr::filter(
  dplyr::filter(tibble::tibble(metaData), bname == "BRINTELLIX"),
  primtime   == 8,
  indication == "MAJOR DEPRESSIVE DISORDER",
  protid     == 5
)

n_patients <- c(128, 124, 129, 122)
```

# Posterior Calculation

In the first step of Bayesian MCPMod, the posterior is calculated by combining 
the prior information with the estimated results of the trial [@fleischer_2022]. 

```{r}
posterior <- getPosterior(
  prior_list = prior_list,
  mu_hat     = trial_data$rslt,
  S_hat      = diag(trial_data$se^2),
  calc_ess   = TRUE
)

knitr::kable(summary(posterior))
```

# Bayesian MCPMod Test Step

The testing step of Bayesian MCPMod is executed using a critical value on the probability scale and a pseudo-optimal contrast matrix. 

The critical value is calculated using (re-estimated) contrasts for frequentist MCPMod to ensure error control when using weakly-informative priors.

A pseudo-optimal contrast matrix is generated based on the variability of the posterior distribution (see [@fleischer_2022] for more details).

```{r}
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
crit_pval <- getCritProb(
  mods           = mods,
  dose_levels    = dose_levels,
  cov_new_trial  = diag(trial_data$se^2),
  alpha_crit_val = 0.05
)

contr_mat <- getContr(
  mods          = mods,
  dose_levels   = dose_levels,
  cov_posterior = diag(summary(posterior)[, 2]^2)
)
```

Please note that there are different ways to derive the contrasts.
The following code shows the implementation of some of these ways but it is not executed and the contrast specification above is used.

```{r}
#| eval: false

# i) the frequentist contrast
contr_mat_prior <- getContr(
  mods           = mods,
  dose_levels    = dose_levels,
  dose_weights   = n_patients,
  prior_list     = prior_list)
# ii) re-estimated frequentist contrasts
contr_mat_prior <- getContr(
  mods           = mods,
  dose_levels    = dose_levels,
  cov_new_trial  = diag(trial_data$se^2))
# iii)  Bayesian approach using number of patients for new trial and prior distribution
contr_mat_prior <- getContr(
  mods           = mods,
  dose_levels    = dose_levels,
  dose_weights   = n_patients,
  prior_list     = prior_list)
```

The Bayesian MCP testing step is then executed:  

```{r}
BMCP_result <- performBayesianMCP(
  posterior_list = posterior,
  contr          = contr_mat, 
  crit_prob_adj  = crit_pval)
```

Summary information:

```{r}
BMCP_result
```

The testing step is significant, indicating a non-flat dose-response shape.
All models are significant, with the `emax` model indicating the greatest deviation from the null hypothesis.

# Model Fitting and Visualization

In the model fitting step the posterior distribution is used as basis.

Both simplified and full fitting are performed. 

For the simplified fit, the multivariate normal distribution of the control group is approximated and reduced by a one-dimensional normal distribution.

The actual fit (on this approximated posterior distribution) is then performed using generalized least squares criterion. In contrast, for the full fit, the non-linear optimization problem is addressed via the Nelder-Mead algorithm [@neldermead_2024a] implemented by the `nloptr` package.

The output of the fit includes information about the predicted effects for the included dose levels, the generalized AIC, and the corresponding weights.

For the considered case, the simplified and the full fit are very similar, so we present the full fit.

```{r}
# If simple = TRUE, uses approx posterior
# Here we use complete posterior distribution
model_fits <- getModelFits(
  models      = mods,
  dose_levels = dose_levels,
  posterior   = posterior,
  simple      = FALSE)
```

Estimates for dose levels not included in the trial:

```{r}
display_params_table(stats::predict(model_fits, doses = c(0, 2.5, 4, 5, 7, 10)))
```

Plots of fitted dose-response models and an AIC-based average model:

```{r}
plot(model_fits)
```

To assess the uncertainty, one can additionally visualize credible bands (orange shaded areas, default levels are 50% and 95%).

These credible bands are calculated with a bootstrap method as follows:

- Samples from the posterior distribution are drawn and for every sample the 
simplified fitting step and a prediction is performed.

- These predictions are then used to identify and visualize the specified quantiles.

```{r}
plot(model_fits, cr_bands = TRUE)
```

The bootstrap-based quantiles can also be directly calculated via the 
`getBootstrapQuantiles()` function and a sample from the model fits can bootstrapped using `getBootstrapSamples()`.

For this example, only 10 samples are bootstrapped for each model fit.

```{r}
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
bootstrap_quantiles <- getBootstrapQuantiles(
  model_fits = model_fits,
  quantiles  = c(0.025, 0.5, 0.975),
  doses      = c(0, 2.5, 4, 5, 7, 10),
  n_samples  = 10)
```
The bootstrap quantiles include information about the absolute quantiles (sample_type=abs) and also about the placebo-adjusted (resp. control-adjusted) quantiles (sample_type=diff). 

```{r, collapse = TRUE}
#| code-fold: true
reactable::reactable(
  data = bootstrap_quantiles |>
    tidyr::pivot_wider(names_from = q_prob, values_from = q_val),
  groupBy = "model",
  columns = list(
    dose    = colDef(format = list(aggregated = colFormat(suffix = " dose"))),
    "0.025" = colDef(format = list(cell = colFormat(digits = 4))),
    "0.5"   = colDef(format = list(cell = colFormat(digits = 4))),
    "0.975" = colDef(format = list(cell = colFormat(digits = 4)))
  )
)
```

**Technical note:** The median quantile of the bootstrap based procedure is not 
necessary similar to the main model fit, as they are derived via different procedures.

The main fit (black line) minimizes residuals for the posterior distribution, 
while the bootstrap median is the median fit of random sampling.

# Assessment of the Minimally Efficacious Dose

The Minimally Efficacious Dose (MED) per model shape can be assessed with the function `getMED()`.
```{r}
getMED(
  delta       = 4,
  model_fits  = model_fits,
  dose_levels = seq(min(dose_levels), max(dose_levels), by = 0.01))
```
For an optional Bayesian decision rule for the MED assessment and further details, please see `?getMED()`.

## Additional Note

Testing, modeling, and MED assessment can also be combined via `performBayesianMCPMod()`:

```{r}
#| eval: false
performBayesianMCPMod(
  posterior_list   = posterior,
  contr            = contr_mat,
  crit_prob_adj    = crit_pval,
  delta            = 4,
  simple           = FALSE)
```

---
title: "Trial Simulation Example of Bayesian MCPMod for Binary Data"
format: 
  html:
    fig-height: 3.5
    self-contained: true
    toc: true
    number-sections: true
    code-summary: setup
    #code-fold: true
    message: false
    warning: false
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{Trial Simulation Example of Bayesian MCPMod for Binary Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup}
#| message: false
#| warning: false

suppressPackageStartupMessages({
  library(BayesianMCPMod)
  library(RBesT)
  library(DoseFinding)
  library(clinDR)
  library(dplyr)
  library(tibble)
  library(reactable)
})

set.seed(7015)
```

# Background and Data

In this vignette, we will show the use of the `BayesianMCPMod` R package for trial planning for binary data. 

For this setting the main specifications need to happen on the logit scale, see also the [binary analysis example vignette](binary_endpoint.html).

<div class="alert alert-info">

- Internally, `BayesianMCPMod` fits binary endpoints on the **logit scale**.
- The function argument `probability_scale` controls whether outputs (summaries, predictions, plots)
  are back-transformed to **probabilities**.
- In `simulateData()`, outcomes are simulated on the response scale if `probability_scale = TRUE`.
- In `getMED()`, `delta` is interpreted on the **probability scale** if `probability_scale = TRUE`.

</div>

As in the [analysis example vignette](analysis_normal.html), we make use of historical data that is included in the clinDR package. 
More specifically, trial results for XELJANZ will be utilized to establish an informative prior for the control group.

This package makes use of the [future](https://cran.r-project.org/package=future) framework for parallel processing, which can be set up for example as follows:
```{r, eval = FALSE}
future::plan(future::multisession, workers = 4L)
```
Kindly note that due to overhead a reduced number of worker nodes can be preferable and that for short calculations sequential execution can be faster.

# Calculation of a MAP Prior
For this example, in a first step, a meta analytic prior will be calculated.
The approach to establish the prior is the same as outlined in the [binary analysis example vignette](binary_endpoint.html). 
Please note that only information from the control group will be integrated leading to an informative mixture prior for the control group, while for the active groups a non-informative prior will be specified.

```{r}
data("metaData")
testdata    <- as.data.frame(metaData)
dataset     <- filter(testdata, bname == "XELJANZ")
histcontrol <- filter(dataset, dose == 0, primtime == 12, indication == "RHEUMATOID ARTHRITIS")

hist_data <- data.frame(
  trial = histcontrol$nctno,
  est   = histcontrol$rslt,
  se    = histcontrol$se,
  sd    = histcontrol$sd,
  r     = round(histcontrol$sampsize*histcontrol$rslt),
  n     = histcontrol$sampsize)

sd_tot <- with(hist_data, sum(sd * n) / sum(n))
```

```{r}
dose_levels <- c(0, 2.5, 5, 10,20)

# 1) Establish MAP prior (beta mixture distribution) 
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
map <- gMAP(
  cbind(hist_data$r, hist_data$n - hist_data$r) ~ 1 | histcontrol$nctno,
  family     = binomial,
  tau.dist   = "HalfNormal",
  tau.prior  = 0.5,
  beta.prior = (1 / sqrt(0.1 * 0.9)),
  warmup     = 1000,
  iter       = 10000,
  chains     = 2,
  thin       = 1
)

prior <- automixfit(map) #fits mixture distribution from MCMC samples from above

p <- summary(prior)[1]

# 2) Robustify prior
prior_rob <- RBesT::robustify(
      priormix = prior,
      mean     = 0.5,
      weight   = 0.2)

# 3) Translate prior to logit scale (to approximate via normal mixture model)
r         <- rmix(prior_rob, n = 1e4)
log_r     <- logit(r)
prior_ctr <- automixfit(log_r, type = "norm")

# Specification of reference scale (this follows the idea of [@Neuenschwander2016]). 
sigma(prior_ctr) <- sqrt(1 / (p * (1 - p)))

# Specify a prior list
prior_trt <- RBesT::mixnorm(
    comp1 = c(w = 1, m = logit(summary(prior)[1]), n = 1),
    sigma = sqrt(1/(p*(1-p))),
    param = "mn")
  
  prior_list <- c(list(prior_ctr),
                  rep(x     = list(prior_trt),
                      times = length(dose_levels[-1])))

dose_names        <- c("Ctr", paste0("DG_", seq_along(dose_levels[-1])))
names(prior_list) <- dose_names
```

Kindly note that a vague prior could be implemented via
```{r Setting Vague Prior Without Execution, eval = TRUE}
prior_list_vague <- rep(list(RBesT::mixnorm(
  comp1 = c(w = 1, m = logit(p), n = 1),
  sigma = sqrt(1 / (p * (1 - p))),
  param = "mn"
)), times = length(dose_levels))
names(prior_list_vague) <- c("Ctrl", "DG_1", "DG_2", "DG_3", "DG_4")
```

# Specification of the New Trial Design 

For the hypothetical new trial, we plan with 4 active dose levels \eqn{2.5, 5, 10, 20} and we specify a broad set of potential dose-response relationships, including a linear, an exponential, an emax, a logistic and a sigEMAX models.  
Furthermore, we assume a maximum response rate of 48% (resp. 30% on top of control) and plan a trial with 40 patients for all active groups and 30 patients for control.
Please note that the response rates need to be provided on the **logit scale**.

```{r}
n_patients <- c(30, 40, 40, 40, 40)

models <- Mods(
  linear      = NULL,
  sigEmax     = c(10, 5),
  logistic    = c(11, 15),
  exponential = 10,
  emax        = 2,
  doses       = dose_levels,
  placEff     = RBesT::logit(0.18),
  maxEff      = (RBesT::logit(0.48) - RBesT::logit(0.18))
)
```

# Calculation of the Success Probabilities

To calculate success probabilities for the different assumed dose-response models and the specified trial design we will apply the `assessDesign()` function. 
We are not only interested in the success probability for the testing step, but also the assessment of the Minimally Efficacious Dose (MED).
This effect delta is provided on the probability scale.
In our case we would like to see a difference of 20% compared to control to claim efficacy. 

For illustration purposes, the number of simulated trial results is reduced to `100` in this example.
```{r}
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities <- assessDesign(
  n_patients        = n_patients,
  mods              = models,
  prior_list        = prior_list,
  probability_scale = TRUE,
  delta             = 0.2,
  n_sim             = 100) # speed up example run-time
success_probabilities
```

As an alternative, we will evaluate a design with the same overall sample size but allocating more patients on the highest dose group and control.
```{r}
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities_uneq <- assessDesign(
  n_patients        = c(40, 30, 30, 30, 50),
  mods              = models,
  prior_list        = prior_list,
  probability_scale = TRUE,
  delta             = 0.2,
  n_sim             = 100) # speed up example run-time
success_probabilities_uneq
```

As a second alternative we will use the vague prior:
```{r}
set.seed(7015) # re-sets seed only for this example; remove in your analysis script
success_probabilities_vague <- assessDesign(
  n_patients        = n_patients,
  mods              = models,
  prior_list        = prior_list_vague,
  probability_scale = TRUE,
  delta             = 0.2,
  n_sim             = 100) # speed up example run-time
success_probabilities_vague
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BMCPMod.R
\name{performBayesianMCP}
\alias{performBayesianMCP}
\title{performBayesianMCP}
\usage{
performBayesianMCP(posterior_list, contr, crit_prob_adj)
}
\arguments{
\item{posterior_list}{An object derived with getPosterior with information about the (mixture) posterior distribution per dose group}

\item{contr}{An object of class 'optContr' as created by the getContr() function. It contains the contrast matrix to be used for the testing step.}

\item{crit_prob_adj}{A getCritProb object, specifying the critical value to be used for the testing (on the probability scale)}
}
\value{
Bayesian MCP test result, with information about p-values for the individual dose-response shapes and overall significance
}
\description{
Performs Bayesian MCP Test step, as described in Fleischer et al. (2022).
Tests for a dose-response effect using a model-based multiple contrast test based on the (provided) posterior distribution. In particular for every dose-response candidate the posterior probability is calculated that the contrast is bigger than 0 (based on the posterior distribution of the dose groups).
In order to obtain significant test decision we consider the maximum of the posterior probabilities across the different models. This maximum is compared with a (multiplicity adjusted) critical value (on the probability scale).
}
\examples{
mods <- DoseFinding::Mods(linear      = NULL,
                          emax        = c(0.5, 1.2),
                          exponential = 2,
                          doses       = c(0, 0.5, 2,4, 8))
dose_levels  <- c(0, 0.5, 2, 4, 8)
sd_posterior <- c(2.8,3,2.5,3.5,4)
contr_mat <- getContr(
  mods          = mods,
  dose_levels   = dose_levels,
  cov_posterior = diag(sd_posterior)^2)
critVal <- getCritProb(
  mods           = mods,
  dose_weights   = c(50, 50, 50, 50, 50), #reflecting the planned sample size
  dose_levels    = dose_levels,
  alpha_crit_val = 0.05)
prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 5), sigma = 2),
                   DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
                   DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
                   DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2) ,
                   DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2))
mu <- c(0, 1, 1.5, 2, 2.5)
S_hat <- diag(c(5, 4, 6, 7, 8)^2)
posterior_list <- getPosterior(
  prior_list = prior_list,
  mu_hat     = mu,
  S_hat      = S_hat,
  calc_ess   = TRUE)

performBayesianMCP(posterior_list = posterior_list,
                   contr          = contr_mat,
                   crit_prob_adj  = critVal)

}
\references{
Fleischer F, Bossert S, Deng Q, Loley C, Gierse J. 2022. Bayesian MCPMod. Pharmaceutical Statistics. 21(3): 654-670. doi:10.1002/pst.2193
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.modelFits}
\alias{plot.modelFits}
\title{plot.modelFits}
\usage{
\method{plot}{modelFits}(
  x,
  probability_scale = attr(x, "probability_scale"),
  gAIC = TRUE,
  cr_intv = TRUE,
  alpha_CrI = 0.05,
  cr_bands = FALSE,
  alpha_CrB = c(0.05, 0.2, 1),
  n_bs_smpl = 1000,
  acc_color = "orange",
  plot_res = 100,
  plot_diffs = FALSE,
  ...
)
}
\arguments{
\item{x}{An object of type \code{modelFits}}

\item{probability_scale}{A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform the output from the logit scale to the probability scale, which can be desirable for a binary outcome. Default \code{attr(x, "probability_scale")}.}

\item{gAIC}{Logical value indicating whether gAIC values are shown in the plot. Default TRUE}

\item{cr_intv}{Logical value indicating whether credible intervals are included in the plot. Default TRUE}

\item{alpha_CrI}{Numerical value of the width of the credible intervals. Default is set to 0.05 (i.e 95\% CI are shown).}

\item{cr_bands}{Logical value indicating whether bootstrapped based credible bands are shown in the plot. Default FALSE}

\item{alpha_CrB}{Numerical vector of the width of the credible bands. Default is set to \code{c(0.05, 0.2, 1)}, i.e, the 95\% CB, 80\% CB and bootstrapped median are shown.}

\item{n_bs_smpl}{Number of bootstrap samples being used. Default 1000.}

\item{acc_color}{Color of the credible bands. Default \code{"orange"}.}

\item{plot_res}{Number of plotted doses within the range of the dose levels, i.e., the resolution of the plot. Default 100.}

\item{plot_diffs}{Logical flag. Plot differences to the control group. Default FALSE.}

\item{...}{optional parameter to be passed to \code{plot()}.}
}
\value{
A ggplot2 object
}
\description{
Plot function based on the ggplot2 package. Providing visualizations for each model and a average Fit.
Black lines show the fitted dose response models and an AIC based average model.
Dots indicate the posterior median and vertical lines show corresponding credible intervals (i.e. the variability of the posterior distribution of the respective dose group).
To assess the uncertainty of the model fit one can in addition visualize credible bands (default coloring as orange shaded areas).
The calculation of these bands is performed via the \code{getBootstrapQuantiles()} function.
The default setting is that these credible bands are not calculated.
}
\examples{
posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
                       DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
                       DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2) ,
                       DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2) ,
                       DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
models <- c("exponential", "linear", "emax")
dose_levels <- c(0, 1, 2, 4, 8)
model_fits <- getModelFits(models      = models,
                           posterior   = posterior_list,
                           dose_levels = dose_levels,
                           simple      = TRUE)

plot(model_fits)

# plot with credible bands
plot(model_fits,
     cr_bands          = TRUE,
     plot_diffs        = FALSE,
     probability_scale = FALSE,
     n_bs_smpl         = 1e2) # speeding up example run-time

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assessDesign.R
\name{assessDesign}
\alias{assessDesign}
\title{assessDesign}
\usage{
assessDesign(
  n_patients,
  mods,
  prior_list,
  sd = NULL,
  contr = NULL,
  dr_means = NULL,
  data_sim = NULL,
  estimates_sim = NULL,
  n_sim = 1000,
  alpha_crit_val = 0.05,
  modeling = FALSE,
  simple = TRUE,
  avg_fit = TRUE,
  reestimate = FALSE,
  delta = NULL,
  evidence_level = NULL,
  n_bs_samples = 1000,
  med_selection = c("avgFit", "bestFit"),
  probability_scale = FALSE
)
}
\arguments{
\item{n_patients}{Vector specifying the planned number of patients per dose group. A minimum of 2 patients are required in each group.}

\item{mods}{An object of class \code{Mods} as specified in the \code{DoseFinding} package.}

\item{prior_list}{A prior_list object specifying the utilized prior for the different dose groups}

\item{sd}{A positive value, specification of assumed sd. Not required if either \code{data_sim} or \code{estimates_sim} is provided. Also not required in case of binary endpoint. Default NULL}

\item{contr}{An object of class \code{optContr} as created by the \code{DoseFinding::getContr} function. Allows specification of a fixed contrasts matrix. Default NULL.}

\item{dr_means}{A vector, allows specification of individual (not model based) assumed effects per dose group. Default NULL.}

\item{data_sim}{An optional data frame for custom simulated data. Must follow the data structure as provided by \code{simulateData()}. Default NULL.}

\item{estimates_sim}{An optional named list of 1) list of vectors for the estimated means per dose group (\code{estimates_sim$mu_hats}) and 2) of list of matrices for the covariance matrices specifying the (estimated) variabilities (\code{estimates_sim$S_hats}). Dimensions of entries must match the number of dose levels. Default NULL.}

\item{n_sim}{Number of simulations to be performed}

\item{alpha_crit_val}{(Un-adjusted) Critical value to be used for the MCP testing step. Passed to the \code{getCritProb} function for the calculation of adjusted critical values (on the probability scale). Default 0.05.}

\item{modeling}{Boolean variable defining whether the Mod part of Bayesian MCP-Mod will be performed in the assessment. More heavy on resources. Default FALSE.}

\item{simple}{Boolean variable defining whether simplified fit will be applied, see \code{?getModelFits}. Set automatically to TRUE if argument \code{delta} is provided. Passed to \code{getModelFits} Default TRUE.}

\item{avg_fit}{Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.}

\item{reestimate}{Boolean variable defining whether critical value should be calculated with re-estimated contrasts (see \code{getCritProb} function for more details). Default FALSE.}

\item{delta}{A numeric value for the threshold Delta for the MED assessment. If NULL, no MED assessment is performed. Default NULL.}

\item{evidence_level}{A numeric value between 0 and 1 for the evidence level gamma for the MED assessment. Only required for Bayesian MED assessment, see \code{?getMED} for details. If NULL, MED assessment will be performed on the fitted model according to the argument \code{med_selection}.  Default NULL.}

\item{n_bs_samples}{Number of bootstrap samples for the MED assessment if \code{evidence_level} is provided. Note that more extreme quantiles (i.e., quantiles closer to 0 or 1) tend to require more bootstrap samples to maintain precision. Default 1000.}

\item{med_selection}{A string, either \code{"avgFit"} or \code{"bestFit"}, for the method of MED selection. Default \code{"avgFit"}.}

\item{probability_scale}{A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform calculations from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.}
}
\value{
Returns success probabilities for the different assumed dose-response shapes, attributes also includes information around average success rate (across all assumed models) and prior Effective sample size.
}
\description{
This function performs simulation based trial design evaluations for a set of specified dose-response models
}
\details{
For binary endpoints, if separation occurs (for example, when one treatment arm has only responders or only non‑responders), we use penalized logistic regression with Firth’s correction to prevent convergence issues and obtain more stable estimates.
See the references for additional details.
}
\examples{
n_patients  <- c(40, 60, 60, 60, 60)
dose_levels <- c(0, 0.5, 2, 4, 8)
 
mods <- DoseFinding::Mods(linear      = NULL,
                          emax        = c(0.5, 1.2),
                          exponential = 2,
                          betaMod     = c(1, 1),
                          doses       = dose_levels,
                          maxEff      = 6)
                          
sd <- 12
prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 12), sigma = 2),
                   DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
                   DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
                   DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2) ,
                   DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2))

success_probabilities <- assessDesign(
  n_patients  = n_patients,
  mods        = mods,
  prior_list  = prior_list,
  sd          = sd,
  n_sim       = 1e2) # speed up example run time

success_probabilities

## Analysis with custom data
data_sim <- simulateData(
  n_patients        = n_patients,
  dose_levels       = dose_levels,
  sd                = sd,
  mods              = mods,
  n_sim             = 10)

success_probabilities_cd <- assessDesign(
  n_patients  = n_patients,
  mods        = mods,
  prior_list  = prior_list,
  data_sim    = data_sim,
  sd          = sd,
  n_sim       = 1e2) # speed up example run time

success_probabilities_cd

## Analysis with custom dose response relationship
custom_dr_means <- c(1, 2, 3, 4, 5)

success_probs_custom_dr <- assessDesign(
  n_patients  = n_patients,
  mods        = mods,
  prior_list  = prior_list,
  dr_means    = custom_dr_means,
  sd          = sd,
  n_sim       = 1e2) # speed up example run time

success_probs_custom_dr

## Analysis with custom estimates for means and variabilies
## No simulated data, only simulated model estimates
estimates_sim <- list(mu_hats = replicate(1e2, list(c(1, 2, 3, 4, 5) + rnorm(5, 0, 1))),
                      S_hats  = list(diag(1, 5)))

success_probs_custom_est <- assessDesign(
  n_patients    = n_patients,
  mods          = mods,
  prior_list    = prior_list,
  estimates_sim = estimates_sim)

success_probs_custom_est

if (interactive()) { # takes typically > 5 seconds

# with MED estimation without bootstrapping
# see ?getMED for details

success_probabilities <- assessDesign(
  n_patients     = n_patients,
  mods           = mods,
  prior_list     = prior_list,
  sd             = sd,
  modeling       = TRUE,
  n_sim          = 10, # speed up example run time
  delta          = 3)

  success_probabilities

# with MED estimation with bootstrapping

success_probabilities <- assessDesign(
  n_patients     = n_patients,
  mods           = mods,
  prior_list     = prior_list,
  sd             = sd,
  modeling       = TRUE,
  n_sim          = 10, # speed up example run time
  delta          = 3,
  evidence_level = 0.8)

  success_probabilities

}

}
\references{
Heinze, G. & Schemper, M. (2002). A solution to the problem of separation
in logistic regression. \emph{Statistics in Medicine}, 21(16), 2409–2419.

Liu et al. (2022). Commentary: analyzing binary data using MCPMod when zero counts are expected.
\emph{arXiv}, 2202.08781, https://arxiv.org/abs/2202.08781
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BMCPMod.R
\name{getCritProb}
\alias{getCritProb}
\title{getCritProb}
\usage{
getCritProb(
  mods,
  dose_levels,
  dose_weights = NULL,
  cov_new_trial = NULL,
  alpha_crit_val = 0.025
)
}
\arguments{
\item{mods}{An object of class "Mods" as specified in the DoseFinding package.}

\item{dose_levels}{Vector containing the different dosage levels.}

\item{dose_weights}{Vector specifying weights for the different doses, only required for Option i). Default NULL}

\item{cov_new_trial}{A covariance matrix, only required for Option ii). Default NULL}

\item{alpha_crit_val}{Significance level. Default set to 0.025.}
}
\value{
Multiplicity adjusted critical value on the probability scale.
}
\description{
This function calculates multiplicity adjusted critical values. The critical values are calculated in such a way that
when using non-informative priors the actual error level for falsely declaring a significant trial in the Bayesian MCPMod is controlled (by the specified alpha level).
Hereby optimal contrasts of the frequentist MCPMod are applied and two options can be distinguished
\enumerate{
\item Frequentist approach: If only dose_weights are provided optimal contrast vectors are calculated from the
regular MCPMod for these specific weights and the corresponding critical value for this set of contrasts is calculated via the critVal() function of the DoseFinding package.
\item Frequentist approach + re-estimation: If only a cov_new_trial (i.e. the covariance matrix of a new trial) is provided, optimal contrast vectors are calculated from the
regular MCPMod for this specific matrix. Here as well the critical value for this set of contrasts is calculated via the critVal() function of the DoseFinding package.
}
}
\examples{
mods <- DoseFinding::Mods(linear      = NULL,
                          emax        = c(0.5, 1.2),
                          exponential = 2,
                          doses       = c(0, 0.5, 2,4, 8))
dose_levels <- c(0, 0.5, 2, 4, 8)
critVal <- getCritProb(
  mods           = mods,
  dose_weights   = c(50,50,50,50,50), #reflecting the planned sample size
  dose_levels    = dose_levels,
  alpha_crit_val = 0.05)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrapping.R
\name{getBootstrapSamples}
\alias{getBootstrapSamples}
\title{getBootstrapSamples}
\usage{
getBootstrapSamples(
  model_fits,
  n_samples = 1000,
  doses = NULL,
  probability_scale = attr(model_fits, "probability_scale")
)
}
\arguments{
\item{model_fits}{An object of class modelFits, i.e. information about fitted models & corresponding model coefficients as well as the posterior distribution that was the basis for the model fitting}

\item{n_samples}{Number of samples that should be drawn}

\item{doses}{A vector of doses for which a prediction should be performed}

\item{probability_scale}{A boolean variable to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform predictions from the logit scale to the probability scale, which can be desirable for a binary outcome. Default \code{attr(model_fits, "probability_scale")}.}
}
\value{
A tibble with columns for sample_id, model, dose, sample, and sample_diff
}
\description{
A function to return bootstrap samples from the fitted dose-response models.
Samples from the posterior distribution are drawn (via the RBesT function rmix()) and for every sample the simplified fitting step (see getModelFits() function) and a prediction is performed.
These samples are returned by this function.
This approach can be considered as the Bayesian equivalent of the frequentist bootstrap approach described in O'Quigley et al. (2017).
Instead of drawing n bootstrap samples from the sampling distribution of the trial dose-response estimates, here the samples are directly taken from the posterior distribution.
}
\examples{
posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
                       DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
                       DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2) ,  
                       DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2) ,
                       DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
models         <- c("exponential", "linear")
dose_levels    <- c(0, 1, 2, 4, 8)
model_fits     <- getModelFits(models      = models,
                               posterior   = posterior_list,
                               dose_levels = dose_levels,
                               simple      = TRUE)
                           
bs_samples <- getBootstrapSamples(model_fits = model_fits,
                                  n_samples  = 10, # speeding up example run time
                                  doses      = c(0, 6, 8))
                      
bs_samples
}
\references{
O'Quigley J, Iasonos A, Bornkamp B. 2017. Handbook of Methods for Designing, Monitoring, and Analyzing Dose-Finding Trials (1st ed.). Chapman and Hall/CRC. doi:10.1201/9781315151984
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modeling.R
\name{getMED}
\alias{getMED}
\title{getMED}
\usage{
getMED(
  delta,
  evidence_level = 0.5,
  dose_levels = NULL,
  model_fits = NULL,
  bs_quantiles = NULL,
  probability_scale = attr(model_fits, "probability_scale")
)
}
\arguments{
\item{delta}{A numeric value for the threshold Delta.}

\item{evidence_level}{A numeric value between 0 and 1 for the evidence level gamma. Used for the bs_quantiles-based evaluation and not used for the model_fits-based evaluation. Default 0.5.}

\item{dose_levels}{A vector of numerics containing the different dosage levels. Default NULL.}

\item{model_fits}{An object of class modelFits as created with getModelFits(). Default NULL.}

\item{bs_quantiles}{A dataframe created with getBootstrapQuantiles(). Default NULL.}

\item{probability_scale}{A boolean variable to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform predictions from the logit scale to the probability scale, which can be desirable for a binary outcome. Default \code{attr(model_fits, "probability_scale")}.}
}
\value{
A matrix with rows for MED reached, MED, and MED index in the vector of dose levels and columns for the dose-response shapes.
}
\description{
This function provides information on the minimally efficacious dose (MED).
The MED evaluation can either be based on the fitted model shapes (model_fits) or on bootstrapped quantiles (bs_quantiles).
}
\details{
The function assumes that the 1st dose group is the control dose group.

The bootstrap approach allows for an MED based on decision rules of the form
\deqn{\widehat{\text{MED}} = \text{arg min}_{d\in\{d_1, \dots, d_k\}} \left\{ \text{Pr}\left(f(d, \hat\theta) - f(d_1, \hat\theta) > \Delta\right) > \gamma \right\} .}
The model-shape approach takes the point estimate of the model into account.
}
\examples{
posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
                       DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
                       DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2) ,
                       DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2) ,
                       DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
models         <- c("exponential", "linear")
dose_levels    <- c(0, 1, 2, 4, 8)
model_fits     <- getModelFits(models      = models,
                               posterior   = posterior_list,
                               dose_levels = dose_levels,
                               simple      = TRUE)

# MED based on the model_fit:
getMED(delta = 5, model_fits = model_fits)
                               
# MED based on bootstrapped quantiles
bs_quantiles <- getBootstrapQuantiles(model_fits = model_fits,
                                      quantiles  = c(0.025, 0.2, 0.5, 0.8),
                                      n_samples  = 100) # speeding up example run time
                                      
getMED(delta          = 5,
       evidence_level = 0.8,
       bs_quantiles   = bs_quantiles)
       
# MED on the probability scale
getMED(delta = 0.1, model_fits = model_fits, probability_scale = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posterior.R
\name{getESS}
\alias{getESS}
\title{getESS}
\usage{
getESS(post_list, method = c("elir", "moment", "morita"), n_digits = 1, ...)
}
\arguments{
\item{post_list}{A posterior list object, for which the effective sample size for each dose group should be calculated}

\item{method}{A string specifying the method of ESS calculation, see \code{?RBesT::ess()}.}

\item{n_digits}{An integer for the number of digits the result should be rounded to.}

\item{...}{Optional arguments applicable to specific methods, see \code{?RBesT::ess()}.}
}
\value{
A vector of the effective sample sizes for each dose group
}
\description{
This function calculates the effective sample size for every dose group via \code{RBesT::ess()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/s3methods.R
\name{predict.modelFits}
\alias{predict.modelFits}
\title{predict.modelFits}
\usage{
\method{predict}{modelFits}(
  object,
  doses = NULL,
  probability_scale = attr(object, "probability_scale"),
  ...
)
}
\arguments{
\item{object}{A modelFits object containing information about the fitted
model coefficients}

\item{doses}{A vector specifying the doses for which a prediction should be
done}

\item{probability_scale}{A boolean variable to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform predictions from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.}

\item{...}{Currently without function}
}
\value{
a list with the model predictions for the specified models and doses
}
\description{
This function performs model predictions based on the provided
model and dose specifications
}
\examples{
posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
                       DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
                       DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2) ,
                       DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2) ,
                       DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
models         <- c("emax", "exponential", "sigEmax", "linear", "betaMod")
dose_levels    <- c(0, 1, 2, 4, 8)
fit            <- getModelFits(models      = models,
                               posterior   = posterior_list,
                               dose_levels = dose_levels)

predict(fit, doses = c(0, 1, 3, 4, 6, 8))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BMCPMod.R
\name{getContr}
\alias{getContr}
\title{getContr}
\usage{
getContr(
  mods,
  dose_levels,
  dose_weights = NULL,
  prior_list = NULL,
  cov_posterior = NULL,
  cov_new_trial = NULL
)
}
\arguments{
\item{mods}{An object of class 'Mods' as created by the function 'DoseFinding::Mods()'}

\item{dose_levels}{Vector containing the different dosage levels.}

\item{dose_weights}{Vector specifying weights for the different doses. Please note that in case this information is provided together with a prior (i.e. Option 1) is planned these two inputs should be provided on the same scale (e.g. patient numbers).  Default NULL}

\item{prior_list}{A list of objects of class 'normMix' as created with 'RBesT::mixnorm()'. Only required as input for Option 1. Default NULL}

\item{cov_posterior}{A covariance matrix with information about the variability of the posterior distribution, only required for Option 3. Default NULL}

\item{cov_new_trial}{A covariance matrix with information about the observed variability, only required for Option 4. Default NULL}
}
\value{
An object of class 'optContr' as provided by the function 'DoseFinding::optContr()'.
}
\description{
This function calculates contrast vectors that are optimal for detecting certain alternatives via applying the function optContr() of the DoseFinding package.
Hereby, 4 different options can be distinguished that are automatically executed based on the input that is provided
\enumerate{
\item Bayesian approach: If dose_weights and a prior_list are provided an optimized contrasts for the posterior sample size is calculated.
In detail,  in a first step the dose_weights (typically the number of patients per dose group) and the prior information is combined by calculating for
each dose group a posterior effective sample. Based on this posterior effective sample sizes the allocation ratio is derived, which allows for a calculation on
pseudo-optimal contrasts via regular MCPMod are calculated from the
regular MCPMod for these specific weights
\item Frequentist approach: If only dose_weights are provided optimal contrast vectors are calculated from the
regular MCPMod for these specific weights
\item Bayesian approach + re-estimation: If only a cov_posterior (i.e. variability of the posterior distribution) is provided, pseudo-optimal contrasts based on these posterior weights will be calculated
\item Frequentist approach+re-estimation: If only a cov_new_trial (i.e. the estimated variability of a new trial) is provided, optimal contrast vectors are calculated from the
regular MCPMod for this specific covariance matrix.
}
}
\examples{
dose_levels  <- c(0, 0.5, 2, 4, 8)
mods <- DoseFinding::Mods(
  linear      = NULL,
  emax        = c(0.5, 1.2),
  exponential = 2,
  doses       = dose_levels,
  maxEff      = 6)
cov_posterior <- diag(c(2.8, 3, 2.5, 3.5, 4)^2)

contr_mat <- getContr(
  mods         = mods,
  dose_levels  = dose_levels,
  cov_posterior = cov_posterior)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation.R
\name{simulateData}
\alias{simulateData}
\title{simulateData}
\usage{
simulateData(
  n_patients,
  dose_levels,
  sd = NULL,
  mods = NULL,
  n_sim = 1000,
  true_model = NULL,
  dr_means = NULL,
  probability_scale = FALSE
)
}
\arguments{
\item{n_patients}{Vector containing number of patients as a numerical
value per dose-group.}

\item{dose_levels}{Vector containing the different dosage levels.}

\item{sd}{Standard deviation on patient level. Can be NULL if \code{probability_scale} is TRUE. Default NULL.}

\item{mods}{An object of class "Mods" as specified in the DoseFinding package. Can be NULL if ´dr_means´ is not NULL. Default NULL.}

\item{n_sim}{Number of simulations to be performed,
Default is 1000}

\item{true_model}{A character for model name, e.g. "emax". Assumed true underlying model.
If NULL, all dose-response models included in the mods input parameter will be used. Default NULL.}

\item{dr_means}{an optional vector, with information about assumed effects per dose group. Default NULL.}

\item{probability_scale}{A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform calculations from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.}
}
\value{
A list object, containing patient level simulated data for all assumed true models.
Also providing information about simulation iteration, patient number as well as dosage levels.
}
\description{
Function to simulate patient level data for a normally distributed endpoint
}
\examples{
models <- DoseFinding::Mods(linear      = NULL,
                            linlog      = NULL,
                            emax        = c(0.5, 1.2),
                            exponential = 2, 
                            doses       = c(0, 0.5, 2,4, 8),
                            maxEff      = 6)
dose_levels <- c(0, 0.5, 2, 4, 8)
sd          <- 12
n_patients  <- c(40, 60, 60, 60, 60)

sim_data <- simulateData(n_patients  = n_patients,
                         dose_levels = dose_levels,
                         sd          = sd,
                         mods        = models)

head(sim_data)

# custom response "model" shape
custom_dose_response <- c(1, 2, 3, 4, 5)
sim_data_custom_dr   <- simulateData(n_patients  = n_patients,
                                     dose_levels = dose_levels,
                                     sd          = sd,
                                     dr_means    = custom_dose_response)

head(sim_data_custom_dr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/posterior.R
\name{getPosterior}
\alias{getPosterior}
\title{getPosterior}
\usage{
getPosterior(
  prior_list,
  data = NULL,
  mu_hat = NULL,
  S_hat = NULL,
  calc_ess = FALSE,
  probability_scale = attr(data, "probability_scale")
)
}
\arguments{
\item{prior_list}{a prior list with information about the prior to be used for every dose group}

\item{data}{dataframe containing the information of dose and response. Also a simulateData object can be provided. Default NULL.}

\item{mu_hat}{vector of estimated mean values (per dose group). Default NULL.}

\item{S_hat}{covariance matrix specifying the (estimated) variability.
The variance-covariance matrix should be provided and the dimension of the matrix needs to match the number of dose groups. Default NULL.}

\item{calc_ess}{boolean variable, indicating whether effective sample size should be calculated. Default FALSE.}

\item{probability_scale}{A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform calculations from the logit scale to the probability scale, which can be desirable for a binary outcome. Default \code{attr(data, "probability_scale")}.}
}
\value{
posterior_list, a posterior list object is returned with information about (mixture) posterior distribution per dose group (more detailed information about the conjugate posterior in case of covariance input for S_hat is provided in the attributes)
}
\description{
Either the patient level data or both mu_hat as well as S_hat must to be provided.
If patient level data is provided mu_hat and S_hat are calculated within the function using a linear model.
This function calculates the posterior distribution. Depending on the input for S_hat this step is either performed for every dose group independently via the RBesT function postmix() or the mvpostmix() function of the DoseFinding package is utilized.
In the latter case conjugate posterior mixture of multivariate normals are calculated (DeGroot 1970, Bernardo and Smith 1994)
}
\details{
Kindly note that one can sample from the \code{posterior_list} with \code{lapply(posterior_list, RBesT::rmix, n = 10)}.

For binary endpoints, if separation occurs (for example, when one treatment arm has only responders or only non‑responders), we use penalized logistic regression with Firth’s correction to prevent convergence issues and obtain more stable estimates.
See the references for additional details.
}
\examples{
prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 5), sigma = 2),
                   DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
                   DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
                   DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2) ,
                   DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2))
                   
mu_hat <- c(0, 1, 1.5, 2, 2.5)
S_hat  <- diag(c(5, 4, 6, 7, 8)^2)

posterior_list <- getPosterior(
   prior_list = prior_list,
   mu_hat     = mu_hat,
   S_hat      = S_hat)

summary(posterior_list)

}
\references{
BERNARDO, Jl. M., and Smith, AFM (1994). Bayesian Theory. 81.

Heinze, G. & Schemper, M. (2002). A solution to the problem of separation
in logistic regression. \emph{Statistics in Medicine}, 21(16), 2409–2419.

Liu et al. (2022). Commentary: analyzing binary data using MCPMod when zero counts are expected.
\emph{arXiv}, 2202.08781, https://arxiv.org/abs/2202.08781
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BMCPMod.R
\name{performBayesianMCPMod}
\alias{performBayesianMCPMod}
\title{performBayesianMCPMod}
\usage{
performBayesianMCPMod(
  posterior_list,
  contr,
  crit_prob_adj,
  simple = FALSE,
  avg_fit = TRUE,
  delta = NULL,
  evidence_level = NULL,
  med_selection = c("avgFit", "bestFit"),
  n_samples = 1000,
  probability_scale = FALSE
)
}
\arguments{
\item{posterior_list}{An object of class 'postList' or a list of 'postList' objects as created by getPosterior() containing information about the (mixture) posterior distribution per dose group}

\item{contr}{An object of class 'optContr' as created by the getContr() function. It contains the contrast matrix to be used for the testing step.}

\item{crit_prob_adj}{A getCritProb object, specifying the critical value to be used for the testing (on the probability scale).}

\item{simple}{Boolean variable, defining whether simplified fit will be applied. Passed to the getModelFits() function. Default FALSE.}

\item{avg_fit}{Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.}

\item{delta}{A numeric value for the threshold Delta for the MED assessment. If NULL, no MED assessment is performed. Default NULL.}

\item{evidence_level}{A numeric value between 0 and 1 for the evidence level gamma for the MED assessment. Only required for Bayesian MED assessment, see ?getMED for details. Default NULL.}

\item{med_selection}{A string, either "avgFit" or "bestFit" based on the lowest gAIC, for the method of MED selection. Default "avgFit".}

\item{n_samples}{A numerical for the number of bootstrapped samples in case the Bayesian MED assessment is performed. Default 1e3.}

\item{probability_scale}{A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform calculations from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.}
}
\value{
Bayesian MCP test result as well as modeling result.
}
\description{
Performs Bayesian MCP Test step and modeling in a combined fashion. See performBayesianMCP() function for MCP Test step and getModelFits() for the modeling step
}
\examples{
mods <- DoseFinding::Mods(linear      = NULL,
                          emax        = c(0.5, 1.2),
                          exponential = 2,
                          doses       = c(0, 0.5, 2,4, 8))
dose_levels  <- c(0, 0.5, 2, 4, 8)
sd_posterior <- c(2.8, 3, 2.5, 3.5, 4)
contr_mat <- getContr(
  mods          = mods,
  dose_levels   = dose_levels,
  cov_posterior = diag(sd_posterior)^2)
critVal <- getCritProb(
  mods           = mods,
  dose_weights   = c(50, 50, 50, 50, 50), #reflecting the planned sample size
  dose_levels    = dose_levels,
  alpha_crit_val = 0.6) # unreasonable alpha chosen for this example, rather choose 0.05
prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 5), sigma = 2),
                   DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
                   DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
                   DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2) ,
                   DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2))
mu <- c(0, 1, 1.5, 2, 2.5)
S_hat <- diag(c(5, 4, 6, 7, 8)^2)
posterior_list <- getPosterior(
  prior_list = prior_list,
  mu_hat     = mu,
  S_hat      = S_hat,
  calc_ess   = TRUE)

performBayesianMCPMod(posterior_list = posterior_list,
                      contr          = contr_mat,
                      crit_prob_adj  = critVal,
                      simple         = FALSE,
                      delta          = 1.1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modeling.R
\name{getModelFits}
\alias{getModelFits}
\title{getModelFits}
\usage{
getModelFits(
  models,
  dose_levels,
  posterior,
  avg_fit = TRUE,
  simple = FALSE,
  probability_scale = FALSE
)
}
\arguments{
\item{models}{A Mods object as created with \code{DoseFinding::Mods()} or a vector
of model names for which a fit will be performed.
Implemented model shapes are \code{"linear"}, \code{"exponential"}, \code{"logistic"},
\code{"emax"}, \code{"sigEmax"}, \code{"quadratic"}, and \code{"betaMod"}.}

\item{dose_levels}{A vector containing the different dosage levels.}

\item{posterior}{A getPosterior object, containing the (multivariate) posterior distribution per dosage level.}

\item{avg_fit}{Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.}

\item{simple}{Boolean variable, defining whether simplified fit will be applied. Default FALSE.}

\item{probability_scale}{A boolean variable to specify if the predicted dose-response should be on the logit scale or the probability scale. Setting to TRUE will transform predictions from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.}
}
\value{
An object of class modelFits. A list containing information about the fitted model coefficients, the prediction per dose group as well as maximum effect and generalized AIC (and corresponding weight) per model.
}
\description{
Fits dose-response curves for the specified dose-response models, based on the posterior distributions.
For the simplified fit, multivariate normal distributions will be approximated and reduced by one-dimensional normal distributions.
For the default case, the Nelder-Mead algorithm is used.
In detail, for both approaches the mean vector \eqn{\theta^{Y}} and the covariance \eqn{\Sigma} of the (mixture) posterior distributions and the corresponding posterior weights \eqn{\tilde{\omega}_{l}} for \eqn{l \in {1,...,L}} are used as basis
For the full fit a GLS estimator is used to minimize the following expression for the respective dose-response models \eqn{m}
\deqn{ \hat{\theta}_{m}=\text{arg min}_{\theta_{m}} \sum_{l=1}^{L} \tilde{\omega}_{l}(\theta_{l_{i}}^{Y}-f(dose_{i},\hat{\theta}_{m}))'\Sigma_{l}^{-1}(\theta_{l_{i}}^{Y}-f(dose_{i},\hat{\theta}_{m}))}
Therefore the function nloptr of the nloptr package is utilized.
In the simplified case \eqn{L=1}, as the dimension of the posterior is reduced to 1 first.
The generalized AIC values are calculated via the formula
\deqn{gAIC_{m} = \sum_{l=1}^{L} \tilde{\omega}_{l} \sum_{i=0}^{K} \frac{1}{\Sigma_{l_{i,i}}} (\theta_{l_i}^Y - f(dose_{i},\hat{\theta}_{m}))^2 + 2p }
where \eqn{p} denotes the number of estimated parameters and \eqn{K} the number of active dose levels.
Here as well for the simplified case the formula reduces to one summand as \eqn{L=1}.
Corresponding gAIC based weights for model \eqn{M} are calculated as outlined in Schorning et al. (2016)
\deqn{
\Omega_I (M) = \frac{\exp(-0.5 gAIC_{M})}{\sum_{m=1}^{Q} \exp(-0.5 gAIC_{m})}
}
where \eqn{Q} denotes the number of models included in the averaging procedure.
}
\examples{
posterior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 1), sigma = 2),
                       DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 3, s = 1.2), sigma = 2),
                       DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 4, s = 1.5), sigma = 2),
                       DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 6, s = 1.2), sigma = 2),
                       DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 6.5, s = 1.1), sigma = 2))
models         <- c("emax", "exponential", "sigEmax", "linear")
dose_levels    <- c(0, 1, 2, 4, 8)

fit        <- getModelFits(models      = models,
                           posterior   = posterior_list,
                           dose_levels = dose_levels)
                           
fit
                           
fit_simple <- getModelFits(models      = models,
                           posterior   = posterior_list,
                           dose_levels = dose_levels,
                           simple      = TRUE)
                           
fit_simple

}
\references{
Schorning K, Bornkamp B, Bretz F, Dette H. 2016. Model selection versus model averaging in dose finding studies. Stat Med; 35; 4021-4040.
}
